@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2009-2017, Intel Corporation                                **
+** Copyright (c) 2009-2018, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -34,13 +34,20 @@
 #include "libxsmm_macros.h"
 
 
-/** Returns the current clock tick of a monotonic timer source; uses a platform-specific resolution. */
-LIBXSMM_API unsigned long long libxsmm_timer_tick(void);
+typedef unsigned long long libxsmm_timer_tickint;
 
-/** Returns the current tick of a (monotonic) platform-specific counter; not necessarily CPU cycles. */
-LIBXSMM_API unsigned long long libxsmm_timer_xtick(void);
+/**
+ * Returns the current clock tick of a monotonic timer source with
+ * platform-specific resolution (not necessarily CPU cycles).
+ */
+LIBXSMM_API libxsmm_timer_tickint libxsmm_timer_tick(void);
+
+/** Helper function to receive the difference between two timer ticks; avoids potential side-effects of LIBXSMM_DIFF. */
+LIBXSMM_API_INLINE libxsmm_timer_tickint libxsmm_timer_diff(libxsmm_timer_tickint tick0, libxsmm_timer_tickint tick1) {
+  return LIBXSMM_DIFF(tick0, tick1);
+}
 
 /** Returns the duration (in seconds) between two values received by libxsmm_timer_tick. */
-LIBXSMM_API double libxsmm_timer_duration(unsigned long long tick0, unsigned long long tick1);
+LIBXSMM_API double libxsmm_timer_duration(libxsmm_timer_tickint tick0, libxsmm_timer_tickint tick1);
 
 #endif /*LIBXSMM_TIMER_H*/
