@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2017, Intel Corporation                                **
+** Copyright (c) 2015-2018, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -43,7 +43,7 @@
 #include <cassert>
 #include <cstdio>
 #include <cmath>
-#if defined(__MKL) || defined(MKL_DIRECT_CALL_SEQ) || defined(MKL_DIRECT_CALL)
+#if defined(__MKL)
 # include <mkl_service.h>
 #endif
 #if defined(_OPENMP)
@@ -58,7 +58,7 @@
 #endif
 
 #if !defined(MAX_SIZE)
-# define MAX_SIZE (LIBXSMM_MAX_MNK / LIBXSMM_MAX_K)
+# define MAX_SIZE ((LIBXSMM_MAX_M) * (LIBXSMM_MAX_N))
 #endif
 
 /** >1: number of locks, =1: omp critical, =0: atomic */
@@ -73,10 +73,6 @@
 #else
 # define CP2K_SCHEDULE
 #endif
-// enable result validation
-// sequential (1), parallel (2)
-// (2) enables proper warmup
-#define CP2K_CHECK 2
 
 
 #if defined(_OPENMP) && defined(CP2K_SYNCHRONIZATION) && (1 < (CP2K_SYNCHRONIZATION))
@@ -90,10 +86,10 @@ public:
   }
 public:
   void acquire(const void* address) {
-    omp_set_lock(m_lock + LIBXSMM_HASH2(address, LIBXSMM_ALIGNMENT, CP2K_SYNCHRONIZATION));
+    omp_set_lock(m_lock + LIBXSMM_FOLD2(address, LIBXSMM_ALIGNMENT, CP2K_SYNCHRONIZATION));
   }
   void release(const void* address) {
-    omp_unset_lock(m_lock + LIBXSMM_HASH2(address, LIBXSMM_ALIGNMENT, CP2K_SYNCHRONIZATION));
+    omp_unset_lock(m_lock + LIBXSMM_FOLD2(address, LIBXSMM_ALIGNMENT, CP2K_SYNCHRONIZATION));
   }
 private:
   omp_lock_t m_lock[CP2K_SYNCHRONIZATION];
@@ -101,14 +97,11 @@ private:
 #endif
 
 
-LIBXSMM_INLINE LIBXSMM_RETARGETABLE void init(int seed, REAL_TYPE *LIBXSMM_RESTRICT dst,
+LIBXSMM_INLINE LIBXSMM_RETARGETABLE void init(libxsmm_blasint seed, REAL_TYPE *LIBXSMM_RESTRICT dst,
   libxsmm_blasint nrows, libxsmm_blasint ncols, libxsmm_blasint ld, double scale)
 {
   const double seed1 = scale * (seed + 1);
   libxsmm_blasint i;
-#if defined(_OPENMP)
-# pragma omp parallel for private(i)
-#endif
   for (i = 0; i < ncols; ++i) {
     libxsmm_blasint j = 0;
     for (; j < nrows; ++j) {
@@ -125,9 +118,9 @@ LIBXSMM_INLINE LIBXSMM_RETARGETABLE void init(int seed, REAL_TYPE *LIBXSMM_RESTR
 
 template<typename T>
 LIBXSMM_INLINE LIBXSMM_RETARGETABLE
-void add(T *LIBXSMM_RESTRICT dst, const T *LIBXSMM_RESTRICT src, int nrows, int ncols, int ld_src = 0)
+void add(T *LIBXSMM_RESTRICT dst, const T *LIBXSMM_RESTRICT src, libxsmm_blasint nrows, libxsmm_blasint ncols, libxsmm_blasint ld_src = 0)
 {
-  const int ld = 0 == ld_src ? ncols : ld_src;
+  const libxsmm_blasint ld = 0 == ld_src ? ncols : ld_src;
 #if defined(_OPENMP) && defined(CP2K_SYNCHRONIZATION) && (0 < (CP2K_SYNCHRONIZATION))
 # if (1 == (CP2K_SYNCHRONIZATION))
 # pragma omp critical(smmadd)
@@ -136,9 +129,9 @@ void add(T *LIBXSMM_RESTRICT dst, const T *LIBXSMM_RESTRICT src, int nrows, int
 # endif
 #endif
   {
-    for (int i = 0; i < nrows; ++i) {
+    for (libxsmm_blasint i = 0; i < nrows; ++i) {
       LIBXSMM_PRAGMA_UNROLL
-      for (int j = 0; j < ncols; ++j) {
+      for (libxsmm_blasint j = 0; j < ncols; ++j) {
         const T value = src[i*ld+j];
 #if defined(_OPENMP) && (!defined(CP2K_SYNCHRONIZATION) || (0 == (CP2K_SYNCHRONIZATION)))
 #       pragma omp atomic
@@ -153,66 +146,50 @@ void add(T *LIBXSMM_RESTRICT dst, const T *LIBXSMM_RESTRICT src, int nrows, int
 }
 
 
-template<typename T>
-LIBXSMM_INLINE LIBXSMM_RETARGETABLE
-double norm_l2(const T *LIBXSMM_RESTRICT expect, const T *LIBXSMM_RESTRICT result, int nrows, int ncols, int ld = 0)
-{
-  const int ldx = 0 == ld ? ncols : ld;
-  double diff = 0;
-  for (int i = 0; i < nrows; ++i) {
-    for (int j = 0; j < ncols; ++j) {
-      const int k = i * ldx + j;
-      const double a = expect[k];
-      const double b = result[k];
-      const double d1 = a - b;
-      const double d2 = d1 * d1;
-      diff = std::max(diff, d2);
-    }
-  }
-  return diff;
-}
-
-
 int main(int argc, char* argv[])
 {
   int result = EXIT_SUCCESS;
   try {
     typedef REAL_TYPE T;
-    const int m = 1 < argc ? std::atoi(argv[1]) : 23;
-    const int q = ((1ULL << 30) / (3 * m * m * sizeof(T)));
-    const int r = 2 < argc ? (0 < std::atoi(argv[2]) ? std::atoi(argv[2]) : ('+' == *argv[2]
+    const libxsmm_blasint m = 1 < argc ? std::atoi(argv[1]) : 23;
+    const libxsmm_blasint q = ((1ULL << 30) / (3 * m * m * sizeof(T)));
+    const libxsmm_blasint r = 2 < argc ? (0 < std::atoi(argv[2]) ? std::atoi(argv[2]) : ('+' == *argv[2]
       ? (q << std::strlen(argv[2])) : ('-' == *argv[2]
       ? (q >> std::strlen(argv[2])) : 0))) : 0;
-    const int t = 3 < argc ? (0 < std::atoi(argv[3]) ? std::atoi(argv[3]) : ('+' == *argv[3]
+    const libxsmm_blasint t = 3 < argc ? (0 < std::atoi(argv[3]) ? std::atoi(argv[3]) : ('+' == *argv[3]
       ? ((CP2K_MIN_NLOCAL) << std::strlen(argv[3])) : ('-' == *argv[3]
       ? ((CP2K_MIN_NLOCAL) >> std::strlen(argv[3])) : -1))) : -1;
-    const int n = 4 < argc ? std::atoi(argv[4]) : m;
-    const int k = 5 < argc ? std::atoi(argv[5]) : m;
+    const libxsmm_blasint k = 5 < argc ? std::atoi(argv[5]) : m;
+    const libxsmm_blasint n = 4 < argc ? std::atoi(argv[4]) : k;
 
-    const int csize = m * n;
+    const libxsmm_blasint csize = m * n;
     if ((MAX_SIZE) < csize) {
       throw "The size M x N is exceeding MAX_SIZE!";
     }
 
-    const int asize = m * k, bsize = k * n, aspace = LIBXSMM_ALIGNMENT / sizeof(T);
-    const int s = 0 < r ? r : ((2ULL << 30) / ((asize + bsize) * sizeof(T))); // 2 GByte
-    const int u = 0 < t ? t : static_cast<int>(std::sqrt(static_cast<double>(s) * CP2K_MIN_NLOCAL / CP2K_MIN_NPARALLEL) + 0.5);
-    const size_t bwsize = (s * (asize + bsize)/*load*/ + ((s + u - 1) / u) * csize * 2/*accumulate*/) * sizeof(T);
+    const libxsmm_blasint asize = m * k, bsize = k * n, aspace = LIBXSMM_ALIGNMENT / sizeof(T);
+    const libxsmm_blasint s = 0 < r ? r : ((2ULL << 30) / ((asize + bsize) * sizeof(T))); // 2 GByte
+    const libxsmm_blasint u = 0 < t ? t : static_cast<libxsmm_blasint>(libxsmm_isqrt_u64(s * CP2K_MIN_NLOCAL / CP2K_MIN_NPARALLEL));
+    const size_t bwsize = static_cast<size_t>((s * (asize + bsize)/*load*/ + ((s + u - 1) / u) * csize * 2/*accumulate*/) * sizeof(T));
     const double gflops = 2.0 * s * m * n * k * 1E-9, scale = 1.0 / s;
+    const char *const env_check = getenv("CHECK");
+    const double check = LIBXSMM_ABS(0 == env_check ? 0 : atof(env_check));
 
     LIBXSMM_RETARGETABLE struct LIBXSMM_RETARGETABLE raii { // avoid std::vector (first-touch init. causes NUMA issue)
       T *a, *b, *c;
-      raii(int asize_, int bsize_, int csize_): a(new T[asize_]), b(new T[bsize_]), c(new T[csize_]) {}
+      raii(libxsmm_blasint asize_, libxsmm_blasint bsize_, libxsmm_blasint csize_)
+        : a(new T[static_cast<size_t>(asize_)]), b(new T[static_cast<size_t>(bsize_)])
+        , c(new T[static_cast<size_t>(csize_)]) {}
       ~raii() { delete[] a; delete[] b; delete[] c; }
     } buffer(s * asize + aspace - 1, s * bsize + aspace - 1, csize);
-    T *const a = LIBXSMM_ALIGN2(buffer.a, LIBXSMM_ALIGNMENT);
-    T *const b = LIBXSMM_ALIGN2(buffer.b, LIBXSMM_ALIGNMENT);
+    T *const a = LIBXSMM_ALIGN(buffer.a, LIBXSMM_ALIGNMENT);
+    T *const b = LIBXSMM_ALIGN(buffer.b, LIBXSMM_ALIGNMENT);
     T * /*const*/ c = buffer.c; // no alignment, but thread-local array will be aligned
 
 #if defined(_OPENMP)
-#   pragma omp parallel for CP2K_SCHEDULE
+#   pragma omp parallel for
 #endif
-    for (int i = 0; i < s; ++i) {
+    for (libxsmm_blasint i = 0; i < s; ++i) {
       init(42 + i, a + i * asize, m, k, m, scale);
       init(24 + i, b + i * bsize, k, n, k, scale);
     }
@@ -228,41 +205,36 @@ int main(int argc, char* argv[])
       libxsmm_init();
 #if !defined(LIBXSMM_OFFLOAD_TARGET)
       // some more setup similar to CP2K/intel branch
-      libxsmm_set_gemm_auto_prefetch(LIBXSMM_X86_AVX512_MIC != libxsmm_get_target_archid() ? LIBXSMM_PREFETCH_AL2BL2_VIA_C : LIBXSMM_PREFETCH_BL2_VIA_C);
+      libxsmm_set_gemm_auto_prefetch(LIBXSMM_X86_AVX512_MIC != libxsmm_get_target_archid() ? LIBXSMM_GEMM_PREFETCH_AL2BL2_VIA_C : LIBXSMM_GEMM_PREFETCH_BL2_VIA_C);
 #endif
       //libxsmm_set_dispatch_trylock(1);
 
-      fprintf(stdout, "m=%i n=%i k=%i size=%i memory=%.1f MB (%s)\n\n", m, n, k, s,
+      fprintf(stdout, "m=%lli n=%lli k=%lli size=%lli memory=%.1f MB (%s)\n\n",
+        static_cast<long long>(m), static_cast<long long>(n), static_cast<long long>(k), static_cast<long long>(s),
         1.0 * (s * (asize + bsize) * sizeof(T)) / (1 << 20), 8 == sizeof(T) ? "DP" : "SP");
 
-      const T zero = 0;
-#if defined(CP2K_CHECK) && 0 < (CP2K_CHECK)
-      LIBXSMM_RETARGETABLE struct LIBXSMM_RETARGETABLE raii { // avoid std::vector (first-touch init. causes NUMA issue)
+      LIBXSMM_RETARGETABLE struct LIBXSMM_RETARGETABLE raii_expect { // avoid std::vector (first-touch init. causes NUMA issue)
         T *expect;
-        explicit raii(int size): expect(new T[size]) {}
-        ~raii() { delete[] expect; }
-      } expect_buffer(csize);
-      T *const expect = expect_buffer.expect;
-      std::fill_n(expect, csize, zero);
-      double diff = 0;
-#else
-      T *const expect = c;
-#endif
+        explicit raii_expect(libxsmm_blasint size): expect(0 < size ? new T[static_cast<size_t>(size)] : 0) {}
+        ~raii_expect() { delete[] expect; }
+      } expect_buffer(LIBXSMM_FEQ(0, check) ? 0 : csize);
+      T *const expect = (0 == expect_buffer.expect ? c : expect_buffer.expect);
+      libxsmm_matdiff_info d, diff;
+      const T zero = 0;
+
       // eventually JIT-compile the requested kernel
       const libxsmm_mmfunction<T> xmm(LIBXSMM_FLAGS, m, n, k, LIBXSMM_PREFETCH);
 
+      memset(&diff, 0, sizeof(diff));
       { // LAPACK/BLAS3 (warmup BLAS Library)
         std::fill_n(expect, csize, zero);
-#if defined(_OPENMP) && (!defined(CP2K_CHECK) || 1 < (CP2K_CHECK))
+#if defined(_OPENMP)
 #       pragma omp parallel for CP2K_SCHEDULE
 #endif
-        for (int i = 0; i < s; i += u) {
-          // make sure that stacksize is covering the problem size
-          T tls[MAX_SIZE]; // LIBXSMM_ALIGNED does not apply to non-static local stack variables
-          T *const tmp = LIBXSMM_ALIGN_LDST(tls);
+        for (libxsmm_blasint i = 0; i < s; i += u) {
+          T tmp[MAX_SIZE] = { 0 }; // make sure that stacksize is covering the problem size
           const T *ai = a + i * asize, *bi = b + i * bsize;
-          for (int j = 0; j < (MAX_SIZE); ++j) tls[j] = 0; // clear
-          for (int j = 0; j < LIBXSMM_MIN(u, s - i); ++j) {
+          for (libxsmm_blasint j = 0; j < LIBXSMM_MIN(u, s - i); ++j) {
             const T *const aij = ai + asize, *const bij = bi + bsize;
             libxsmm_blas_gemm(0/*transa*/, 0/*transb*/, m, n, k,
               0/*alpha*/, ai, 0/*lda*/, bi, 0/*ldb*/,
@@ -281,18 +253,15 @@ int main(int argc, char* argv[])
 #if defined(_OPENMP)
 #       pragma omp parallel for CP2K_SCHEDULE
 #endif
-        for (int i = 0; i < s; i += u) {
-          // make sure that stacksize is covering the problem size
-          T tls[MAX_SIZE]; // LIBXSMM_ALIGNED does not apply to non-static local stack variables
-          T *const tmp = LIBXSMM_ALIGN_LDST(tls);
+        for (libxsmm_blasint i = 0; i < s; i += u) {
+          T tmp[MAX_SIZE] = { 0 }; // make sure that stacksize is covering the problem size
           const T *ai = a + i * asize, *bi = b + i * bsize;
-          for (int j = 0; j < (MAX_SIZE); ++j) tls[j] = 0; // clear
-          for (int j = 0; j < LIBXSMM_MIN(u, s - i); ++j) {
+          for (libxsmm_blasint j = 0; j < LIBXSMM_MIN(u, s - i); ++j) {
             const T *const aij = ai + asize, *const bij = bi + bsize;
             // alternatively libxsmm_blas_gemm can be called (see above)
             LIBXSMM_BLAS_GEMM(LIBXSMM_FLAGS, m, n, k,
-              LIBXSMM_ALPHA, ai, LIBXSMM_LD(m, k), bi, LIBXSMM_LD(k, n),
-              LIBXSMM_BETA, tmp, LIBXSMM_LD(m, n));
+              LIBXSMM_ALPHA, ai, m, bi, k,
+              LIBXSMM_BETA, tmp, m);
             ai = aij;
             bi = bij;
           }
@@ -305,11 +274,10 @@ int main(int argc, char* argv[])
           fprintf(stdout, "\tcalls/s: %.0f Hz\n", s / duration);
         }
         fprintf(stdout, "\tduration: %.0f ms\n", 1000.0 * duration);
-#if defined(CP2K_CHECK) && 0 < (CP2K_CHECK)
-        const double d = norm_l2(expect, c, m, n);
-        fprintf(stdout, "\tdiff=%f\n", d);
-        diff = std::max(diff, d);
-#endif
+        if (!LIBXSMM_FEQ(0, check) && EXIT_SUCCESS == libxsmm_matdiff(LIBXSMM_DATATYPE(REAL_TYPE), m, n, expect, c, 0, 0, &d)) {
+          fprintf(stdout, "\tdiff: L2abs=%f Linfo=%f\n", d.l2_abs, d.linf_abs);
+          libxsmm_matdiff_reduce(&diff, &d);
+        }
       }
 
       { // inline an optimized implementation
@@ -319,17 +287,14 @@ int main(int argc, char* argv[])
 #if defined(_OPENMP)
 #       pragma omp parallel for CP2K_SCHEDULE
 #endif
-        for (int i = 0; i < s; i += u) {
-          // make sure that stacksize is covering the problem size
-          T tls[MAX_SIZE]; // LIBXSMM_ALIGNED does not apply to non-static local stack variables
-          T *const tmp = LIBXSMM_ALIGN_LDST(tls);
+        for (libxsmm_blasint i = 0; i < s; i += u) {
+          T tmp[MAX_SIZE] = { 0 }; // make sure that stacksize is covering the problem size
           const T *ai = a + i * asize, *bi = b + i * bsize;
-          for (int j = 0; j < (MAX_SIZE); ++j) tls[j] = 0; // clear
-          for (int j = 0; j < LIBXSMM_MIN(u, s - i); ++j) {
+          for (libxsmm_blasint j = 0; j < LIBXSMM_MIN(u, s - i); ++j) {
             const T *const aij = ai + asize, *const bij = bi + bsize;
             LIBXSMM_INLINE_GEMM(LIBXSMM_FLAGS, m, n, k,
-              LIBXSMM_ALPHA, ai, LIBXSMM_LD(m, k), bi, LIBXSMM_LD(k, n),
-              LIBXSMM_BETA, tmp, LIBXSMM_LD(m, n));
+              LIBXSMM_ALPHA, ai, m, bi, k,
+              LIBXSMM_BETA, tmp, m);
             ai = aij;
             bi = bij;
           }
@@ -342,11 +307,10 @@ int main(int argc, char* argv[])
           fprintf(stdout, "\tcalls/s: %.0f Hz\n", s / duration);
         }
         fprintf(stdout, "\tduration: %.0f ms\n", 1000.0 * duration);
-#if defined(CP2K_CHECK) && 0 < (CP2K_CHECK)
-        const double d = norm_l2(expect, c, m, n);
-        fprintf(stdout, "\tdiff=%f\n", d);
-        diff = std::max(diff, d);
-#endif
+        if (!LIBXSMM_FEQ(0, check) && EXIT_SUCCESS == libxsmm_matdiff(LIBXSMM_DATATYPE(REAL_TYPE), m, n, expect, c, 0, 0, &d)) {
+          fprintf(stdout, "\tdiff: L2abs=%f Linfo=%f\n", d.l2_abs, d.linf_abs);
+          libxsmm_matdiff_reduce(&diff, &d);
+        }
       }
 
       { // auto-dispatched
@@ -356,13 +320,10 @@ int main(int argc, char* argv[])
 #if defined(_OPENMP)
 #       pragma omp parallel for CP2K_SCHEDULE
 #endif
-        for (int i = 0; i < s; i += u) {
-          // make sure that stacksize is covering the problem size
-          T tls[MAX_SIZE]; // LIBXSMM_ALIGNED does not apply to non-static local stack variables
-          T *const tmp = LIBXSMM_ALIGN_LDST(tls);
+        for (libxsmm_blasint i = 0; i < s; i += u) {
+          T tmp[MAX_SIZE] = { 0 }; // make sure that stacksize is covering the problem size
           const T *ai = a + i * asize, *bi = b + i * bsize;
-          for (int j = 0; j < (MAX_SIZE); ++j) tls[j] = 0; // clear
-          for (int j = 0; j < LIBXSMM_MIN(u, s - i); ++j) {
+          for (libxsmm_blasint j = 0; j < LIBXSMM_MIN(u, s - i); ++j) {
             const T *const aij = ai + asize, *const bij = bi + bsize;
             libxsmm_gemm(0/*transa*/, 0/*transb*/, m, n, k,
               0/*alpha*/, ai, 0/*lda*/, bi, 0/*ldb*/,
@@ -379,11 +340,10 @@ int main(int argc, char* argv[])
           fprintf(stdout, "\tcalls/s: %.0f Hz\n", s / duration);
         }
         fprintf(stdout, "\tduration: %.0f ms\n", 1000.0 * duration);
-#if defined(CP2K_CHECK) && 0 < (CP2K_CHECK)
-        const double d = norm_l2(expect, c, m, n);
-        fprintf(stdout, "\tdiff=%f\n", d);
-        diff = std::max(diff, d);
-#endif
+        if (!LIBXSMM_FEQ(0, check) && EXIT_SUCCESS == libxsmm_matdiff(LIBXSMM_DATATYPE(REAL_TYPE), m, n, expect, c, 0, 0, &d)) {
+          fprintf(stdout, "\tdiff: L2abs=%f Linfo=%f\n", d.l2_abs, d.linf_abs);
+          libxsmm_matdiff_reduce(&diff, &d);
+        }
       }
 
       if (xmm) { // specialized routine
@@ -393,19 +353,16 @@ int main(int argc, char* argv[])
 #if defined(_OPENMP)
 #       pragma omp parallel for CP2K_SCHEDULE
 #endif
-        for (int i = 0; i < s; i += u) {
-          // make sure that stacksize is covering the problem size
-          T tls[MAX_SIZE]; // LIBXSMM_ALIGNED does not apply to non-static local stack variables
-          T *const tmp = LIBXSMM_ALIGN_LDST(tls);
+        for (libxsmm_blasint i = 0; i < s; i += u) {
+          T tmp[MAX_SIZE] = { 0 }; // make sure that stacksize is covering the problem size
           const T *ai = a + i * asize, *bi = b + i * bsize;
-          for (int j = 0; j < (MAX_SIZE); ++j) tls[j] = 0; // clear
-          for (int j = 0; j < LIBXSMM_MIN(u, s - i); ++j) {
+          for (libxsmm_blasint j = 0; j < LIBXSMM_MIN(u, s - i); ++j) {
             const T *const aij = ai + asize, *const bij = bi + bsize;
 #if (0 != LIBXSMM_PREFETCH)
             xmm(ai, bi, tmp,
-              LIBXSMM_PREFETCH_A(aij + asize),
-              LIBXSMM_PREFETCH_B(bij + bsize),
-              LIBXSMM_PREFETCH_C(tmp));
+              LIBXSMM_GEMM_PREFETCH_A(aij + asize),
+              LIBXSMM_GEMM_PREFETCH_B(bij + bsize),
+              LIBXSMM_GEMM_PREFETCH_C(tmp));
 #else
             xmm(ai, bi, tmp);
 #endif
@@ -421,20 +378,22 @@ int main(int argc, char* argv[])
           fprintf(stdout, "\tcalls/s: %.0f Hz\n", s / duration);
         }
         fprintf(stdout, "\tduration: %.0f ms\n", 1000.0 * duration);
-#if defined(CP2K_CHECK) && 0 < (CP2K_CHECK)
-        const double d = norm_l2(expect, c, m, n);
-        fprintf(stdout, "\tdiff=%f\n", d);
-        diff = std::max(diff, d);
-#endif
+        if (!LIBXSMM_FEQ(0, check) && EXIT_SUCCESS == libxsmm_matdiff(LIBXSMM_DATATYPE(REAL_TYPE), m, n, expect, c, 0, 0, &d)) {
+          fprintf(stdout, "\tdiff: L2abs=%f Linfo=%f\n", d.l2_abs, d.linf_abs);
+          libxsmm_matdiff_reduce(&diff, &d);
+        }
       }
 
       // finalize LIBXSMM
       libxsmm_finalize();
       fprintf(stdout, "Finished\n");
 
-#if defined(CP2K_CHECK) && 0 < (CP2K_CHECK)
-      if (1.0 < diff) result = EXIT_FAILURE;
-#endif
+      if (!LIBXSMM_FEQ(0, check)) {
+        if (check < 100.0 * diff.normf_rel) {
+          fprintf(stderr, "FAILED with an error of %f%%!\n", 100.0 * diff.normf_rel);
+          result = EXIT_FAILURE;
+        }
+      }
     }
   }
   catch(const std::exception& e) {
