@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2016-2017, Intel Corporation                                **
+** Copyright (c) 2016-2018, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -51,15 +51,18 @@
  */
 int main(int argc, char* argv[])
 {
-  const int size = LIBXSMM_DEFAULT(10000000, 1 < argc ? atoi(argv[1]) : 0);
-  const int nthreads = LIBXSMM_DEFAULT(1, 2 < argc ? atoi(argv[2]) : 0);
-  unsigned long long start;
-  double dcall, ddisp;
-  int i;
-
 #if defined(_OPENMP)
-  if (1 < nthreads) omp_set_num_threads(nthreads);
+  const int max_nthreads = omp_get_max_threads();
+#else
+  const int max_nthreads = 1;
 #endif
+  const int size = LIBXSMM_MAX(1 < argc ? atoi(argv[1]) : 10000/*default*/, 1);
+  const int nthreads = LIBXSMM_CLMP(2 < argc ? atoi(argv[2]) : 1/*default*/, 1, max_nthreads);
+  const int maxksize = LIBXSMM_CLMP(3 < argc ? atoi(argv[3]) : 64/*default*/, 1, LIBXSMM_MAX_M);
+  const int minksize = LIBXSMM_CLMP(4 < argc ? atoi(argv[4]) : 4/*default*/, 1, maxksize);
+  libxsmm_timer_tickint tdsp0 = 0, tdsp1 = 0, tcgen = 0, tcall;
+  const int krange = maxksize - minksize;
+  int ncgens = size;
 
   fprintf(stdout, "Dispatching %i calls %s internal synchronization using %i thread%s...\n", size,
 #if 0 != LIBXSMM_SYNC
@@ -71,7 +74,7 @@ int main(int argc, char* argv[])
     1 >= nthreads ? "" : "s");
 
 #if 0 != LIBXSMM_JIT
-  if (LIBXSMM_X86_AVX > libxsmm_get_target_archid()) {
+  if (LIBXSMM_X86_SSE3 > libxsmm_get_target_archid()) {
     fprintf(stderr, "\tWarning: JIT support is not available at runtime!\n");
   }
 #else
@@ -82,39 +85,94 @@ int main(int argc, char* argv[])
 # pragma offload target(LIBXSMM_OFFLOAD_TARGET)
 #endif
   {
-    /* first invocation may initialize some internals (libxsmm_init),
-     * or actually generate code (code gen. time is out of scope)
-     */
-    libxsmm_dmmdispatch(LIBXSMM_AVG_M, LIBXSMM_AVG_N, LIBXSMM_AVG_K,
-      NULL/*lda*/, NULL/*ldb*/, NULL/*ldc*/, NULL/*alpha*/, NULL/*beta*/,
-      NULL/*flags*/, NULL/*prefetch*/);
+    libxsmm_blasint *const r = (libxsmm_blasint*)malloc(3/*m,n,k*/ * size * sizeof(libxsmm_blasint));
+    libxsmm_registry_info reginfo;
+    int i;
+
+    assert(0 != r);
+    /* generate a set of random numbers outside of any parallel region */
+    for (i = 0; i < (3/*m,n,k*/ * size); ++i) {
+      r[i] = (1 < krange) ? ((rand() % krange) + minksize) : minksize;
+    }
 
     /* run non-inline function to measure call overhead of an "empty" function */
-    start = libxsmm_timer_tick();
-#if defined(_OPENMP)
-#   pragma omp parallel for default(none) private(i)
-#endif
+    tcall = libxsmm_timer_tick();
     for (i = 0; i < size; ++i) {
       libxsmm_init(); /* subsequent calls are not doing any work */
     }
-    dcall = libxsmm_timer_duration(start, libxsmm_timer_tick());
+    tcall = libxsmm_timer_diff(tcall, libxsmm_timer_tick());
+
+    /* first invocation may initialize some internals (libxsmm_init),
+     * or actually generate code (code gen. time is out of scope)
+     */
+    libxsmm_dmmdispatch(23, 23, 23,
+      NULL/*lda*/, NULL/*ldb*/, NULL/*ldc*/, NULL/*alpha*/, NULL/*beta*/,
+      NULL/*flags*/, NULL/*prefetch*/);
 
-    start = libxsmm_timer_tick();
 #if defined(_OPENMP)
-#   pragma omp parallel for default(none) private(i)
+#   pragma omp parallel for num_threads(nthreads) private(i)
 #endif
     for (i = 0; i < size; ++i) {
-      libxsmm_dmmdispatch(LIBXSMM_AVG_M, LIBXSMM_AVG_N, LIBXSMM_AVG_K,
+      const libxsmm_timer_tickint t0 = libxsmm_timer_tick();
+      libxsmm_dmmdispatch(23, 23, 23,
         NULL/*lda*/, NULL/*ldb*/, NULL/*ldc*/, NULL/*alpha*/, NULL/*beta*/,
         NULL/*flags*/, NULL/*prefetch*/);
+#if defined(_OPENMP)
+#     pragma omp atomic
+#endif
+      tdsp1 += libxsmm_timer_diff(t0, libxsmm_timer_tick());
     }
-    ddisp = libxsmm_timer_duration(start, libxsmm_timer_tick());
+
+#if defined(_OPENMP)
+#   pragma omp parallel for num_threads(nthreads) private(i)
+#endif
+    for (i = 0; i < size; ++i) {
+      const int j = 3 * i;
+      const libxsmm_timer_tickint t0 = libxsmm_timer_tick();
+      libxsmm_dmmdispatch(r[j], r[j+1], r[j+2],
+        NULL/*lda*/, NULL/*ldb*/, NULL/*ldc*/, NULL/*alpha*/, NULL/*beta*/,
+        NULL/*flags*/, NULL/*prefetch*/);
+#if defined(_OPENMP)
+#     pragma omp atomic
+#endif
+      tcgen += libxsmm_timer_diff(t0, libxsmm_timer_tick());
+    }
+
+    /* correct for duplicated code generation requests */
+    if (EXIT_SUCCESS == libxsmm_get_registry_info(&reginfo)) {
+      ncgens = (int)(reginfo.size - 1/*initial code gen.*/);
+      tcgen -= tdsp1 * (size - ncgens) / size;
+    }
+
+#if defined(_OPENMP)
+#   pragma omp parallel for num_threads(nthreads) private(i)
+#endif
+    for (i = 0; i < size; ++i) {
+      const int j = 3 * i;
+      const libxsmm_timer_tickint t0 = libxsmm_timer_tick();
+      libxsmm_dmmdispatch(r[j], r[j + 1], r[j + 2],
+        NULL/*lda*/, NULL/*ldb*/, NULL/*ldc*/, NULL/*alpha*/, NULL/*beta*/,
+        NULL/*flags*/, NULL/*prefetch*/);
+#if defined(_OPENMP)
+#     pragma omp atomic
+#endif
+      tdsp0 += libxsmm_timer_diff(t0, libxsmm_timer_tick());
+    }
+
+    free(r); /* release random numbers */
   }
 
-  if (0 < dcall && 0 < ddisp) {
-    fprintf(stdout, "\tdispatch calls/s: %.1f MHz\n", 1E-6 * size / ddisp);
-    fprintf(stdout, "\tempty calls/s: %.1f MHz\n", 1E-6 * size / dcall);
-    fprintf(stdout, "\toverhead: %.1fx\n", ddisp / dcall);
+  if (0 < size) {
+    const double dcall = libxsmm_timer_duration(0, tcall) / size;
+    const double ddsp0 = libxsmm_timer_duration(0, tdsp0) / size;
+    const double ddsp1 = libxsmm_timer_duration(0, tdsp1) / size;
+    const double dcgen = libxsmm_timer_duration(0, tcgen) / size;
+    if (0 < tcall && 0 < tdsp0 && 0 < tdsp1 && 0 < tcgen) {
+      fprintf(stdout, "\tfunction-call (empty): %.0f ns (%.0f MHz)\n", 1E9 * dcall, 1E-6 / dcall);
+      fprintf(stdout, "\tdispatch (ro/cached): %.0f ns (%.0f MHz)\n", 1E9 * ddsp1, 1E-6 / ddsp1);
+      fprintf(stdout, "\tdispatch (ro): %.0f ns (%.0f MHz)\n", 1E9 * ddsp0, 1E-6 / ddsp0);
+      fprintf(stdout, "\tcode-gen (rw): %.0f us (%.0f kHz)\n", 1E6 * dcgen, 1E-3 / dcgen);
+    }
   }
   fprintf(stdout, "Finished\n");
 
