@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2016-2017, Intel Corporation                                **
+** Copyright (c) 2016-2018, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -28,169 +28,63 @@
 ******************************************************************************/
 /* Alexander Heinecke (Intel Corp.)
 ******************************************************************************/
+#include "edge_proxy_common.h"
 #include <libxsmm.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
 #include <math.h>
+
 #if defined(_OPENMP)
 # include <omp.h>
 #endif
 
-#if defined(_WIN32) || defined(__CYGWIN__)
-/* note: later on, this leads to (correct but) different than expected norm-values */
+#if defined(_WIN32) || defined(__CYGWIN__) || !(defined(_SVID_SOURCE) || defined(_XOPEN_SOURCE))
 # define drand48() ((double)rand() / RAND_MAX)
 # define srand48 srand
 #endif
 
-LIBXSMM_INLINE
-void edge_sparse_csr_reader( const char*           i_csr_file_in,
-                             unsigned int**        o_row_idx,
-                             unsigned int**        o_column_idx,
-                             double**              o_values,
-                             unsigned int*         o_row_count,
-                             unsigned int*         o_column_count,
-                             unsigned int*         o_element_count ) {
-  FILE *l_csr_file_handle;
-  const unsigned int l_line_length = 512;
-  char l_line[512/*l_line_length*/+1];
-  unsigned int l_header_read = 0;
-  unsigned int* l_row_idx_id = NULL;
-  unsigned int l_i = 0;
-
-  l_csr_file_handle = fopen( i_csr_file_in, "r" );
-  if ( l_csr_file_handle == NULL ) {
-    fprintf( stderr, "cannot open CSR file!\n" );
-    return;
-  }
-
-  while (fgets(l_line, l_line_length, l_csr_file_handle) != NULL) {
-    if ( strlen(l_line) == l_line_length ) {
-      fprintf( stderr, "could not read file length!\n" );
-      return;
-    }
-    /* check if we are still reading comments header */
-    if ( l_line[0] == '%' ) {
-      continue;
-    } else {
-      /* if we are the first line after comment header, we allocate our data structures */
-      if ( l_header_read == 0 ) {
-        if ( sscanf(l_line, "%u %u %u", o_row_count, o_column_count, o_element_count) == 3 ) {
-          /* allocate CSC datastructue matching mtx file */
-          *o_column_idx = (unsigned int*) malloc(sizeof(unsigned int) * (*o_element_count));
-          *o_row_idx = (unsigned int*) malloc(sizeof(unsigned int) * (*o_row_count + 1));
-          *o_values = (double*) malloc(sizeof(double) * (*o_element_count));
-          l_row_idx_id = (unsigned int*) malloc(sizeof(unsigned int) * (*o_row_count));
-
-          /* check if mallocs were successful */
-          if ( ( *o_row_idx == NULL )      ||
-               ( *o_column_idx == NULL )   ||
-               ( *o_values == NULL )       ||
-               ( l_row_idx_id == NULL )    ) {
-            fprintf( stderr, "could not allocate sp data!\n" );
-            return;
-          }
-
-          /* set everything to zero for init */
-          memset(*o_row_idx, 0, sizeof(unsigned int)*(*o_row_count + 1));
-          memset(*o_column_idx, 0, sizeof(unsigned int)*(*o_element_count));
-          memset(*o_values, 0, sizeof(double)*(*o_element_count));
-          memset(l_row_idx_id, 0, sizeof(unsigned int)*(*o_row_count));
-
-          /* init column idx */
-          for ( l_i = 0; l_i < (*o_row_count + 1); l_i++)
-            (*o_row_idx)[l_i] = (*o_element_count);
-
-          /* init */
-          (*o_row_idx)[0] = 0;
-          l_i = 0;
-          l_header_read = 1;
-        } else {
-          fprintf( stderr, "could not csr descripton!\n" );
-          return;
-        }
-      /* now we read the actual content */
-      } else {
-        unsigned int l_row, l_column;
-        double l_value;
-        /* read a line of content */
-        if ( sscanf(l_line, "%u %u %lf", &l_row, &l_column, &l_value) != 3 ) {
-          fprintf( stderr, "could not read element!\n" );
-          return;
-        }
-        /* adjust numbers to zero termination */
-        l_row--;
-        l_column--;
-        /* add these values to row and value strucuture */
-        (*o_column_idx)[l_i] = l_column;
-        (*o_values)[l_i] = l_value;
-        l_i++;
-        /* handle columns, set id to onw for this column, yeah we need to hanle empty columns */
-        l_row_idx_id[l_row] = 1;
-        (*o_row_idx)[l_row+1] = l_i;
-      }
-    }
-  }
-
-  /* close mtx file */
-  fclose( l_csr_file_handle );
-
-  /* check if we read a file which was consitent */
-  if ( l_i != (*o_element_count) ) {
-    fprintf( stderr, "we were not able to read all elements!\n" );
-    return;
-  }
-
-  /* let's handle empty rows */
-  for ( l_i = 0; l_i < (*o_row_count); l_i++) {
-    if ( l_row_idx_id[l_i] == 0 ) {
-      (*o_row_idx)[l_i+1] = (*o_row_idx)[l_i];
-    }
-  }
-
-  /* free helper data structure */
-  if ( l_row_idx_id != NULL ) {
-    free( l_row_idx_id );
-  }
-}
-
 
 int main(int argc, char* argv[])
 {
-  char* mat_a;
+  char* mat_a = 0;
   unsigned int *mat_a_rowptr, *mat_a_colidx;
   unsigned int mat_a_rowcount, mat_a_colcount, mat_a_nnz;
   double* mat_a_values;
   libxsmm_dmmfunction a_kernel;
 
-  char* mat_b;
+  char* mat_b = 0;
   unsigned int *mat_b_rowptr, *mat_b_colidx;
   unsigned int mat_b_rowcount, mat_b_colcount, mat_b_nnz;
   double* mat_b_values;
   libxsmm_dmmfunction b_kernel;
 
-  char* mat_c;
+  char* mat_c = 0;
   unsigned int *mat_c_rowptr, *mat_c_colidx;
   unsigned int mat_c_rowcount, mat_c_colcount, mat_c_nnz;
   double* mat_c_values;
   libxsmm_dmmfunction c_kernel;
 
-  char* mat_st;
+  char* mat_st = 0;
   unsigned int *mat_st_rowptr, *mat_st_colidx;
   unsigned int mat_st_rowcount, mat_st_colcount, mat_st_nnz;
   double* mat_st_values;
   libxsmm_dmmfunction st_kernel;
 
-  size_t num_elems;
-  size_t num_modes;
-  size_t num_quants = 9;
+  int num_modes = 9;
+  int num_quants = 9;
+  size_t num_elems = 0;
   size_t num_cfr = 8;
-  size_t num_reps;
+  size_t num_reps = 1;
   size_t elem_size;
-  size_t i, j;
+  /* OpenMP: signed induction variables */
+  int i, j;
 
-  libxsmm_gemm_descriptor l_xgemm_desc_stiff;
-  libxsmm_gemm_descriptor l_xgemm_desc_star;
+  const libxsmm_gemm_descriptor *l_xgemm_desc_stiff = 0, *l_xgemm_desc_star = 0;
+  libxsmm_descriptor_blob l_xgemm_blob_stiff, l_xgemm_blob_star;
+  const libxsmm_gemm_prefetch_type prefetch = LIBXSMM_GEMM_PREFETCH_NONE;
+  const int flags = LIBXSMM_GEMM_FLAGS('N', 'N');
+  const double alpha = 1, beta = 1;
 
   double* q;
   double* qt;
@@ -219,31 +113,33 @@ int main(int argc, char* argv[])
 
   /* read matrices */
   printf("reading sparse matrices... ");
-  edge_sparse_csr_reader( mat_a, &mat_a_rowptr, &mat_a_colidx, &mat_a_values, &mat_a_rowcount, &mat_a_colcount, &mat_a_nnz );
-  edge_sparse_csr_reader( mat_b, &mat_b_rowptr, &mat_b_colidx, &mat_b_values, &mat_b_rowcount, &mat_b_colcount, &mat_b_nnz );
-  edge_sparse_csr_reader( mat_c, &mat_c_rowptr, &mat_c_colidx, &mat_c_values, &mat_c_rowcount, &mat_c_colcount, &mat_c_nnz );
-  edge_sparse_csr_reader( mat_st, &mat_st_rowptr, &mat_st_colidx, &mat_st_values, &mat_st_rowcount, &mat_st_colcount, &mat_st_nnz );
+  edge_sparse_csr_reader_double( mat_a, &mat_a_rowptr, &mat_a_colidx, &mat_a_values, &mat_a_rowcount, &mat_a_colcount, &mat_a_nnz );
+  edge_sparse_csr_reader_double( mat_b, &mat_b_rowptr, &mat_b_colidx, &mat_b_values, &mat_b_rowcount, &mat_b_colcount, &mat_b_nnz );
+  edge_sparse_csr_reader_double( mat_c, &mat_c_rowptr, &mat_c_colidx, &mat_c_values, &mat_c_rowcount, &mat_c_colcount, &mat_c_nnz );
+  edge_sparse_csr_reader_double( mat_st, &mat_st_rowptr, &mat_st_colidx, &mat_st_values, &mat_st_rowcount, &mat_st_colcount, &mat_st_nnz );
   printf("done!\n\n");
 
   /* generate kernels */
   printf("generating code... ");
-  LIBXSMM_GEMM_DESCRIPTOR(l_xgemm_desc_stiff, 1, 0, num_quants, num_modes, num_modes, num_modes, 0, num_modes, 1.0, 1.0, LIBXSMM_PREFETCH_NONE);
-  LIBXSMM_GEMM_DESCRIPTOR(l_xgemm_desc_star, 1, 0, num_quants, num_modes, num_quants, 0, num_modes, num_modes, 1.0, 1.0, LIBXSMM_PREFETCH_NONE);
-  a_kernel = libxsmm_create_dcsr_soa( &l_xgemm_desc_stiff, mat_a_rowptr, mat_a_colidx, mat_a_values ).dmm;
-  b_kernel = libxsmm_create_dcsr_soa( &l_xgemm_desc_stiff, mat_b_rowptr, mat_b_colidx, mat_b_values ).dmm;
-  c_kernel = libxsmm_create_dcsr_soa( &l_xgemm_desc_stiff, mat_c_rowptr, mat_c_colidx, mat_c_values ).dmm;
-  st_kernel = libxsmm_create_dcsr_soa( &l_xgemm_desc_star, mat_st_rowptr, mat_st_colidx, mat_st_values ).dmm;
+  l_xgemm_desc_stiff = libxsmm_dgemm_descriptor_init(&l_xgemm_blob_stiff,
+    num_quants, num_modes, num_modes, num_modes, 0, num_modes, alpha, beta, flags, prefetch);
+  l_xgemm_desc_star = libxsmm_dgemm_descriptor_init(&l_xgemm_blob_star,
+    num_quants, num_modes, num_quants, 0, num_modes, num_modes, alpha, beta, flags, prefetch);
+  a_kernel =  libxsmm_create_xcsr_soa( l_xgemm_desc_stiff, mat_a_rowptr,  mat_a_colidx,  (const void*)mat_a_values ).dmm;
+  b_kernel =  libxsmm_create_xcsr_soa( l_xgemm_desc_stiff, mat_b_rowptr,  mat_b_colidx,  (const void*)mat_b_values ).dmm;
+  c_kernel =  libxsmm_create_xcsr_soa( l_xgemm_desc_stiff, mat_c_rowptr,  mat_c_colidx,  (const void*)mat_c_values ).dmm;
+  st_kernel = libxsmm_create_xcsr_soa( l_xgemm_desc_star, mat_st_rowptr, mat_st_colidx, (const void*)mat_st_values ).dmm;
   if ( a_kernel == 0 || b_kernel == 0 || c_kernel == 0 || st_kernel == 0 ) {
     printf("one of the kernels could not be built -> exit!");
     exit(-1);
   }
   printf("done!\n\n");
 
-  /* create unkowns and tunkowns */
+  /* create unknowns and t-unknowns */
   printf("allocating and initializing fake data... \n");
   printf("   q: %f MiB\n", ((double)(num_elems*num_modes*num_quants*num_cfr*sizeof(double)))/ ( 1024.0*1024.0) );
   printf("  qt: %f MiB\n", ((double)(num_elems*num_modes*num_quants*num_cfr*sizeof(double)))/ ( 1024.0*1024.0) );
-#ifdef _OPENMP
+#if defined(_OPENMP)
   printf("   t: %f MiB\n", ((double)(omp_get_max_threads()*num_modes*num_quants*num_cfr*sizeof(double)))/ ( 1024.0*1024.0) );
 #else
   printf("   t: %f MiB\n", ((double)(num_modes*num_quants*num_cfr*sizeof(double)))/ ( 1024.0*1024.0) );
@@ -251,15 +147,19 @@ int main(int argc, char* argv[])
   q = (double*)libxsmm_aligned_malloc( num_elems*num_modes*num_quants*num_cfr*sizeof(double), 2097152);
   qt = (double*)libxsmm_aligned_malloc( num_elems*num_modes*num_quants*num_cfr*sizeof(double), 2097152);
 
-  #pragma omp parallel for private(i,j)
-  for ( i = 0; i < num_elems; i++ ) {
-    for ( j = 0; j < elem_size; j++) {
+#if defined(_OPENMP)
+# pragma omp parallel for private(i,j)
+#endif
+  for (i = 0; i < (int)num_elems; i++) {
+    for (j = 0; j < (int)elem_size; j++) {
       q[i*elem_size + j] = drand48();
     }
   }
-  #pragma omp parallel for private(i,j)
-  for ( i = 0; i < num_elems; i++ ) {
-    for ( j = 0; j < elem_size; j++) {
+#if defined(_OPENMP)
+# pragma omp parallel for private(i,j)
+#endif
+  for (i = 0; i < (int)num_elems; i++) {
+    for (j = 0; j < (int)elem_size; j++) {
       qt[i*elem_size + j] = drand48();
     }
   }
@@ -269,7 +169,7 @@ int main(int argc, char* argv[])
   /* benchmark single core all kernels */
   printf("benchmarking kernels... \n");
   l_start = libxsmm_timer_tick();
-  for ( i = 0; i < num_reps; i++) {
+  for (i = 0; i < (int)num_reps; i++) {
     a_kernel( qt, mat_a_values, q );
   }
   l_end = libxsmm_timer_tick();
@@ -278,7 +178,7 @@ int main(int argc, char* argv[])
   printf("%f GFLOPS for stiff1 (asm)\n", ((double)((double)num_reps * (double)num_quants * (double)mat_a_nnz * (double)num_cfr) * 2.0) / (l_total * 1.0e9));
 
   l_start = libxsmm_timer_tick();
-  for ( i = 0; i < num_reps; i++) {
+  for (i = 0; i < (int)num_reps; i++) {
     b_kernel( qt, mat_b_values, q );
   }
   l_end = libxsmm_timer_tick();
@@ -287,7 +187,7 @@ int main(int argc, char* argv[])
   printf("%f GFLOPS for stiff2 (asm)\n", ((double)((double)num_reps * (double)num_quants * (double)mat_b_nnz * (double)num_cfr) * 2.0) / (l_total * 1.0e9));
 
   l_start = libxsmm_timer_tick();
-  for ( i = 0; i < num_reps; i++) {
+  for (i = 0; i < (int)num_reps; i++) {
     c_kernel( qt, mat_c_values, q );
   }
   l_end = libxsmm_timer_tick();
@@ -296,7 +196,7 @@ int main(int argc, char* argv[])
   printf("%f GFLOPS for stiff3 (asm)\n", ((double)((double)num_reps * (double)num_quants * (double)mat_c_nnz * (double)num_cfr) * 2.0) / (l_total * 1.0e9));
 
   l_start = libxsmm_timer_tick();
-  for ( i = 0; i < num_reps; i++) {
+  for (i = 0; i < (int)num_reps; i++) {
     st_kernel( mat_st_values, qt, q );
   }
   l_end = libxsmm_timer_tick();
@@ -306,27 +206,35 @@ int main(int argc, char* argv[])
   printf("done!\n\n");
 
   /* benchmark volumne integration */
-  #pragma omp parallel for private(i,j)
-  for ( i = 0; i < num_elems; i++ ) {
-    for ( j = 0; j < elem_size; j++) {
+#if defined(_OPENMP)
+# pragma omp parallel for private(i,j)
+#endif
+  for (i = 0; i < (int)num_elems; i++) {
+    for (j = 0; j < (int)elem_size; j++) {
       q[i*elem_size + j] = drand48();
     }
   }
-  #pragma omp parallel for private(i,j)
-  for ( i = 0; i < num_elems; i++ ) {
-    for ( j = 0; j < elem_size; j++) {
+#if defined(_OPENMP)
+# pragma omp parallel for private(i,j)
+#endif
+  for (i = 0; i < (int)num_elems; i++) {
+    for (j = 0; j < (int)elem_size; j++) {
       qt[i*elem_size + j] = drand48();
     }
   }
 
   l_start = libxsmm_timer_tick();
-  for ( i = 0; i < num_reps; i++) {
-    #pragma omp parallel private(i, j)
+  for (i = 0; i < (int)num_reps; i++) {
+#if defined(_OPENMP)
+#   pragma omp parallel private(i, j)
+#endif
     {
-      __attribute__((aligned(64))) double tp[20*8*9];
+      LIBXSMM_ALIGNED(double tp[20*8*9], LIBXSMM_ALIGNMENT);
 
-      #pragma omp for private(j)
-      for ( j = 0; j < num_elems; j++ ) {
+#if defined(_OPENMP)
+#     pragma omp for private(j)
+#endif
+      for (j = 0; j < (int)num_elems; j++) {
         st_kernel( mat_st_values, qt+(j*elem_size), tp );
         a_kernel( tp, mat_a_values, q+(j*elem_size) );
 
@@ -345,8 +253,6 @@ int main(int argc, char* argv[])
   printf("%f GiB/s for vol (asm)\n", (double)((double)num_elems * (double)elem_size * 8.0 * 3.0 * (double)num_reps) / (l_total * 1024.0*1024.0*1024.0) );
   printf("done!\n\n");
 
-
-
   /* some empty lines at the end */
   printf("\n\n");
 
