@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2014-2017, Intel Corporation                                **
+** Copyright (c) 2014-2018, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -40,25 +40,6 @@
 
 #define REALTYPE double
 
-/* forward decelration of generated code */
-void libxsmm_code(const double* A, const double* B, double* C);
-libxsmm_dmmfunction libxsmm_jit;
-
-void libxsmm_kernel(const double* A, const double* B, double* C, const unsigned int N, const unsigned int vlen) {
-  unsigned int n;
-
-#ifdef _OPENMP
-  #pragma omp parallel for private(n)
-#endif
-  for (n = 0; n < N; n+=vlen) {
-#if 0
-    libxsmm_code(A, B+n, C+n);
-#else
-    libxsmm_jit(A, B+n, C+n);
-#endif
-  }
-}
-
 static double sec(struct timeval start, struct timeval end) {
   return ((double)(((end.tv_sec * 1000000 + end.tv_usec) - (start.tv_sec * 1000000 + start.tv_usec)))) / 1.0e6;
 }
@@ -95,7 +76,7 @@ int my_csr_reader( const char*           i_csr_file_in,
       /* if we are the first line after comment header, we allocate our data structures */
       if ( l_header_read == 0 ) {
         if ( sscanf(l_line, "%u %u %u", o_row_count, o_column_count, o_element_count) == 3 ) {
-          /* allocate CSC datastructue matching mtx file */
+          /* allocate CSC datastructure matching mtx file */
           *o_column_idx = (unsigned int*) malloc(sizeof(unsigned int) * (*o_element_count));
           *o_row_idx = (unsigned int*) malloc(sizeof(unsigned int) * (*o_row_count + 1));
           *o_values = (REALTYPE*) malloc(sizeof(double) * (*o_element_count));
@@ -105,7 +86,7 @@ int my_csr_reader( const char*           i_csr_file_in,
           if ( ( *o_row_idx == NULL )      ||
                ( *o_column_idx == NULL )   ||
                ( *o_values == NULL )       ||
-               ( l_row_idx_id == NULL )    ) {
+               ( l_row_idx_id == NULL ) ) {
             fprintf( stderr, "could not allocate sp data!\n" );
             return -1;
           }
@@ -125,7 +106,7 @@ int my_csr_reader( const char*           i_csr_file_in,
           l_i = 0;
           l_header_read = 1;
         } else {
-          fprintf( stderr, "could not csr descripton!\n" );
+          fprintf( stderr, "could not csr description!\n" );
           return -1;
         }
       /* now we read the actual content */
@@ -140,11 +121,11 @@ int my_csr_reader( const char*           i_csr_file_in,
         /* adjust numbers to zero termination */
         l_row--;
         l_column--;
-        /* add these values to row and value strucuture */
+        /* add these values to row and value structure */
         (*o_column_idx)[l_i] = l_column;
         (*o_values)[l_i] = l_value;
         l_i++;
-        /* handle columns, set id to onw for this column, yeah we need to hanle empty columns */
+        /* handle columns, set id to own for this column, yeah we need to handle empty columns */
         l_row_idx_id[l_row] = 1;
         (*o_row_idx)[l_row+1] = l_i;
       }
@@ -154,7 +135,7 @@ int my_csr_reader( const char*           i_csr_file_in,
   /* close mtx file */
   fclose( l_csr_file_handle );
 
-  /* check if we read a file which was consitent */
+  /* check if we read a file which was consistent */
   if ( l_i != (*o_element_count) ) {
     fprintf( stderr, "we were not able to read all elements!\n" );
     return -1;
@@ -175,11 +156,6 @@ int my_csr_reader( const char*           i_csr_file_in,
 }
 
 int main(int argc, char* argv[]) {
-  if (argc != 4 ) {
-    fprintf( stderr, "need csr-filename N reps!\n" );
-    exit(-1);
-  }
-
   char* l_csr_file;
   REALTYPE* l_a_sp;
   unsigned int* l_rowptr;
@@ -188,9 +164,12 @@ int main(int argc, char* argv[]) {
 
   REALTYPE* l_a_dense;
   REALTYPE* l_b;
-  REALTYPE* l_c;
-  REALTYPE* l_c_gold;
-  REALTYPE* l_c_dense;
+  REALTYPE* l_c_betaone;
+  REALTYPE* l_c_betazero;
+  REALTYPE* l_c_gold_betaone;
+  REALTYPE* l_c_gold_betazero;
+  REALTYPE* l_c_dense_betaone;
+  REALTYPE* l_c_dense_betazero;
   REALTYPE l_max_error = 0.0;
   unsigned int l_m;
   unsigned int l_n;
@@ -201,12 +180,23 @@ int main(int argc, char* argv[]) {
   unsigned int l_z;
   unsigned int l_elems;
   unsigned int l_reps;
-  unsigned int l_vlen;
+  unsigned int l_n_block;
 
   struct timeval l_start, l_end;
   double l_total;
 
-  libxsmm_gemm_descriptor* l_xgemm_desc = NULL;
+  double alpha = 1.0;
+  double beta = 1.0;
+  char trans = 'N';
+
+  libxsmm_dfsspmdm* gemm_op_betazero = NULL;
+  libxsmm_dfsspmdm* gemm_op_betaone = NULL;
+
+  if (argc != 4 ) {
+    fprintf( stderr, "need csr-filename N reps!\n" );
+    exit(-1);
+  }
+
 
   /* read sparse A */
   l_csr_file = argv[1];
@@ -225,16 +215,15 @@ int main(int argc, char* argv[]) {
   printf("CSR matrix data structure we just read:\n");
   printf("rows: %u, columns: %u, elements: %u\n", l_rowcount, l_colcount, l_elements);
 
-  l_vlen = 8;
-  l_xgemm_desc = libxsmm_create_dgemm_descriptor('n', 'n', l_m, l_vlen, l_k, 0, l_n, l_n, 1.0, 1.0, LIBXSMM_PREFETCH_NONE);
-  libxsmm_jit = libxsmm_create_dcsr_reg( l_xgemm_desc, l_rowptr, l_colidx, l_a_sp ).dmm;
-
   /* allocate dense matrices */
   l_a_dense = (REALTYPE*)_mm_malloc(l_k * l_m * sizeof(REALTYPE), 64);
   l_b = (REALTYPE*)_mm_malloc(l_k * l_n * sizeof(REALTYPE), 64);
-  l_c = (REALTYPE*)_mm_malloc(l_m * l_n * sizeof(REALTYPE), 64);
-  l_c_gold = (REALTYPE*)_mm_malloc(l_m * l_n * sizeof(REALTYPE), 64);
-  l_c_dense = (REALTYPE*)_mm_malloc(l_m * l_n * sizeof(REALTYPE), 64);
+  l_c_betazero = (REALTYPE*)_mm_malloc(l_m * l_n * sizeof(REALTYPE), 64);
+  l_c_betaone = (REALTYPE*)_mm_malloc(l_m * l_n * sizeof(REALTYPE), 64);
+  l_c_gold_betazero = (REALTYPE*)_mm_malloc(l_m * l_n * sizeof(REALTYPE), 64);
+  l_c_gold_betaone = (REALTYPE*)_mm_malloc(l_m * l_n * sizeof(REALTYPE), 64);
+  l_c_dense_betazero = (REALTYPE*)_mm_malloc(l_m * l_n * sizeof(REALTYPE), 64);
+  l_c_dense_betaone = (REALTYPE*)_mm_malloc(l_m * l_n * sizeof(REALTYPE), 64);
 
   /* touch B */
   for ( l_i = 0; l_i < l_k*l_n; l_i++) {
@@ -255,18 +244,47 @@ int main(int argc, char* argv[]) {
 
   /* touch C */
   for ( l_i = 0; l_i < l_m*l_n; l_i++) {
-    l_c[l_i] = (REALTYPE)0.0;
-    l_c_gold[l_i] = (REALTYPE)0.0;
-    l_c_dense[l_i] = (REALTYPE)0.0;
+    l_c_gold_betaone[l_i] = (REALTYPE)drand48();
   }
+  for ( l_i = 0; l_i < l_m*l_n; l_i++) {
+    l_c_betaone[l_i] = l_c_gold_betaone[l_i];
+  }
+  for ( l_i = 0; l_i < l_m*l_n; l_i++) {
+    l_c_dense_betaone[l_i] = l_c_gold_betaone[l_i];
+  }
+  for ( l_i = 0; l_i < l_m*l_n; l_i++) {
+    l_c_betazero[l_i] = l_c_betaone[l_i];
+  }
+  for ( l_i = 0; l_i < l_m*l_n; l_i++) {
+    l_c_gold_betazero[l_i] = l_c_gold_betaone[l_i];
+  }
+  for ( l_i = 0; l_i < l_m*l_n; l_i++) {
+    l_c_dense_betazero[l_i] = l_c_dense_betaone[l_i];
+  }
+
+  /* setting up fsspmdm */
+  l_n_block = 48;
+  beta = 0.0;
+  gemm_op_betazero = libxsmm_dfsspmdm_create( l_m, l_n_block, l_k, l_k, l_n, l_n, 1.0, beta, l_a_dense );
+  beta = 1.0;
+  gemm_op_betaone = libxsmm_dfsspmdm_create( l_m, l_n_block, l_k, l_k, l_n, l_n, 1.0, beta, l_a_dense );
 
   /* compute golden results */
   printf("computing golden solution...\n");
   for ( l_j = 0; l_j < l_n; l_j++ ) {
     for (l_i = 0; l_i < l_m; l_i++ ) {
       l_elems = l_rowptr[l_i+1] - l_rowptr[l_i];
+      l_c_gold_betazero[(l_n*l_i) + l_j] = 0.0;
       for (l_z = 0; l_z < l_elems; l_z++) {
-        l_c_gold[(l_n*l_i) + l_j] +=  l_a_sp[l_rowptr[l_i]+l_z] * l_b[(l_n*l_colidx[l_rowptr[l_i]+l_z])+l_j];
+        l_c_gold_betazero[(l_n*l_i) + l_j] +=  l_a_sp[l_rowptr[l_i]+l_z] * l_b[(l_n*l_colidx[l_rowptr[l_i]+l_z])+l_j];
+      }
+    }
+  }
+  for ( l_j = 0; l_j < l_n; l_j++ ) {
+    for (l_i = 0; l_i < l_m; l_i++ ) {
+      l_elems = l_rowptr[l_i+1] - l_rowptr[l_i];
+      for (l_z = 0; l_z < l_elems; l_z++) {
+        l_c_gold_betaone[(l_n*l_i) + l_j] +=  l_a_sp[l_rowptr[l_i]+l_z] * l_b[(l_n*l_colidx[l_rowptr[l_i]+l_z])+l_j];
       }
     }
   }
@@ -274,57 +292,114 @@ int main(int argc, char* argv[]) {
 
   /* libxsmm generated code */
   printf("computing libxsmm (A sparse) solution...\n");
-  libxsmm_kernel(NULL, l_b, l_c, l_n, l_vlen);
+#ifdef _OPENMP
+  #pragma omp parallel for private(l_z)
+#endif
+  for (l_z = 0; l_z < l_n; l_z+=l_n_block) {
+    libxsmm_dfsspmdm_execute( gemm_op_betazero, l_b+l_z, l_c_betazero+l_z );
+  }
+#ifdef _OPENMP
+  #pragma omp parallel for private(l_z)
+#endif
+  for (l_z = 0; l_z < l_n; l_z+=l_n_block) {
+    libxsmm_dfsspmdm_execute( gemm_op_betaone, l_b+l_z, l_c_betaone+l_z );
+  }
   printf("...done!\n");
 
   /* BLAS code */
   printf("computing BLAS (A dense) solution...\n");
-  double alpha = 1.0;
-  double beta = 1.0;
-  char trans = 'N';
-  dgemm(&trans, &trans, &l_n, &l_m, &l_k, &alpha, l_b, &l_n, l_a_dense, &l_k, &beta, l_c_dense, &l_n );
+  beta = 0.0;
+  dgemm(&trans, &trans, &l_n, &l_m, &l_k, &alpha, l_b, &l_n, l_a_dense, &l_k, &beta, l_c_dense_betazero, &l_n );
+  beta = 1.0;
+  dgemm(&trans, &trans, &l_n, &l_m, &l_k, &alpha, l_b, &l_n, l_a_dense, &l_k, &beta, l_c_dense_betaone, &l_n );
   printf("...done!\n");
 
   /* check for errors */
   l_max_error = (REALTYPE)0.0;
   for ( l_i = 0; l_i < l_m*l_n; l_i++) {
-    if (fabs(l_c[l_i]-l_c_gold[l_i]) > l_max_error ) {
-      l_max_error = fabs(l_c[l_i]-l_c_gold[l_i]);
+    if (fabs(l_c_betazero[l_i]-l_c_gold_betazero[l_i]) > l_max_error ) {
+      l_max_error = fabs(l_c_betazero[l_i]-l_c_gold_betazero[l_i]);
     }
   }
-  printf("max error (libxmm vs. gold): %f\n", l_max_error);
+  printf("max error beta=0 (libxmm vs. gold): %f\n", l_max_error);
   l_max_error = (REALTYPE)0.0;
   for ( l_i = 0; l_i < l_m*l_n; l_i++) {
-    if (fabs(l_c_dense[l_i]-l_c_gold[l_i]) > l_max_error ) {
-      l_max_error = fabs(l_c_dense[l_i]-l_c_gold[l_i]);
+    if (fabs(l_c_betaone[l_i]-l_c_gold_betaone[l_i]) > l_max_error ) {
+      l_max_error = fabs(l_c_betaone[l_i]-l_c_gold_betaone[l_i]);
     }
   }
-  printf("max error (dense vs. gold): %f\n", l_max_error);
+  printf("max error beta=1 (libxmm vs. gold): %f\n", l_max_error);
+  l_max_error = (REALTYPE)0.0;
+  for ( l_i = 0; l_i < l_m*l_n; l_i++) {
+    if (fabs(l_c_dense_betazero[l_i]-l_c_gold_betazero[l_i]) > l_max_error ) {
+      l_max_error = fabs(l_c_dense_betazero[l_i]-l_c_gold_betazero[l_i]);
+    }
+  }
+  printf("max error beta=0 (dense vs. gold): %f\n", l_max_error);
+  l_max_error = (REALTYPE)0.0;
+  for ( l_i = 0; l_i < l_m*l_n; l_i++) {
+    if (fabs(l_c_dense_betaone[l_i]-l_c_gold_betaone[l_i]) > l_max_error ) {
+      l_max_error = fabs(l_c_dense_betaone[l_i]-l_c_gold_betaone[l_i]);
+    }
+  }
+  printf("max error beta=1 (dense vs. gold): %f\n", l_max_error);
 
   /* Let's measure performance */
   gettimeofday(&l_start, NULL);
   for ( l_j = 0; l_j < l_reps; l_j++ ) {
-    libxsmm_kernel(NULL, l_b, l_c, l_n, l_vlen);
+#ifdef _OPENMP
+    #pragma omp parallel for private(l_z)
+#endif
+    for (l_z = 0; l_z < l_n; l_z+=l_n_block) {
+      libxsmm_dfsspmdm_execute( gemm_op_betazero, l_b+l_z, l_c_betazero+l_z );
+    }
   }
   gettimeofday(&l_end, NULL);
   l_total = sec(l_start, l_end);
-  fprintf(stdout, "time[s] LIBXSMM (RM, M=%i, N=%i, K=%i): %f\n", l_m, l_n, l_k, l_total/(double)l_reps );
-  fprintf(stdout, "GFLOPS  LIBXSMM (RM, M=%i, N=%i, K=%i): %f (sparse)\n", l_m, l_n, l_k, (2.0 * (double)l_elements * (double)l_n * (double)l_reps * 1.0e-9) / l_total );
-  fprintf(stdout, "GFLOPS  LIBXSMM (RM, M=%i, N=%i, K=%i): %f (dense)\n", l_m, l_n, l_k, (2.0 * (double)l_m * (double)l_n * (double)l_k * (double)l_reps * 1.0e-9) / l_total );
-  fprintf(stdout, "GB/s    LIBXSMM (RM, M=%i, N=%i, K=%i): %f\n", l_m, l_n, l_k, ((double)sizeof(double) * ((2.0*(double)l_m * (double)l_n) + ((double)l_k * (double)l_n)) * (double)l_reps * 1.0e-9) / l_total );
+  fprintf(stdout, "time[s] LIBXSMM (RM, M=%i, N=%i, K=%i, beta=0): %f\n", l_m, l_n, l_k, l_total/(double)l_reps );
+  fprintf(stdout, "GFLOPS  LIBXSMM (RM, M=%i, N=%i, K=%i, beta=0): %f (sparse)\n", l_m, l_n, l_k, (2.0 * (double)l_elements * (double)l_n * (double)l_reps * 1.0e-9) / l_total );
+  fprintf(stdout, "GFLOPS  LIBXSMM (RM, M=%i, N=%i, K=%i, beta=0): %f (dense)\n", l_m, l_n, l_k, (2.0 * (double)l_m * (double)l_n * (double)l_k * (double)l_reps * 1.0e-9) / l_total );
+  fprintf(stdout, "GB/s    LIBXSMM (RM, M=%i, N=%i, K=%i, beta=0): %f\n", l_m, l_n, l_k, ((double)sizeof(double) * ((2.0*(double)l_m * (double)l_n) + ((double)l_k * (double)l_n)) * (double)l_reps * 1.0e-9) / l_total );
 
   gettimeofday(&l_start, NULL);
   for ( l_j = 0; l_j < l_reps; l_j++ ) {
-    dgemm(&trans, &trans, &l_n, &l_m, &l_k, &alpha, l_b, &l_n, l_a_dense, &l_k, &beta, l_c_dense, &l_n );
+#ifdef _OPENMP
+    #pragma omp parallel for private(l_z)
+#endif
+    for (l_z = 0; l_z < l_n; l_z+=l_n_block) {
+      libxsmm_dfsspmdm_execute( gemm_op_betaone, l_b+l_z, l_c_betaone+l_z );
+    }
   }
   gettimeofday(&l_end, NULL);
   l_total = sec(l_start, l_end);
-  fprintf(stdout, "time[s] MKL     (RM, M=%i, N=%i, K=%i): %f\n", l_m, l_n, l_k, l_total/(double)l_reps );
-  fprintf(stdout, "GFLOPS  MKL     (RM, M=%i, N=%i, K=%i): %f\n", l_m, l_n, l_k, (2.0 * (double)l_m * (double)l_n * (double)l_k * (double)l_reps * 1.0e-9) / l_total );
-  fprintf(stdout, "GB/s    MKL     (RM, M=%i, N=%i, K=%i): %f\n", l_m, l_n, l_k, ((double)sizeof(double) * ((2.0*(double)l_m * (double)l_n) + ((double)l_k * (double)l_n)) * (double)l_reps * 1.0e-9) / l_total );
+  fprintf(stdout, "time[s] LIBXSMM (RM, M=%i, N=%i, K=%i, beta=1): %f\n", l_m, l_n, l_k, l_total/(double)l_reps );
+  fprintf(stdout, "GFLOPS  LIBXSMM (RM, M=%i, N=%i, K=%i, beta=1): %f (sparse)\n", l_m, l_n, l_k, (2.0 * (double)l_elements * (double)l_n * (double)l_reps * 1.0e-9) / l_total );
+  fprintf(stdout, "GFLOPS  LIBXSMM (RM, M=%i, N=%i, K=%i, beta=1): %f (dense)\n", l_m, l_n, l_k, (2.0 * (double)l_m * (double)l_n * (double)l_k * (double)l_reps * 1.0e-9) / l_total );
+  fprintf(stdout, "GB/s    LIBXSMM (RM, M=%i, N=%i, K=%i, beta=1): %f\n", l_m, l_n, l_k, ((double)sizeof(double) * ((2.0*(double)l_m * (double)l_n) + ((double)l_k * (double)l_n)) * (double)l_reps * 1.0e-9) / l_total );
+
+  gettimeofday(&l_start, NULL);
+  beta = 0.0;
+  for ( l_j = 0; l_j < l_reps; l_j++ ) {
+    dgemm(&trans, &trans, &l_n, &l_m, &l_k, &alpha, l_b, &l_n, l_a_dense, &l_k, &beta, l_c_dense_betazero, &l_n );
+  }
+  gettimeofday(&l_end, NULL);
+  l_total = sec(l_start, l_end);
+  fprintf(stdout, "time[s] MKL     (RM, M=%i, N=%i, K=%i, beta=0): %f\n", l_m, l_n, l_k, l_total/(double)l_reps );
+  fprintf(stdout, "GFLOPS  MKL     (RM, M=%i, N=%i, K=%i, beta=0): %f\n", l_m, l_n, l_k, (2.0 * (double)l_m * (double)l_n * (double)l_k * (double)l_reps * 1.0e-9) / l_total );
+  fprintf(stdout, "GB/s    MKL     (RM, M=%i, N=%i, K=%i, beta=0): %f\n", l_m, l_n, l_k, ((double)sizeof(double) * ((2.0*(double)l_m * (double)l_n) + ((double)l_k * (double)l_n)) * (double)l_reps * 1.0e-9) / l_total );
+
+  gettimeofday(&l_start, NULL);
+  beta = 1.0;
+  for ( l_j = 0; l_j < l_reps; l_j++ ) {
+    dgemm(&trans, &trans, &l_n, &l_m, &l_k, &alpha, l_b, &l_n, l_a_dense, &l_k, &beta, l_c_dense_betaone, &l_n );
+  }
+  gettimeofday(&l_end, NULL);
+  l_total = sec(l_start, l_end);
+  fprintf(stdout, "time[s] MKL     (RM, M=%i, N=%i, K=%i, beta=1): %f\n", l_m, l_n, l_k, l_total/(double)l_reps );
+  fprintf(stdout, "GFLOPS  MKL     (RM, M=%i, N=%i, K=%i, beta=1): %f\n", l_m, l_n, l_k, (2.0 * (double)l_m * (double)l_n * (double)l_k * (double)l_reps * 1.0e-9) / l_total );
+  fprintf(stdout, "GB/s    MKL     (RM, M=%i, N=%i, K=%i, beta=1): %f\n", l_m, l_n, l_k, ((double)sizeof(double) * ((2.0*(double)l_m * (double)l_n) + ((double)l_k * (double)l_n)) * (double)l_reps * 1.0e-9) / l_total );
 
   /* free */
-  libxsmm_release_kernel(libxsmm_jit);
-  libxsmm_release_gemm_descriptor(l_xgemm_desc);
-  /* @TODO */
+  libxsmm_dfsspmdm_destroy( gemm_op_betazero );
+  libxsmm_dfsspmdm_destroy( gemm_op_betaone );
 }
