@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2017, Intel Corporation                                **
+** Copyright (c) 2015-2018, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -26,7 +26,7 @@
 ** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
 ** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
 ******************************************************************************/
-/* Alexander Heinecke (Intel Corp.)
+/* Alexander Heinecke, Greg Henry (Intel Corp.)
 ******************************************************************************/
 #ifndef GENERATOR_COMMON_H
 #define GENERATOR_COMMON_H
@@ -89,6 +89,11 @@
 #define LIBXSMM_X86_INSTR_VMOVDQU16      10032
 #define LIBXSMM_X86_INSTR_VMOVDQU32      10033
 #define LIBXSMM_X86_INSTR_VMOVDQU64      10034
+#define LIBXSMM_X86_INSTR_VMASKMOVPD     10035
+#define LIBXSMM_X86_INSTR_VMASKMOVPS     10036
+#define LIBXSMM_X86_INSTR_VMOVNTPD       10037
+#define LIBXSMM_X86_INSTR_VMOVNTPS       10038
+#define LIBXSMM_X86_INSTR_VMOVNTDQ       10039
 
 /* SSE */
 #define LIBXSMM_X86_INSTR_MOVAPD         10009
@@ -119,6 +124,13 @@
 #define LIBXSMM_X86_INSTR_VSCATTERQPS    11006
 #define LIBXSMM_X86_INSTR_VSCATTERQPD    11007
 
+/* Shuffle/Permute/Blend instructions */
+#define LIBXSMM_X86_INSTR_VSHUFPS        12000
+#define LIBXSMM_X86_INSTR_VPERM2F128     12001
+#define LIBXSMM_X86_INSTR_VSHUFF64X2     12002
+#define LIBXSMM_X86_INSTR_VEXTRACTF32X8  12003
+#define LIBXSMM_X86_INSTR_VEXTRACTF64X4  12004
+
 /* Vector compute instructions */
 /* AVX1,AVX2,AVX512 */
 #define LIBXSMM_X86_INSTR_VXORPD         20000
@@ -151,32 +163,65 @@
 #define LIBXSMM_X86_INSTR_VFMSUB231SS    20027
 #define LIBXSMM_X86_INSTR_VFNMADD231SS   20028
 #define LIBXSMM_X86_INSTR_VFNMSUB231SS   20029
+#define LIBXSMM_X86_INSTR_VPERMW         20030
 /* SSE */
-#define LIBXSMM_X86_INSTR_XORPD          20030
-#define LIBXSMM_X86_INSTR_MULPD          20031
-#define LIBXSMM_X86_INSTR_ADDPD          20032
-#define LIBXSMM_X86_INSTR_SUBPD          20033
-#define LIBXSMM_X86_INSTR_MULSD          20034
-#define LIBXSMM_X86_INSTR_ADDSD          20035
-#define LIBXSMM_X86_INSTR_SUBSD          20036
-#define LIBXSMM_X86_INSTR_XORPS          20037
-#define LIBXSMM_X86_INSTR_MULPS          20038
-#define LIBXSMM_X86_INSTR_ADDPS          20039
-#define LIBXSMM_X86_INSTR_SUBPS          20040
-#define LIBXSMM_X86_INSTR_MULSS          20041
-#define LIBXSMM_X86_INSTR_ADDSS          20042
-#define LIBXSMM_X86_INSTR_SUBSS          20043
+#define LIBXSMM_X86_INSTR_XORPD          20031
+#define LIBXSMM_X86_INSTR_MULPD          20032
+#define LIBXSMM_X86_INSTR_ADDPD          20033
+#define LIBXSMM_X86_INSTR_SUBPD          20034
+#define LIBXSMM_X86_INSTR_MULSD          20035
+#define LIBXSMM_X86_INSTR_ADDSD          20036
+#define LIBXSMM_X86_INSTR_SUBSD          20037
+#define LIBXSMM_X86_INSTR_XORPS          20038
+#define LIBXSMM_X86_INSTR_MULPS          20039
+#define LIBXSMM_X86_INSTR_ADDPS          20040
+#define LIBXSMM_X86_INSTR_SUBPS          20041
+#define LIBXSMM_X86_INSTR_MULSS          20042
+#define LIBXSMM_X86_INSTR_ADDSS          20043
+#define LIBXSMM_X86_INSTR_SUBSS          20044
 /* AVX512, IMCI Integer XOR as there is no FP */
-#define LIBXSMM_X86_INSTR_VPXORD         20044
+#define LIBXSMM_X86_INSTR_VPXORD         20045
 /* additional integer stuff */
-#define LIBXSMM_X86_INSTR_VPADDQ         20045
-#define LIBXSMM_X86_INSTR_VPADDD         20046
-#define LIBXSMM_X86_INSTR_VPADDW         20047
-#define LIBXSMM_X86_INSTR_VPADDB         20048
-#define LIBXSMM_X86_INSTR_VPMADDWD       20049
-#define LIBXSMM_X86_INSTR_VPMADDUBSW     20050
-#define LIBXSMM_X86_INSTR_VPADDSW        20051
-#define LIBXSMM_X86_INSTR_VPADDSB        20052
+#define LIBXSMM_X86_INSTR_VPADDQ         20046
+#define LIBXSMM_X86_INSTR_VPADDD         20047
+#define LIBXSMM_X86_INSTR_VPADDW         20048
+#define LIBXSMM_X86_INSTR_VPADDB         20049
+#define LIBXSMM_X86_INSTR_VPMADDWD       20050
+#define LIBXSMM_X86_INSTR_VPMADDUBSW     20051
+#define LIBXSMM_X86_INSTR_VPADDSW        20052
+#define LIBXSMM_X86_INSTR_VPADDSB        20053
+/* Additional vector manipulations */
+#define LIBXSMM_X86_INSTR_VUNPCKLPD      20054
+#define LIBXSMM_X86_INSTR_VUNPCKLPS      20055
+#define LIBXSMM_X86_INSTR_VUNPCKHPD      20056
+#define LIBXSMM_X86_INSTR_VUNPCKHPS      20057
+#define LIBXSMM_X86_INSTR_VPSRAVD        20058
+#define LIBXSMM_X86_INSTR_VCVTDQ2PS      20059
+#define LIBXSMM_X86_INSTR_VDIVPS         20060
+#define LIBXSMM_X86_INSTR_VDIVPD         20061
+#define LIBXSMM_X86_INSTR_VFMADD213PS    20062
+#define LIBXSMM_X86_INSTR_VCVTPS2PD      20063
+#define LIBXSMM_X86_INSTR_VBLENDMPS      20064
+#define LIBXSMM_X86_INSTR_VCMPPS         20065
+#define LIBXSMM_X86_INSTR_VPANDD         20066
+#define LIBXSMM_X86_INSTR_VPANDQ         20067
+#define LIBXSMM_X86_INSTR_VMAXPD         20068
+#define LIBXSMM_X86_INSTR_VMAXPS         20069
+#define LIBXSMM_X86_INSTR_VCVTPS2PH      20070
+#define LIBXSMM_X86_INSTR_VCVTPH2PS      20071
+#define LIBXSMM_X86_INSTR_VPERMD         20072
+
+/* AVX512, QUAD MADD, QUAD VNNI and VNNI */
+#define LIBXSMM_X86_INSTR_V4FMADDPS      26000
+#define LIBXSMM_X86_INSTR_V4FNMADDPS     26001
+#define LIBXSMM_X86_INSTR_V4FMADDSS      26002
+#define LIBXSMM_X86_INSTR_V4FNMADDSS     26003
+#define LIBXSMM_X86_INSTR_VP4DPWSSD      26004
+#define LIBXSMM_X86_INSTR_VP4DPWSSDS     26005
+#define LIBXSMM_X86_INSTR_VPDPWSSD       26006
+#define LIBXSMM_X86_INSTR_VPDPWSSDS      26007
+#define LIBXSMM_X86_INSTR_VPDPBUSD       26008
+#define LIBXSMM_X86_INSTR_VPDPBUSDS      26009
 
 /* GP instructions */
 #define LIBXSMM_X86_INSTR_ADDQ           30000
@@ -190,69 +235,94 @@
 #define LIBXSMM_X86_INSTR_PREFETCHT1     30008
 #define LIBXSMM_X86_INSTR_PREFETCHT2     30009
 #define LIBXSMM_X86_INSTR_PREFETCHNTA    30010
+#define LIBXSMM_X86_INSTR_MOVL           30011
+#define LIBXSMM_X86_INSTR_MOVSLQ         30012
+#define LIBXSMM_X86_INSTR_SALQ           30013
+#define LIBXSMM_X86_INSTR_IMUL           30014
+#define LIBXSMM_X86_INSTR_CMOVZ          30015
+#define LIBXSMM_X86_INSTR_CMOVNZ         30016
 
 /* Mask move instructions */
 #define LIBXSMM_X86_INSTR_KMOV           40000
 #define LIBXSMM_X86_INSTR_KMOVW          40001
+#define LIBXSMM_X86_INSTR_KMOVB          40002
+#define LIBXSMM_X86_INSTR_KMOVD          40003
+#define LIBXSMM_X86_INSTR_KMOVQ          40004
 
 /* Mask compute instructions */
 #define LIBXSMM_X86_INSTR_KXNORW         45000
 
 /* define error codes */
 #define LIBXSMM_ERR_GENERAL              90000
-#define LIBXSMM_ERR_ARCH_PREC            90001
-#define LIBXSMM_ERR_ARCH                 90002
-#define LIBXSMM_ERR_LDA                  90003
-#define LIBXSMM_ERR_LDB                  90004
-#define LIBXSMM_ERR_LDC                  90005
-#define LIBXSMM_ERR_SPGEMM_GEN           90006
-#define LIBXSMM_ERR_CSC_INPUT            90007
-#define LIBXSMM_ERR_CSC_READ_LEN         90008
-#define LIBXSMM_ERR_CSC_READ_DESC        90009
-#define LIBXSMM_ERR_CSC_READ_ELEMS       90010
-#define LIBXSMM_ERR_CSC_LEN              90011
-#define LIBXSMM_ERR_N_BLOCK              90012
-#define LIBXSMM_ERR_M_BLOCK              90013
-#define LIBXSMM_ERR_NO_IMCI              90014
-#define LIBXSMM_ERR_REG_BLOCK            90015
-#define LIBXSMM_ERR_VEC_MOVE_IMCI        90016
-#define LIBXSMM_ERR_APPEND_STR           90017
-#define LIBXSMM_ERR_ALLOC                90018
-#define LIBXSMM_ERR_NO_IMCI_AVX512_BCAST 90019
-#define LIBXSMM_ERR_CALLEE_SAVE_A        90020
-#define LIBXSMM_ERR_CALLEE_SAVE_B        90021
-#define LIBXSMM_ERR_CALLEE_SAVE_C        90022
-#define LIBXSMM_ERR_CALLEE_SAVE_A_PREF   90023
-#define LIBXSMM_ERR_CALLEE_SAVE_B_PREF   90024
-#define LIBXSMM_ERR_NO_INDEX_SCALE_ADDR  90025
-#define LIBXSMM_ERR_UNSUPPORTED_JUMP     90026
-#define LIBXSMM_ERR_NO_JMPLBL_AVAIL      90027
-#define LIBXSMM_ERR_EXCEED_JMPLBL        90028
-#define LIBXSMM_ERR_CSC_ALLOC_DATA       90029
-#define LIBXSMM_ERR_CSR_INPUT            90030
-#define LIBXSMM_ERR_CSR_READ_LEN         90031
-#define LIBXSMM_ERR_CSR_READ_DESC        90032
-#define LIBXSMM_ERR_CSR_READ_ELEMS       90033
-#define LIBXSMM_ERR_CSR_LEN              90034
-#define LIBXSMM_ERR_CSR_ALLOC_DATA       90035
-#define LIBXSMM_ERR_UNSUP_CONV_FORMAT    90036
-#define LIBXSMM_ERR_INVALID_KW_UNROLL    90037
-#define LIBXSMM_ERR_INVALID_KH_UNROLL    90038
-#define LIBXSMM_ERR_INVALID_OFW_UNROLL   90039
-#define LIBXSMM_ERR_INVALID_OFH_UNROLL   90040
-#define LIBXSMM_ERR_INVALID_CONV_ACC     90041
-#define LIBXSMM_ERR_UNSUP_ARCH           90042
-#define LIBXSMM_ERR_CONV_IFM_VEC         90043
-#define LIBXSMM_ERR_CONV_OFM_VEC         90044
-#define LIBXSMM_ERR_CONV_CONT_STRIDE     90045
-#define LIBXSMM_ERR_UNSUP_DATATYPE       90046
-#define LIBXSMM_ERR_UNSUP_DT_FORMAT      90047
-#define LIBXSMM_ERR_K_BLOCK              90048
-#define LIBXSMM_ERR_INVALID_GEMM_CONFIG  90049
-#define LIBXSMM_ERR_UNIQUE_VAL           90050
+#define LIBXSMM_ERR_ALLOC                90001
+#define LIBXSMM_ERR_BUFFER_TOO_SMALL     90002
+#define LIBXSMM_ERR_APPEND_STR           90003
+#define LIBXSMM_ERR_ARCH_PREC            90004
+#define LIBXSMM_ERR_ARCH                 90005
+#define LIBXSMM_ERR_UNSUP_ARCH           90006
+#define LIBXSMM_ERR_LDA                  90007
+#define LIBXSMM_ERR_LDB                  90008
+#define LIBXSMM_ERR_LDC                  90009
+#define LIBXSMM_ERR_SPGEMM_GEN           90010
+#define LIBXSMM_ERR_CSC_INPUT            90011
+#define LIBXSMM_ERR_CSC_READ_LEN         90012
+#define LIBXSMM_ERR_CSC_READ_DESC        90013
+#define LIBXSMM_ERR_CSC_READ_ELEMS       90014
+#define LIBXSMM_ERR_CSC_LEN              90015
+#define LIBXSMM_ERR_N_BLOCK              90016
+#define LIBXSMM_ERR_M_BLOCK              90017
+#define LIBXSMM_ERR_K_BLOCK              90018
+#define LIBXSMM_ERR_NO_IMCI              90019
+#define LIBXSMM_ERR_REG_BLOCK            90020
+#define LIBXSMM_ERR_VEC_MOVE_IMCI        90021
+#define LIBXSMM_ERR_NO_IMCI_AVX512_BCAST 90022
+#define LIBXSMM_ERR_NO_AVX512_QFMA       90023
+#define LIBXSMM_ERR_CALLEE_SAVE_A        90024
+#define LIBXSMM_ERR_CALLEE_SAVE_B        90025
+#define LIBXSMM_ERR_CALLEE_SAVE_C        90026
+#define LIBXSMM_ERR_CALLEE_SAVE_A_PREF   90027
+#define LIBXSMM_ERR_CALLEE_SAVE_B_PREF   90028
+#define LIBXSMM_ERR_NO_INDEX_SCALE_ADDR  90029
+#define LIBXSMM_ERR_UNSUPPORTED_JUMP     90030
+#define LIBXSMM_ERR_NO_JMPLBL_AVAIL      90031
+#define LIBXSMM_ERR_EXCEED_JMPLBL        90032
+#define LIBXSMM_ERR_CSC_ALLOC_DATA       90033
+#define LIBXSMM_ERR_CSR_ALLOC_DATA       90034
+#define LIBXSMM_ERR_CSR_INPUT            90035
+#define LIBXSMM_ERR_CSR_READ_LEN         90036
+#define LIBXSMM_ERR_CSR_READ_DESC        90037
+#define LIBXSMM_ERR_CSR_READ_ELEMS       90038
+#define LIBXSMM_ERR_CSR_LEN              90039
+#define LIBXSMM_ERR_UNSUP_CONV_FORMAT    90040
+#define LIBXSMM_ERR_INVALID_KW_UNROLL    90041
+#define LIBXSMM_ERR_INVALID_KH_UNROLL    90042
+#define LIBXSMM_ERR_INVALID_OFW_UNROLL   90043
+#define LIBXSMM_ERR_INVALID_OFH_UNROLL   90044
+#define LIBXSMM_ERR_INVALID_CONV_ACC     90045
+#define LIBXSMM_ERR_CONV_OFM_VEC         90046
+#define LIBXSMM_ERR_CONV_IFM_VEC         90047
+#define LIBXSMM_ERR_CONV_CONT_STRIDE     90048
+#define LIBXSMM_ERR_UNSUP_DATATYPE       90049
+#define LIBXSMM_ERR_UNSUP_DT_FORMAT      90050
+#define LIBXSMM_ERR_INVALID_GEMM_CONFIG  90051
+#define LIBXSMM_ERR_UNIQUE_VAL           90052
+#define LIBXSMM_ERR_VEC_REG_MUST_BE_UNDEF 90053
+
+#if !defined(LIBXSMM_VERBOSE_BACKEND) && !defined(NDEBUG)
+# define LIBXSMM_VERBOSE_BACKEND
+#endif
+#if defined(LIBXSMM_VERBOSE_BACKEND)
+# define LIBXSMM_BACKEND_VERBOSITY 1
+#else
+# define LIBXSMM_BACKEND_VERBOSITY 0
+#endif
+#define LIBXSMM_HANDLE_ERROR(GENERATED_CODE, ERROR_CODE) libxsmm_handle_error( \
+  GENERATED_CODE, ERROR_CODE, LIBXSMM_CALLER, LIBXSMM_BACKEND_VERBOSITY)
+#define LIBXSMM_HANDLE_ERROR_VERBOSE(GENERATED_CODE, ERROR_CODE) libxsmm_handle_error( \
+  GENERATED_CODE, ERROR_CODE, LIBXSMM_CALLER, 1)
 
 /* micro kernel config */
-typedef struct libxsmm_micro_kernel_config_struct {
+LIBXSMM_EXTERN_C typedef struct libxsmm_micro_kernel_config {
   unsigned int instruction_set;
   unsigned int vector_reg_count;
   unsigned int vector_length;
@@ -275,7 +345,7 @@ typedef struct libxsmm_micro_kernel_config_struct {
 } libxsmm_micro_kernel_config;
 
 /* struct for storing the current gp reg mapping */
-typedef struct libxsmm_gp_reg_mapping_struct {
+LIBXSMM_EXTERN_C typedef struct libxsmm_gp_reg_mapping_struct {
   unsigned int gp_reg_a;
   unsigned int gp_reg_b;
   unsigned int gp_reg_c;
@@ -294,7 +364,7 @@ typedef struct libxsmm_gp_reg_mapping_struct {
 } libxsmm_gp_reg_mapping;
 
 /* convolution kernel config */
-typedef struct libxsmm_convolution_kernel_config_struct {
+LIBXSMM_EXTERN_C typedef struct libxsmm_convolution_kernel_config_struct {
   unsigned int instruction_set;
   unsigned int vector_reg_count;
   unsigned int vector_length_in;
@@ -323,7 +393,7 @@ typedef struct libxsmm_convolution_kernel_config_struct {
 } libxsmm_convolution_kernel_config;
 
 /* struct for storing the current gp reg mapping for convolution */
-typedef struct libxsmm_convolution_forward_gp_reg_mapping_struct {
+LIBXSMM_EXTERN_C typedef struct libxsmm_convolution_forward_gp_reg_mapping_struct {
   unsigned int gp_reg_input;
   unsigned int gp_reg_weight;
   unsigned int gp_reg_output;
@@ -333,6 +403,7 @@ typedef struct libxsmm_convolution_forward_gp_reg_mapping_struct {
   unsigned int gp_reg_kw_loop;
   unsigned int gp_reg_kh_loop;
   unsigned int gp_reg_ifmInner_loop;
+  unsigned int gp_reg_ifmOuter_loop;
   unsigned int gp_reg_help_0;
   unsigned int gp_reg_help_1;
   unsigned int gp_reg_help_2;
@@ -343,7 +414,7 @@ typedef struct libxsmm_convolution_forward_gp_reg_mapping_struct {
 } libxsmm_convolution_forward_gp_reg_mapping;
 
 /* struct for storing the current gp reg mapping for convolution */
-typedef struct libxsmm_convolution_backward_gp_reg_mapping_struct {
+LIBXSMM_EXTERN_C typedef struct libxsmm_convolution_backward_gp_reg_mapping_struct {
   unsigned int gp_reg_input;
   unsigned int gp_reg_weight;
   unsigned int gp_reg_output;
@@ -364,7 +435,7 @@ typedef struct libxsmm_convolution_backward_gp_reg_mapping_struct {
 } libxsmm_convolution_backward_gp_reg_mapping;
 
 /* struct for storing the current gp reg mapping for convolution */
-typedef struct libxsmm_convolution_weight_update_gp_reg_mapping_struct {
+LIBXSMM_EXTERN_C typedef struct libxsmm_convolution_weight_update_gp_reg_mapping_struct {
   unsigned int gp_reg_input;
   unsigned int gp_reg_weight;
   unsigned int gp_reg_output;
@@ -383,74 +454,121 @@ typedef struct libxsmm_convolution_weight_update_gp_reg_mapping_struct {
   unsigned int gp_reg_help_6;
 } libxsmm_convolution_weight_update_gp_reg_mapping;
 
+/* struct for storing the current gp reg mapping for matcopy */
+LIBXSMM_EXTERN_C typedef struct libxsmm_matcopy_gp_reg_mapping_struct {
+  unsigned int gp_reg_a;
+  unsigned int gp_reg_lda;
+  unsigned int gp_reg_b;
+  unsigned int gp_reg_ldb;
+  unsigned int gp_reg_a_pf;
+  unsigned int gp_reg_b_pf;
+  unsigned int gp_reg_m_loop;
+  unsigned int gp_reg_n_loop;
+  unsigned int gp_reg_help_0;
+} libxsmm_matcopy_gp_reg_mapping;
+
+/* matcopy kernel config */
+LIBXSMM_EXTERN_C typedef struct libxsmm_matcopy_kernel_config_struct {
+  unsigned int instruction_set;
+  unsigned int vector_reg_count;
+  unsigned int vector_length;
+  unsigned int datatype_size;
+  unsigned int prefetch_instruction;
+  unsigned int vmove_instruction;
+  unsigned int alu_add_instruction;
+  unsigned int alu_cmp_instruction;
+  unsigned int alu_jmp_instruction;
+  unsigned int alu_mov_instruction;
+  unsigned int vxor_instruction;
+  char vector_name;
+} libxsmm_matcopy_kernel_config;
+
+/* struct for storing the current gp reg mapping for transpose */
+LIBXSMM_EXTERN_C typedef struct libxsmm_transpose_gp_reg_mapping_struct {
+  unsigned int gp_reg_a;
+  unsigned int gp_reg_lda;
+  unsigned int gp_reg_b;
+  unsigned int gp_reg_ldb;
+  unsigned int gp_reg_m_loop;
+  unsigned int gp_reg_n_loop;
+  unsigned int gp_reg_help_0;
+  unsigned int gp_reg_help_1;
+  unsigned int gp_reg_help_2;
+  unsigned int gp_reg_help_3;
+  unsigned int gp_reg_help_4;
+  unsigned int gp_reg_help_5;
+} libxsmm_transpose_gp_reg_mapping;
+
+/* transpose kernel config */
+LIBXSMM_EXTERN_C typedef struct libxsmm_transpose_kernel_config_struct {
+  unsigned int instruction_set;
+  unsigned int vector_reg_count;
+  char vector_name;
+} libxsmm_transpose_kernel_config;
+
 /* struct for tracking local labels in assembly
    we don't allow overlapping loops */
-typedef struct libxsmm_loop_label_tracker_struct {
+LIBXSMM_EXTERN_C typedef struct libxsmm_loop_label_tracker_struct {
   unsigned int label_address[32];
   unsigned int label_count;
 } libxsmm_loop_label_tracker;
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_reset_loop_label_tracker( libxsmm_loop_label_tracker* io_loop_label_tracker );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_get_x86_gp_reg_name( const unsigned int i_gp_reg_number,
                                   char*              o_gp_reg_name,
                                   const int          i_gp_reg_name_max_length );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 unsigned int libxsmm_check_x86_gp_reg_name_callee_save( const unsigned int i_gp_reg_number );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_get_x86_instr_name( const unsigned int i_instr_number,
                                  char*              o_instr_name,
                                  const int          i_instr_name_max_length );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_reset_x86_gp_reg_mapping( libxsmm_gp_reg_mapping* io_gp_reg_mapping );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 unsigned int libxsmm_is_x86_vec_instr_single_precision( const unsigned int i_instr_number );
 
 /* some string manipulation helper needed to
    generated code */
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_append_code_as_string( libxsmm_generated_code* io_generated_code,
                                     const char*             i_code_to_append,
                                     const int               i_append_length );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_close_function( libxsmm_generated_code* io_generated_code );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_mmfunction_signature( libxsmm_generated_code*       io_generated_code,
                                   const char*                    i_routine_name,
                                   const libxsmm_gemm_descriptor* i_xgemm_desc );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_isa_check_header( libxsmm_generated_code* io_generated_code,
                                          const char*             i_arch );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_isa_check_footer( libxsmm_generated_code* io_generated_code,
                                          const char*             i_arch );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_handle_error( libxsmm_generated_code* io_generated_code,
-                           const unsigned int      i_error_code );
+                           const unsigned int      i_error_code,
+                           const char*             context,
+                           int emit_message );
 
-
-LIBXSMM_INTERNAL_API
-void libxsmm_strncpy( char*        o_dest,
-                      const char*  i_src,
-                      unsigned int i_dest_length,
-                      unsigned int i_src_length );
-
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_convfunction_signature_fp32( libxsmm_generated_code*         io_generated_code,
                                           const char*                     i_routine_name     );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_convfunction_signature_int16( libxsmm_generated_code*         io_generated_code,
                                            const char*                     i_routine_name     );
 
