@@ -39,13 +39,19 @@
 #include "generator_convolution_weight_update_avx2.h"
 #include "generator_convolution_weight_update_avx512.h"
 
+#if defined(LIBXSMM_OFFLOAD_TARGET)
+# pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
+#endif
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
 #include <stdio.h>
+#if defined(LIBXSMM_OFFLOAD_TARGET)
+# pragma offload_attribute(pop)
+#endif
 
 /* @TODO change int based architecture value */
-LIBXSMM_INTERNAL_API_DEFINITION
+LIBXSMM_API
 void libxsmm_generator_convolution_forward_kernel( libxsmm_generated_code*                        io_generated_code,
                                                    const libxsmm_convolution_forward_descriptor*  i_conv_desc,
                                                    const char*                                    i_arch ) {
@@ -55,19 +61,22 @@ void libxsmm_generator_convolution_forward_kernel( libxsmm_generated_code*
   /* select datatype */
   if ( i_conv_desc->datatype == LIBXSMM_DNN_DATATYPE_F32 && i_conv_desc->datatype_itm == LIBXSMM_DNN_DATATYPE_F32 ) {
     if ( (strcmp(i_arch, "knl") == 0) ||
-         (strcmp(i_arch, "skx") == 0)    ) {
+         (strcmp(i_arch, "knm") == 0) ||
+         (strcmp(i_arch, "skx") == 0) ) {
       libxsmm_generator_convolution_forward_avx512_kernel( io_generated_code, i_conv_desc, i_arch );
     } else if ( (strcmp(i_arch, "hsw") == 0) ) {
       libxsmm_generator_convolution_forward_avx2_kernel( io_generated_code, i_conv_desc, i_arch );
     } else {
-      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_ARCH );
+      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_ARCH );
       return;
     }
   } else if ( i_conv_desc->datatype == LIBXSMM_DNN_DATATYPE_I16 && i_conv_desc->datatype_itm == LIBXSMM_DNN_DATATYPE_I32 ) {
-    if ( (strcmp(i_arch, "skx") == 0) ) {
+    if ( (strcmp(i_arch, "skx") == 0) ||
+         (strcmp(i_arch, "knm") == 0) ) {
+      /* call actual kernel generation with revised parameters */
       libxsmm_generator_convolution_forward_avx512_kernel( io_generated_code, i_conv_desc, i_arch );
     } else {
-      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_ARCH );
+      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_ARCH );
       return;
     }
   } else if ( (i_conv_desc->datatype == LIBXSMM_DNN_DATATYPE_I8  && i_conv_desc->datatype_itm == LIBXSMM_DNN_DATATYPE_I16
@@ -75,7 +84,7 @@ void libxsmm_generator_convolution_forward_kernel( libxsmm_generated_code*
     if ( (strcmp(i_arch, "skx") == 0) ) {
       libxsmm_generator_convolution_forward_avx512_kernel( io_generated_code, i_conv_desc, i_arch );
     } else {
-      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_ARCH );
+      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_ARCH );
       return;
     }
   } else if ( (i_conv_desc->datatype == LIBXSMM_DNN_DATATYPE_I8  && i_conv_desc->datatype_itm == LIBXSMM_DNN_DATATYPE_I32
@@ -83,12 +92,12 @@ void libxsmm_generator_convolution_forward_kernel( libxsmm_generated_code*
     if ( (strcmp(i_arch, "skx") == 0) ) {
       libxsmm_generator_convolution_forward_avx512_kernel( io_generated_code, i_conv_desc, i_arch );
     } else {
-      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_ARCH );
+      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_ARCH );
       return;
     }
   } else {
     /* TODO fix this error */
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_ARCH );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_ARCH );
     return;
   }
 
@@ -96,8 +105,9 @@ void libxsmm_generator_convolution_forward_kernel( libxsmm_generated_code*
   libxsmm_generator_isa_check_footer( io_generated_code, i_arch );
 }
 
+
 /* @TODO change int based architecture value */
-LIBXSMM_INTERNAL_API_DEFINITION
+LIBXSMM_API
 void libxsmm_generator_convolution_backward_kernel( libxsmm_generated_code*                        io_generated_code,
                                                     const libxsmm_convolution_backward_descriptor* i_conv_desc,
                                                     const char*                                    i_arch ) {
@@ -107,22 +117,18 @@ void libxsmm_generator_convolution_backward_kernel( libxsmm_generated_code*
   /* select datatype */
   if ( i_conv_desc->datatype == LIBXSMM_DNN_DATATYPE_F32 ) {
     if ( (strcmp(i_arch, "knl") == 0) ||
-         (strcmp(i_arch, "skx") == 0)    ) {
-      if ( ((i_conv_desc->format & LIBXSMM_DNN_TENSOR_FORMAT_NHWC) > 0) ||
-           ((i_conv_desc->format & LIBXSMM_DNN_TENSOR_FORMAT_RSCK) > 0) ) {
-        libxsmm_generator_convolution_backward_avx2_kernel( io_generated_code, i_conv_desc, i_arch );
-      } else {
-        libxsmm_generator_convolution_backward_avx512_kernel( io_generated_code, i_conv_desc, i_arch );
-      }
+         (strcmp(i_arch, "knm") == 0) ||
+         (strcmp(i_arch, "skx") == 0) ) {
+      libxsmm_generator_convolution_backward_avx512_kernel( io_generated_code, i_conv_desc, i_arch );
     } else if ( (strcmp(i_arch, "hsw") == 0) ) {
       libxsmm_generator_convolution_backward_avx2_kernel( io_generated_code, i_conv_desc, i_arch );
     } else {
-      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_ARCH );
+      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_ARCH );
       return;
     }
   } else {
     /* TODO fix this error */
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_ARCH );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_ARCH );
     return;
   }
 
@@ -132,7 +138,7 @@ void libxsmm_generator_convolution_backward_kernel( libxsmm_generated_code*
 
 
 /* @TODO change int based architecture value */
-LIBXSMM_INTERNAL_API_DEFINITION
+LIBXSMM_API
 void libxsmm_generator_convolution_weight_update_kernel( libxsmm_generated_code*                        io_generated_code,
                                                    const libxsmm_convolution_weight_update_descriptor*  i_conv_desc,
                                                    const char*                                    i_arch ) {
@@ -142,9 +148,10 @@ void libxsmm_generator_convolution_weight_update_kernel( libxsmm_generated_code*
   /* select datatype */
   if ( i_conv_desc->datatype == LIBXSMM_DNN_DATATYPE_F32 && i_conv_desc->datatype_itm == LIBXSMM_DNN_DATATYPE_F32 ) {
     if ( (strcmp(i_arch, "knl") == 0) ||
-         (strcmp(i_arch, "skx") == 0)    ) {
-      if ( ((i_conv_desc->format & LIBXSMM_DNN_TENSOR_FORMAT_NHWC) > 0) ||
-           ((i_conv_desc->format & LIBXSMM_DNN_TENSOR_FORMAT_RSCK) > 0) ) {
+         (strcmp(i_arch, "knm") == 0) ||
+         (strcmp(i_arch, "skx") == 0) ) {
+      if ( 0/*((i_conv_desc->format & LIBXSMM_DNN_TENSOR_FORMAT_NHWC) > 0) ||*/
+           /*((i_conv_desc->format & LIBXSMM_DNN_TENSOR_FORMAT_RSCK) > 0)*/ ) {
         libxsmm_generator_convolution_weight_update_avx2_kernel( io_generated_code, i_conv_desc, i_arch );
       } else {
         libxsmm_generator_convolution_weight_update_avx512_kernel( io_generated_code, i_conv_desc, i_arch );
@@ -152,12 +159,12 @@ void libxsmm_generator_convolution_weight_update_kernel( libxsmm_generated_code*
     } else if ( (strcmp(i_arch, "hsw") == 0) ) {
       libxsmm_generator_convolution_weight_update_avx2_kernel( io_generated_code, i_conv_desc, i_arch );
     } else {
-      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_ARCH );
+      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_ARCH );
       return;
     }
   } else {
     /* TODO fix this error */
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_ARCH );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_ARCH );
     return;
   }
 
@@ -165,7 +172,7 @@ void libxsmm_generator_convolution_weight_update_kernel( libxsmm_generated_code*
   libxsmm_generator_isa_check_footer( io_generated_code, i_arch );
 }
 
-LIBXSMM_INTERNAL_API_DEFINITION
+LIBXSMM_API
 void libxsmm_generator_convolution_forward_inlineasm( const char*                                   i_file_out,
                                                       const char*                                   i_routine_name,
                                                       const libxsmm_convolution_forward_descriptor* i_conv_desc,
@@ -184,7 +191,7 @@ void libxsmm_generator_convolution_forward_inlineasm( const char*
   } else if ( i_conv_desc->datatype == LIBXSMM_DNN_DATATYPE_I16 && i_conv_desc->datatype_itm == LIBXSMM_DNN_DATATYPE_I32 ) {
     libxsmm_convfunction_signature_int16( &l_generated_code, i_routine_name );
   } else {
-    fprintf(stderr, "LIBXSMM ERROR : inline assembly for convolutions is only supported for FP32 and int16!\n");
+    fprintf(stderr, "LIBXSMM error: inline assembly for convolutions is only supported for FP32 and int16!\n");
     return;
   }
 
@@ -196,8 +203,7 @@ void libxsmm_generator_convolution_forward_inlineasm( const char*
 
   /* check for errors during code generation */
   if ( l_generated_code.last_error != 0 ) {
-    fprintf(stderr, "LIBXSMM ERROR there was an error generating code. Last known error is:\n%s\n",
-      libxsmm_strerror(l_generated_code.last_error));
+    LIBXSMM_HANDLE_ERROR_VERBOSE( &l_generated_code, l_generated_code.last_error );
     return;
   }
 
@@ -218,7 +224,7 @@ void libxsmm_generator_convolution_forward_inlineasm( const char*
   free( l_generated_code.generated_code );
 }
 
-LIBXSMM_INTERNAL_API_DEFINITION
+LIBXSMM_API
 void libxsmm_generator_convolution_forward_directasm( const char*                                   i_file_out,
                                                       const char*                                   i_routine_name,
                                                       const libxsmm_convolution_forward_descriptor* i_conv_desc,
@@ -243,7 +249,7 @@ void libxsmm_generator_convolution_forward_directasm( const char*
   } else if ( i_conv_desc->datatype == LIBXSMM_DNN_DATATYPE_I16 && i_conv_desc->datatype_itm == LIBXSMM_DNN_DATATYPE_I32 ) {
     libxsmm_convfunction_signature_int16( &l_generated_code, i_routine_name );
   } else {
-    fprintf(stderr, "LIBXSMM ERROR : inline assembly for convolutions is only supported for FP32 and int16!\n");
+    fprintf(stderr, "LIBXSMM error: inline assembly for convolutions is only supported for FP32 and int16!\n");
     return;
   }
 
@@ -252,8 +258,7 @@ void libxsmm_generator_convolution_forward_directasm( const char*
 
   /* check for errors during code generation */
   if ( l_generated_code.last_error != 0 ) {
-    fprintf(stderr, "LIBXSMM ERROR there was an error generating code. Last known error is:\n%s\n",
-      libxsmm_strerror(l_generated_code.last_error));
+    LIBXSMM_HANDLE_ERROR_VERBOSE( &l_generated_code, l_generated_code.last_error );
     exit(-1);
   }
 
