@@ -26,25 +26,31 @@
 ** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
 ** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
 ******************************************************************************/
-/* Rajkishore Barik (Intel Corp.), Alexander Heinecke (Intel Corp.)
+/* Rajkishore Barik, Alexander Heinecke (Intel Corp.)
 ******************************************************************************/
 
 #include "generator_convolution_backward_avx2.h"
 #include "generator_convolution_common.h"
 #include "generator_x86_instructions.h"
 #include "generator_common.h"
-
 #include <libxsmm_intrinsics_x86.h>
+
+#if defined(LIBXSMM_OFFLOAD_TARGET)
+# pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
+#endif
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
+#if defined(LIBXSMM_OFFLOAD_TARGET)
+# pragma offload_attribute(pop)
+#endif
 
-LIBXSMM_INTERNAL_API_DEFINITION
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_backward_avx2_kernel( libxsmm_generated_code*                        io_generated_code,
                                                          const libxsmm_convolution_backward_descriptor* i_conv_desc,
                                                          const char*                                    i_arch ) {
-  /* code gen datastructures */
-  libxsmm_convolution_kernel_config l_conv_kernel_config = { 0/*avoid warning "maybe used uninitialized" */ };
+  /* code gen data structures */
+  libxsmm_convolution_kernel_config l_conv_kernel_config;
   libxsmm_convolution_backward_gp_reg_mapping l_gp_reg_mapping;
   libxsmm_loop_label_tracker l_loop_label_tracker;
 
@@ -83,10 +89,11 @@ void libxsmm_generator_convolution_backward_avx2_kernel( libxsmm_generated_code*
   libxsmm_generator_init_convolution_kernel_config( &l_conv_kernel_config );
   if ( strcmp( i_arch, "knl" ) == 0 ||
        strcmp( i_arch, "skx" ) == 0 ||
+       strcmp( i_arch, "knm" ) == 0 ||
        strcmp( i_arch, "hsw" ) == 0 ) {
     l_conv_kernel_config.instruction_set = LIBXSMM_X86_AVX2;
   } else {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_UNSUP_ARCH );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_UNSUP_ARCH );
     return;
   }
   l_conv_kernel_config.vector_reg_count = 16;
@@ -122,30 +129,29 @@ void libxsmm_generator_convolution_backward_avx2_kernel( libxsmm_generated_code*
     l_found_act_format = 1;
   }
   if ( (i_conv_desc->format & LIBXSMM_DNN_TENSOR_FORMAT_RSCK) > 0 ) {
+#if 0
+    /* copy of weights goes into LIBXSMM format when RSCK is used */
     l_conv_kernel_config.l_ld_ifm_fil = i_conv_desc->ifm_block * i_conv_desc->blocks_ifm;
     l_conv_kernel_config.l_ld_ofm_fil = i_conv_desc->ofm_block * i_conv_desc->blocks_ofm;
+#endif
+    l_conv_kernel_config.l_ld_ifm_fil = i_conv_desc->ifm_block;
+    l_conv_kernel_config.l_ld_ofm_fil = i_conv_desc->ofm_block;
     l_found_fil_format = 1;
   }
   if ( (l_found_act_format == 0) || (l_found_fil_format == 0) ) {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_UNSUP_CONV_FORMAT );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_UNSUP_CONV_FORMAT );
     return;
   }
 
   /* check if we have full vectors */
   if ( i_conv_desc->ifm_block % l_conv_kernel_config.vector_length_in != 0 ) {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_CONV_IFM_VEC );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_CONV_IFM_VEC );
     return;
   }
 
-  /* check if we have  stride of 1 */
-  if ( i_conv_desc->stride_h != 1 || i_conv_desc->stride_w != 1 ) {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_CONV_CONT_STRIDE );
-    return;
-  }
-
-  /* initilize KW and OFW unrolling */
+  /* initialize KW and OFW unrolling */
   if (i_conv_desc->unroll_kw != 0) {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_INVALID_KW_UNROLL );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_INVALID_KW_UNROLL );
     return;
   }
 
@@ -188,7 +194,7 @@ void libxsmm_generator_convolution_backward_avx2_kernel( libxsmm_generated_code*
 }
 
 
-LIBXSMM_INTERNAL_API_DEFINITION
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_backward_avx2_ofwloop( libxsmm_generated_code*                             io_generated_code,
                                                           const libxsmm_convolution_backward_gp_reg_mapping*  i_gp_reg_mapping,
                                                           const libxsmm_convolution_kernel_config*            i_conv_kernel_config,
@@ -230,7 +236,7 @@ void libxsmm_generator_convolution_backward_avx2_ofwloop( libxsmm_generated_code
                                         i_gp_reg_mapping->gp_reg_input,
                                         LIBXSMM_X86_GP_REG_UNDEF, 0,
                                         (l_m * i_conv_kernel_config->vector_length_in * i_conv_kernel_config->datatype_size_in) +
-                                          (l_n * i_conv_kernel_config->l_ld_ifm_act * i_conv_kernel_config->datatype_size_in),
+                                          (l_n * i_conv_kernel_config->l_ld_ifm_act * i_conv_desc->stride_w * i_conv_kernel_config->datatype_size_in),
                                         i_conv_kernel_config->vector_name,
                                         l_vec_reg_acc_start + l_m + (l_n * l_ifm_blocking), 0, 0 );
     }
@@ -253,7 +259,7 @@ void libxsmm_generator_convolution_backward_avx2_ofwloop( libxsmm_generated_code
                                         i_gp_reg_mapping->gp_reg_input,
                                         LIBXSMM_X86_GP_REG_UNDEF, 0,
                                         (l_m * i_conv_kernel_config->vector_length_in * i_conv_kernel_config->datatype_size_in) +
-                                          (l_n * i_conv_kernel_config->l_ld_ifm_act * i_conv_kernel_config->datatype_size_in),
+                                          (l_n * i_conv_kernel_config->l_ld_ifm_act * i_conv_desc->stride_w * i_conv_kernel_config->datatype_size_in),
                                         i_conv_kernel_config->vector_name,
                                         l_vec_reg_acc_start + l_m + (l_n * l_ifm_blocking), 0, 1 );
     }
@@ -292,7 +298,7 @@ void libxsmm_generator_convolution_backward_avx2_ofwloop( libxsmm_generated_code
     libxsmm_x86_instruction_alu_imm( io_generated_code,
                                      i_conv_kernel_config->alu_add_instruction,
                                      i_gp_reg_mapping->gp_reg_input,
-                                     i_ofw_rb * i_conv_kernel_config->l_ld_ifm_act * i_conv_kernel_config->datatype_size_in  );
+                                     i_ofw_rb * i_conv_kernel_config->l_ld_ifm_act * i_conv_desc->stride_w * i_conv_kernel_config->datatype_size_in  );
     /* advance output */
     libxsmm_x86_instruction_alu_imm( io_generated_code,
                                      i_conv_kernel_config->alu_add_instruction,
@@ -308,7 +314,7 @@ void libxsmm_generator_convolution_backward_avx2_ofwloop( libxsmm_generated_code
 }
 
 
-LIBXSMM_INTERNAL_API_DEFINITION
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_backward_avx2_ofmloop( libxsmm_generated_code*                             io_generated_code,
                                                           const libxsmm_convolution_backward_gp_reg_mapping*  i_gp_reg_mapping,
                                                           const libxsmm_convolution_kernel_config*            i_conv_kernel_config,
@@ -337,11 +343,11 @@ void libxsmm_generator_convolution_backward_avx2_ofmloop( libxsmm_generated_code
 
   /* Some checks */
   if ( i_conv_desc->ofh_rb != 1 ) {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_INVALID_OFH_UNROLL );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_INVALID_OFH_UNROLL );
     return;
   }
   if ( i_ofw_rb*l_ifm_blocking > i_conv_kernel_config->vector_reg_count-4 ) {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_INVALID_CONV_ACC );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_INVALID_CONV_ACC );
     return;
   }
 
