@@ -26,7 +26,7 @@
 ** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
 ** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
 ******************************************************************************/
-/* Alexander Heinecke (Intel Corp.) and Rajkishore Barik (Intel Corp.)
+/* Alexander Heinecke, Rajkishore Barik (Intel Corp.)
 ******************************************************************************/
 
 #ifndef GENERATOR_DENSE_COMMON_H
@@ -34,188 +34,227 @@
 
 #include "generator_common.h"
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
+void libxsmm_generator_convolution_header_m_loop( libxsmm_generated_code*                  io_generated_code,
+                                                 libxsmm_loop_label_tracker*               io_loop_label_tracker,
+                                                 const libxsmm_matcopy_kernel_config*      i_kernel_config,
+                                                 const unsigned int                        i_gp_reg_m_loop );
+
+LIBXSMM_API_INTERN
+void libxsmm_generator_convolution_footer_m_loop( libxsmm_generated_code*                      io_generated_code,
+                                                 libxsmm_loop_label_tracker*                   io_loop_label_tracker,
+                                                 const libxsmm_matcopy_kernel_config*          i_kernel_config,
+                                                 const unsigned int                            i_gp_reg_m_loop,
+                                                 const unsigned int                            i_m );
+
+LIBXSMM_API_INTERN
+void libxsmm_generator_convolution_header_n_loop( libxsmm_generated_code*                  io_generated_code,
+                                                 libxsmm_loop_label_tracker*               io_loop_label_tracker,
+                                                 const libxsmm_matcopy_kernel_config*      i_kernel_config,
+                                                 const unsigned int                        i_gp_reg_n_loop );
+
+LIBXSMM_API_INTERN
+void libxsmm_generator_convolution_footer_n_loop( libxsmm_generated_code*                      io_generated_code,
+                                                 libxsmm_loop_label_tracker*                   io_loop_label_tracker,
+                                                 const libxsmm_matcopy_kernel_config*          i_kernel_config,
+                                                 const unsigned int                            i_gp_reg_n_loop,
+                                                 const unsigned int                            i_n );
+
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_header_oi_loop( libxsmm_generated_code*                   io_generated_code,
                                                    libxsmm_loop_label_tracker*               io_loop_label_tracker,
                                                    const libxsmm_convolution_kernel_config*  i_conv_kernel_config,
                                                    const unsigned int                        i_gp_reg_oi_loop );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_footer_oi_loop( libxsmm_generated_code*                       io_generated_code,
                                                    libxsmm_loop_label_tracker*                   io_loop_label_tracker,
                                                    const libxsmm_convolution_kernel_config*      i_conv_kernel_config,
                                                    const unsigned int                            i_gp_reg_oi_loop,
                                                    const unsigned int                            i_oi );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_header_oj_loop( libxsmm_generated_code*                   io_generated_code,
                                                    libxsmm_loop_label_tracker*               io_loop_label_tracker,
                                                    const libxsmm_convolution_kernel_config*  i_conv_kernel_config,
                                                    const unsigned int                        i_gp_reg_oj_loop );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_footer_oj_loop( libxsmm_generated_code*                       io_generated_code,
                                                    libxsmm_loop_label_tracker*                   io_loop_label_tracker,
                                                    const libxsmm_convolution_kernel_config*      i_conv_kernel_config,
                                                    const unsigned int                            i_gp_reg_oj_loop,
                                                    const unsigned int                            i_oj );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_header_kh_loop( libxsmm_generated_code*                   io_generated_code,
                                                    libxsmm_loop_label_tracker*               io_loop_label_tracker,
                                                    const libxsmm_convolution_kernel_config*  i_conv_kernel_config,
                                                    const unsigned int                        i_gp_reg_kh_loop );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_footer_kh_loop( libxsmm_generated_code*                       io_generated_code,
                                                    libxsmm_loop_label_tracker*                   io_loop_label_tracker,
                                                    const libxsmm_convolution_kernel_config*      i_conv_kernel_config,
                                                    const unsigned int                            i_gp_reg_kh_loop,
                                                    const unsigned int                            i_kh );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_header_kw_loop( libxsmm_generated_code*                   io_generated_code,
                                                    libxsmm_loop_label_tracker*               io_loop_label_tracker,
                                                    const libxsmm_convolution_kernel_config*  i_conv_kernel_config,
                                                    const unsigned int                        i_gp_reg_kw_loop );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_footer_kw_loop( libxsmm_generated_code*                       io_generated_code,
                                                    libxsmm_loop_label_tracker*                   io_loop_label_tracker,
                                                    const libxsmm_convolution_kernel_config*      i_conv_kernel_config,
                                                    const unsigned int                            i_gp_reg_kw_loop,
                                                    const unsigned int                            i_kw );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
+void libxsmm_generator_convolution_header_ifmOuter_loop( libxsmm_generated_code*                   io_generated_code,
+                                                         libxsmm_loop_label_tracker*               io_loop_label_tracker,
+                                                         const libxsmm_convolution_kernel_config*  i_conv_kernel_config,
+                                                         const unsigned int                        i_gp_reg_ifmOuter_loop );
+
+LIBXSMM_API_INTERN
+void libxsmm_generator_convolution_footer_ifmOuter_loop( libxsmm_generated_code*                       io_generated_code,
+                                                         libxsmm_loop_label_tracker*                   io_loop_label_tracker,
+                                                         const libxsmm_convolution_kernel_config*      i_conv_kernel_config,
+                                                         const unsigned int                            i_gp_reg_ifmOuter_loop,
+                                                         const unsigned int                            i_ifmOuter_blocking );
+
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_header_ifm_loop( libxsmm_generated_code*                   io_generated_code,
                                                     libxsmm_loop_label_tracker*               io_loop_label_tracker,
                                                     const libxsmm_convolution_kernel_config*  i_conv_kernel_config,
                                                     const unsigned int                        i_gp_reg_ifmInner_loop,
                                                     const unsigned int                        i_unrolled_trips );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_footer_ifm_loop( libxsmm_generated_code*                   io_generated_code,
                                                     libxsmm_loop_label_tracker*               io_loop_label_tracker,
                                                     const libxsmm_convolution_kernel_config*  i_conv_kernel_config,
                                                     const unsigned int                        i_gp_reg_ifmInner_loop,
                                                     const unsigned int                        i_trip_count );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_forward_load_output( libxsmm_generated_code*                           io_generated_code,
                                                         const libxsmm_convolution_forward_gp_reg_mapping* i_gp_reg_mapping,
                                                         const libxsmm_convolution_kernel_config*          i_conv_kernel_config,
                                                         const libxsmm_convolution_forward_descriptor*     i_conv_desc );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_backward_load_input( libxsmm_generated_code*                           io_generated_code,
                                                         const libxsmm_convolution_backward_gp_reg_mapping* i_gp_reg_mapping,
                                                         const libxsmm_convolution_kernel_config*          i_conv_kernel_config,
                                                         const libxsmm_convolution_backward_descriptor*     i_conv_desc );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_weight_update_load_weight( libxsmm_generated_code*                           io_generated_code,
                                                         const libxsmm_convolution_weight_update_gp_reg_mapping* i_gp_reg_mapping,
                                                         const libxsmm_convolution_kernel_config*          i_conv_kernel_config,
                                                         const libxsmm_convolution_weight_update_descriptor*     i_conv_desc );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_forward_store_output( libxsmm_generated_code*                           io_generated_code,
                                                          const libxsmm_convolution_forward_gp_reg_mapping* i_gp_reg_mapping,
                                                          const libxsmm_convolution_kernel_config*          i_conv_kernel_config,
                                                          const libxsmm_convolution_forward_descriptor*     i_conv_desc );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_backward_store_input( libxsmm_generated_code*                           io_generated_code,
                                                          const libxsmm_convolution_backward_gp_reg_mapping* i_gp_reg_mapping,
                                                          const libxsmm_convolution_kernel_config*          i_conv_kernel_config,
                                                          const libxsmm_convolution_backward_descriptor*     i_conv_desc );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_weight_update_store_weight( libxsmm_generated_code*                           io_generated_code,
                                                          const libxsmm_convolution_weight_update_gp_reg_mapping* i_gp_reg_mapping,
                                                          const libxsmm_convolution_kernel_config*          i_conv_kernel_config,
                                                          const libxsmm_convolution_weight_update_descriptor*     i_conv_desc );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_weight_update_transpose_store_weight( libxsmm_generated_code*                           io_generated_code,
                                                          const libxsmm_convolution_weight_update_gp_reg_mapping* i_gp_reg_mapping,
                                                          const libxsmm_convolution_kernel_config*          i_conv_kernel_config,
                                                          const libxsmm_convolution_weight_update_descriptor*     i_conv_desc );
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_header_ofm_loop( libxsmm_generated_code*                   io_generated_code,
                                                     libxsmm_loop_label_tracker*               io_loop_label_tracker,
                                                     const libxsmm_convolution_kernel_config*  i_conv_kernel_config,
                                                     const unsigned int                        i_gp_reg_ofmInner_loop,
                                                     const unsigned int                        i_unrolled_trips );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_footer_ofm_loop( libxsmm_generated_code*                   io_generated_code,
                                                     libxsmm_loop_label_tracker*               io_loop_label_tracker,
                                                     const libxsmm_convolution_kernel_config*  i_conv_kernel_config,
                                                     const unsigned int                        i_gp_reg_ofmInner_loop,
                                                     const unsigned int                        i_trip_count ) ;
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_backward_load_weight( libxsmm_generated_code*                           io_generated_code,
                                                         const libxsmm_convolution_backward_gp_reg_mapping* i_gp_reg_mapping,
                                                         const libxsmm_convolution_kernel_config*          i_conv_kernel_config,
                                                         const libxsmm_convolution_backward_descriptor*     i_conv_desc ) ;
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_backward_fma( libxsmm_generated_code*                           io_generated_code,
                                                         const libxsmm_convolution_backward_gp_reg_mapping* i_gp_reg_mapping,
                                                         const libxsmm_convolution_kernel_config*          i_conv_kernel_config,
                                                         const libxsmm_convolution_backward_descriptor*     i_conv_desc );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_reset_x86_convolution_forward_gp_reg_mapping( libxsmm_convolution_forward_gp_reg_mapping* io_gp_reg_mapping );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_reset_x86_convolution_backward_gp_reg_mapping( libxsmm_convolution_backward_gp_reg_mapping* io_gp_reg_mapping );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_reset_x86_convolution_weight_update_gp_reg_mapping( libxsmm_convolution_weight_update_gp_reg_mapping* io_gp_reg_mapping );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_init_convolution_kernel_config( libxsmm_convolution_kernel_config* io_conv_kernel_config );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_weight_update_store_weight( libxsmm_generated_code*                                 io_generated_code,
                                                                const libxsmm_convolution_weight_update_gp_reg_mapping* i_gp_reg_mapping,
                                                                const libxsmm_convolution_kernel_config*                i_conv_kernel_config,
                                                                const libxsmm_convolution_weight_update_descriptor*                       i_conv_desc) ;
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_weight_update_load_weight( libxsmm_generated_code*                                 io_generated_code,
                                                               const libxsmm_convolution_weight_update_gp_reg_mapping* i_gp_reg_mapping,
                                                               const libxsmm_convolution_kernel_config*                i_conv_kernel_config,
                                                               const libxsmm_convolution_weight_update_descriptor*                       i_conv_desc) ;
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_weight_update_transpose_load_weight( libxsmm_generated_code*                                 io_generated_code,
                                                               const libxsmm_convolution_weight_update_gp_reg_mapping* i_gp_reg_mapping,
                                                               const libxsmm_convolution_kernel_config*                i_conv_kernel_config,
                                                               const libxsmm_convolution_weight_update_descriptor*                       i_conv_desc) ;
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_header_ofw_loop( libxsmm_generated_code*                  io_generated_code,
                                                     libxsmm_loop_label_tracker*              io_loop_label_tracker,
                                                     const libxsmm_convolution_kernel_config* i_conv_kernel_config,
                                                     const unsigned int                       i_gp_reg_oi_loop );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_footer_ofw_loop( libxsmm_generated_code*                  io_generated_code,
                                                     libxsmm_loop_label_tracker*              io_loop_label_tracker,
                                                     const libxsmm_convolution_kernel_config* i_conv_kernel_config,
                                                     const unsigned int                       i_gp_reg_oi_loop, const unsigned int                            i_ofw );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_header_ofh_loop( libxsmm_generated_code*                  io_generated_code,
                                                     libxsmm_loop_label_tracker*              io_loop_label_tracker,
                                                     const libxsmm_convolution_kernel_config* i_conv_kernel_config,
                                                     const unsigned int                       i_gp_reg_ofh_loop );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_footer_ofh_loop( libxsmm_generated_code*                  io_generated_code,
                                                     libxsmm_loop_label_tracker*              io_loop_label_tracker,
                                                     const libxsmm_convolution_kernel_config* i_conv_kernel_config,
