@@ -26,27 +26,33 @@
 ** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
 ** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
 ******************************************************************************/
-/* Rajkishore Barik (Intel Corp.), Alexander Heinecke (Intel Corp.)
+/* Rajkishore Barik, Alexander Heinecke (Intel Corp.)
 ******************************************************************************/
 
 #include "generator_convolution_weight_update_avx2.h"
 #include "generator_convolution_common.h"
 #include "generator_x86_instructions.h"
 #include "generator_common.h"
-
 #include <libxsmm_intrinsics_x86.h>
+
+#if defined(LIBXSMM_OFFLOAD_TARGET)
+# pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
+#endif
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
+#if defined(LIBXSMM_OFFLOAD_TARGET)
+# pragma offload_attribute(pop)
+#endif
 
-LIBXSMM_INTERNAL_API_DEFINITION
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_weight_update_avx2_kernel( libxsmm_generated_code*                             io_generated_code,
                                                               const libxsmm_convolution_weight_update_descriptor* i_conv_desc,
                                                               const char*                                         i_arch ) {
 /***** Code to generate
 Example for ofm_block = 32, ifm_block%2 == 0
 
-  for(ifm2 = 0; ifm2 < handle->ifmblock; ifm2+=2) {
+  for (ifm2 = 0; ifm2 < handle->ifmblock; ifm2+=2) {
     __m256 acc00 = _mm256_load_ps(&weight[ofm1][ifm1][kj][ki][ifm2][0]);
     __m256 acc01 = _mm256_load_ps(&weight[ofm1][ifm1][kj][ki][ifm2][8]);
     __m256 acc02 = _mm256_load_ps(&weight[ofm1][ifm1][kj][ki][ifm2][16]);
@@ -55,8 +61,8 @@ Example for ofm_block = 32, ifm_block%2 == 0
     __m256 acc11 = _mm256_load_ps(&weight[ofm1][ifm1][kj][ki][ifm2+1][8]);
     __m256 acc12 = _mm256_load_ps(&weight[ofm1][ifm1][kj][ki][ifm2+1][16]);
     __m256 acc13 = _mm256_load_ps(&weight[ofm1][ifm1][kj][ki][ifm2+1][24]);
-    for(ij=0, oj=0; oj < handle->ofh; ij+=stride_h, oj++) {
-      for(ii=0, oi=0; oi < handle->ofw; ii+=stride_w, oi++) {
+    for (ij=0, oj=0; oj < handle->ofh; ij+=stride_h, oj++) {
+      for (ii=0, oi=0; oi < handle->ofw; ii+=stride_w, oi++) {
         __m256 out0 = _mm256_load_ps(&output[img][ofm1][oj][oi][0]);
         __m256 out1 = _mm256_load_ps(&output[img][ofm1][oj][oi][8]);
         __m256 out2 = _mm256_load_ps(&output[img][ofm1][oj][oi][16]);
@@ -83,7 +89,7 @@ Example for ofm_block = 32, ifm_block%2 == 0
     _mm256_store_ps( &weight[ofm1][ifm1][kj][ki][ifm2+1][24], acc13 );
   }
 *****/
-  libxsmm_convolution_kernel_config l_conv_kernel_config = { 0 };
+  libxsmm_convolution_kernel_config l_conv_kernel_config;
   libxsmm_convolution_weight_update_gp_reg_mapping l_gp_reg_mapping;
   libxsmm_loop_label_tracker l_loop_label_tracker;
 
@@ -119,10 +125,11 @@ Example for ofm_block = 32, ifm_block%2 == 0
   libxsmm_generator_init_convolution_kernel_config( &l_conv_kernel_config );
   if ( strcmp( i_arch, "knl" ) == 0 ||
        strcmp( i_arch, "skx" ) == 0 ||
-       strcmp( i_arch, "hsw" ) == 0  ) {
+       strcmp( i_arch, "knm" ) == 0 ||
+       strcmp( i_arch, "hsw" ) == 0 ) {
     l_conv_kernel_config.instruction_set = LIBXSMM_X86_AVX2;
   } else {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_UNSUP_ARCH );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_UNSUP_ARCH );
     return;
   }
   l_conv_kernel_config.vector_reg_count = 16;
@@ -163,17 +170,17 @@ Example for ofm_block = 32, ifm_block%2 == 0
     l_found_fil_format = 1;
   }
   if ( (l_found_act_format == 0) || (l_found_fil_format == 0) ) {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_UNSUP_CONV_FORMAT );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_UNSUP_CONV_FORMAT );
     return;
   }
 
   /* check if we have full vectors */
   if ( i_conv_desc->ofm_block % l_conv_kernel_config.vector_length_out != 0 ) {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_CONV_OFM_VEC );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_CONV_OFM_VEC );
     return;
   }
 
-  /* caclulate the ifm unrolling */
+  /* calculate the ifm unrolling */
   for (l_m = 3; l_m > 0; l_m--) {
     if ( i_conv_desc->ifm_block%l_m == 0 ) {
       l_ifm_blocking = l_m;
@@ -187,7 +194,7 @@ Example for ofm_block = 32, ifm_block%2 == 0
 
   /* check accumulator size */
   if ( l_ofm_blocking*l_ifm_blocking > l_conv_kernel_config.vector_reg_count-4 ) {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_INVALID_CONV_ACC );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_INVALID_CONV_ACC );
     return;
   }
 
@@ -253,7 +260,7 @@ Example for ofm_block = 32, ifm_block%2 == 0
   libxsmm_x86_instruction_close_stream_convolution( io_generated_code, i_arch );
 }
 
-LIBXSMM_INTERNAL_API_DEFINITION
+LIBXSMM_API_INTERN
 void libxsmm_generator_convolution_weight_update_avx2_ofhofwloops( libxsmm_generated_code*                                 io_generated_code,
                                                                    libxsmm_loop_label_tracker*                             io_loop_label_tracker,
                                                                    const libxsmm_convolution_weight_update_gp_reg_mapping* i_gp_reg_mapping,
