@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2017, Intel Corporation                                **
+** Copyright (c) 2015-2018, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -29,14 +29,21 @@
 /* Alexander Heinecke (Intel Corp.)
 ******************************************************************************/
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
 #include "generator_gemm_avx2_microkernel.h"
 #include "generator_x86_instructions.h"
+#include "libxsmm_main.h"
 
-LIBXSMM_INTERNAL_API_DEFINITION
+#if defined(LIBXSMM_OFFLOAD_TARGET)
+# pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
+#endif
+#include <stdlib.h>
+#include <string.h>
+#include <stdio.h>
+#if defined(LIBXSMM_OFFLOAD_TARGET)
+# pragma offload_attribute(pop)
+#endif
+
+LIBXSMM_API_INTERN
 void libxsmm_generator_gemm_avx2_microkernel( libxsmm_generated_code*             io_generated_code,
                                                const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
                                                const libxsmm_micro_kernel_config* i_micro_kernel_config,
@@ -56,11 +63,11 @@ void libxsmm_generator_gemm_avx2_microkernel( libxsmm_generated_code*
 
 #if !defined(NDEBUG)
   if ( (i_n_blocking > 3) || (i_n_blocking < 1) ) {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_N_BLOCK );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_N_BLOCK );
     return;
   }
   if ( i_m_blocking % i_micro_kernel_config->vector_length != 0 ) {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_M_BLOCK );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_M_BLOCK );
     return;
   }
 #endif
@@ -151,7 +158,7 @@ void libxsmm_generator_gemm_avx2_microkernel( libxsmm_generated_code*
 
     if (l_m_blocking == 4) {
       /* load column vectors of A and multiply with all broadcasted row entries of B */
-      for ( l_m = 0; l_m < l_m_blocking ; l_m++ ) {
+      for ( l_m = 0; l_m < l_m_blocking; l_m++ ) {
         libxsmm_x86_instruction_vec_move( io_generated_code,
                                       i_micro_kernel_config->instruction_set,
                                       i_micro_kernel_config->a_vmove_instruction,
@@ -181,7 +188,7 @@ void libxsmm_generator_gemm_avx2_microkernel( libxsmm_generated_code*
       }
     } else {
       /* load column vectors of A and multiply with all broadcasted row entries of B */
-      for ( l_m = 0; l_m < l_m_blocking ; l_m++ ) {
+      for ( l_m = 0; l_m < l_m_blocking; l_m++ ) {
         libxsmm_x86_instruction_vec_move( io_generated_code,
                                       i_micro_kernel_config->instruction_set,
                                       i_micro_kernel_config->a_vmove_instruction,
@@ -191,7 +198,7 @@ void libxsmm_generator_gemm_avx2_microkernel( libxsmm_generated_code*
                                       i_micro_kernel_config->vector_name,
                                       i_n_blocking+l_m, i_micro_kernel_config->use_masking_a_c, 0 );
       }
-      for ( l_m = 0; l_m < l_m_blocking ; l_m++ ) {
+      for ( l_m = 0; l_m < l_m_blocking; l_m++ ) {
         for ( l_n = 0; l_n < i_n_blocking; l_n++ ) {
           /* post increment early */
           if ( (l_m == (l_m_blocking-1)) && (l_n == 0) ) {
