@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2017, Intel Corporation                                **
+** Copyright (c) 2015-2018, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -35,7 +35,7 @@
 #include "generator_common.h"
 #include "generator_gemm_common.h"
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_gemm_avx512_microkernel( libxsmm_generated_code*             io_generated_code,
                                                  const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
                                                  const libxsmm_micro_kernel_config* i_micro_kernel_config,
@@ -43,8 +43,16 @@ void libxsmm_generator_gemm_avx512_microkernel( libxsmm_generated_code*
                                                  const unsigned int                 i_n_blocking,
                                                  const unsigned int                 i_k_blocking );
 
+LIBXSMM_API_INTERN
+void libxsmm_generator_gemm_avx512_microkernel_qfma( libxsmm_generated_code*             io_generated_code,
+                                                     const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
+                                                     const libxsmm_micro_kernel_config* i_micro_kernel_config,
+                                                     const libxsmm_gemm_descriptor*     i_xgemm_desc,
+                                                     const unsigned int                 i_n_blocking,
+                                                     const unsigned int                 i_k_blocking );
+
 #if 0
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_gemm_avx512_microkernel_k_large( libxsmm_generated_code*             io_generated_code,
                                                          const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
                                                          const libxsmm_micro_kernel_config* i_micro_kernel_config,
@@ -53,14 +61,14 @@ void libxsmm_generator_gemm_avx512_microkernel_k_large( libxsmm_generated_code*
                                                          const unsigned int                 i_k_blocking );
 #endif
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_gemm_avx512_microkernel_k_large_n_nine( libxsmm_generated_code*             io_generated_code,
                                                                 const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
                                                                 const libxsmm_micro_kernel_config* i_micro_kernel_config,
                                                                 const libxsmm_gemm_descriptor*     i_xgemm_desc,
                                                                 const unsigned int                 i_k_blocking );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 unsigned int libxsmm_generator_gemm_avx512_kernel_kloop( libxsmm_generated_code*             io_generated_code,
                                                           libxsmm_loop_label_tracker*        io_loop_label_tracker,
                                                           const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
