@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2017, Intel Corporation                                **
+** Copyright (c) 2015-2018, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -28,20 +28,26 @@
 ******************************************************************************/
 /* Alexander Heinecke (Intel Corp.)
 ******************************************************************************/
-
 #include "generator_gemm_imci_avx512.h"
 #include "generator_gemm_imci_microkernel.h"
 #include "generator_gemm_avx512_microkernel.h"
 #include "generator_gemm_common.h"
 #include "generator_x86_instructions.h"
 #include "generator_common.h"
+#include "libxsmm_main.h"
 
-#include <libxsmm_intrinsics_x86.h>
+#if defined(LIBXSMM_OFFLOAD_TARGET)
+# pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
+#endif
 #include <string.h>
 #include <stdlib.h>
+#include <assert.h>
 #include <stdio.h>
+#if defined(LIBXSMM_OFFLOAD_TARGET)
+# pragma offload_attribute(pop)
+#endif
 
-LIBXSMM_INLINE
+LIBXSMM_API_INLINE
 void libxsmm_generator_gemm_imci_avx512_kernel_initialize_mask( libxsmm_generated_code*            io_generated_code,
                                                                 const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
                                                                 const libxsmm_micro_kernel_config* i_micro_kernel_config,
@@ -50,7 +56,7 @@ void libxsmm_generator_gemm_imci_avx512_kernel_initialize_mask( libxsmm_generate
   unsigned int l_mask;
 
   /* init full mask */
-  if ( (LIBXSMM_GEMM_FLAG_F32PREC & i_xgemm_desc->flags) == 0 ) {
+  if ( LIBXSMM_GEMM_PRECISION_F64 == LIBXSMM_GETENUM_INP( i_xgemm_desc->datatype )  ) {
     l_mask = 0xff;
   } else {
     l_mask = 0xffff;
@@ -69,8 +75,10 @@ void libxsmm_generator_gemm_imci_avx512_kernel_initialize_mask( libxsmm_generate
                                    LIBXSMM_X86_INSTR_KMOV,
                                    i_gp_reg_mapping->gp_reg_help_5,
                                    LIBXSMM_X86_IMCI_AVX512_MASK );
-  } else if ( i_micro_kernel_config->instruction_set == LIBXSMM_X86_AVX512_MIC ||
-              i_micro_kernel_config->instruction_set == LIBXSMM_X86_AVX512_CORE   ) {
+  } else if ( i_micro_kernel_config->instruction_set == LIBXSMM_X86_AVX512_MIC  ||
+              i_micro_kernel_config->instruction_set == LIBXSMM_X86_AVX512_KNM  ||
+              i_micro_kernel_config->instruction_set == LIBXSMM_X86_AVX512_CORE ||
+              i_micro_kernel_config->instruction_set == LIBXSMM_X86_AVX512_ICL     ) {
     libxsmm_x86_instruction_mask_move( io_generated_code,
                                    LIBXSMM_X86_INSTR_KMOVW,
                                    i_gp_reg_mapping->gp_reg_help_5,
@@ -78,7 +86,7 @@ void libxsmm_generator_gemm_imci_avx512_kernel_initialize_mask( libxsmm_generate
   } else {}
 }
 
-LIBXSMM_INLINE
+LIBXSMM_API_INLINE
 void libxsmm_generator_gemm_imci_avx512_kernel_mloop( libxsmm_generated_code*            io_generated_code,
                                                       libxsmm_loop_label_tracker*        io_loop_label_tracker,
                                                       const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
@@ -104,16 +112,25 @@ void libxsmm_generator_gemm_imci_avx512_kernel_mloop( libxsmm_generated_code*
     l_generator_microkernel_kloop = libxsmm_generator_gemm_avx512_kernel_kloop;
     l_generator_load = libxsmm_generator_gemm_load_C;
     l_generator_store = libxsmm_generator_gemm_store_C;
+  } else if ( (strcmp(i_arch, "knm") == 0) ) {
+    l_generator_microkernel_kloop = libxsmm_generator_gemm_avx512_kernel_kloop;
+    l_generator_load = libxsmm_generator_gemm_load_C;
+    l_generator_store = libxsmm_generator_gemm_store_C;
+  } else if ( (strcmp(i_arch, "icl") == 0) ) {
+    l_generator_microkernel_kloop = libxsmm_generator_gemm_avx512_kernel_kloop;
+    l_generator_load = libxsmm_generator_gemm_load_C;
+    l_generator_store = libxsmm_generator_gemm_store_C;
   } else if ( (strcmp(i_arch, "knc") == 0) ) {
     l_generator_microkernel_kloop = libxsmm_generator_gemm_imci_kernel_kloop;
     l_generator_load = libxsmm_generator_gemm_load_C_imci;
     l_generator_store = libxsmm_generator_gemm_store_C_imci;
   } else {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_ARCH );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_ARCH );
     return;
   }
 
-  /* we proceed as much as we can in vector length steps, remainder is handled uisng masking */
+  /* we proceed as much as we can in vector length steps, remainder is handled using masking */
+  assert(0 < i_micro_kernel_config->vector_length);
   l_m_done = (i_xgemm_desc->m / i_micro_kernel_config->vector_length) * i_micro_kernel_config->vector_length;
 
   /* multiples of vector_length in M */
@@ -122,14 +139,10 @@ void libxsmm_generator_gemm_imci_avx512_kernel_mloop( libxsmm_generated_code*
     l_generator_load( io_generated_code, i_gp_reg_mapping, i_micro_kernel_config,
                       i_xgemm_desc, i_micro_kernel_config->vector_length, i_n_blocking );
 
-    /* if we are generating for KNL && i_n_blocking is greater 26 && we prefetch via C -> push prefetch gpr */
-    if ( (i_n_blocking > 26)          &&
-         (strcmp(i_arch, "knc") != 0) &&
-         (i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_BL2_VIA_C ||
-          i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C ||
-          i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C_AHEAD ||
-          i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C_JPST ) ) {
-      libxsmm_x86_instruction_push_reg( io_generated_code, i_gp_reg_mapping->gp_reg_b_prefetch );
+    /* if we are generating for KNL && i_n_blocking is greater 18  -> push prefetch gpr */
+    if ( (i_n_blocking > 18)          &&
+         (strcmp(i_arch, "knc") != 0) ) {
+      libxsmm_x86_instruction_push_reg( io_generated_code, i_gp_reg_mapping->gp_reg_c );
     }
 
     l_k_unrolled = l_generator_microkernel_kloop( io_generated_code,
@@ -140,14 +153,14 @@ void libxsmm_generator_gemm_imci_avx512_kernel_mloop( libxsmm_generated_code*
                                                   i_arch,
                                                   i_n_blocking );
 
-    /* if we are generating for KNL && i_n_blocking is greater 26 && we prefetch via C -> push prefetch gpr */
-    if ( (i_n_blocking > 26)          &&
-         (strcmp(i_arch, "knc") != 0) &&
-         (i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_BL2_VIA_C ||
-          i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C ||
-          i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C_AHEAD ||
-          i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C_JPST ) ) {
-      libxsmm_x86_instruction_pop_reg( io_generated_code, i_gp_reg_mapping->gp_reg_b_prefetch );
+    if (0 != io_generated_code->last_error) {
+      return; /* propagate error */
+    }
+
+    /* if we are generating for KNL && i_n_blocking is greater 18  -> pop prefetch gpr */
+    if ( (i_n_blocking > 18)          &&
+         (strcmp(i_arch, "knc") != 0) ) {
+      libxsmm_x86_instruction_pop_reg( io_generated_code, i_gp_reg_mapping->gp_reg_c );
     }
 
     l_generator_store( io_generated_code, i_gp_reg_mapping, i_micro_kernel_config,
@@ -158,7 +171,7 @@ void libxsmm_generator_gemm_imci_avx512_kernel_mloop( libxsmm_generated_code*
 
   /* Remainder Handling using Masking, we are using M loop counter register as GP register for the mask */
   if ( l_m_done != (unsigned int)i_xgemm_desc->m ) {
-    /* request masking support, @TODO performance penality here, as a new object is created */
+    /* request masking support, @TODO performance penalty here, as a new object is created */
     libxsmm_micro_kernel_config l_micro_kernel_config_mask;
     libxsmm_generator_gemm_init_micro_kernel_config_fullvector( &l_micro_kernel_config_mask, i_xgemm_desc, i_arch, 1 );
 
@@ -173,14 +186,10 @@ void libxsmm_generator_gemm_imci_avx512_kernel_mloop( libxsmm_generated_code*
     l_generator_load( io_generated_code, i_gp_reg_mapping, &l_micro_kernel_config_mask,
                       i_xgemm_desc, l_micro_kernel_config_mask.vector_length, i_n_blocking );
 
-    /* if we are generating for KNL && i_n_blocking is greater 26 && we prefetch via C -> push prefetch gpr */
-    if ( (i_n_blocking > 26)          &&
-         (strcmp(i_arch, "knc") != 0) &&
-         (i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_BL2_VIA_C ||
-          i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C ||
-          i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C_AHEAD ||
-          i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C_JPST ) ) {
-      libxsmm_x86_instruction_push_reg( io_generated_code, i_gp_reg_mapping->gp_reg_b_prefetch );
+    /* if we are generating for KNL && i_n_blocking is greater 18  -> push prefetch gpr */
+    if ( (i_n_blocking > 18)          &&
+         (strcmp(i_arch, "knc") != 0) ) {
+      libxsmm_x86_instruction_push_reg( io_generated_code, i_gp_reg_mapping->gp_reg_c );
     }
 
     l_k_unrolled = l_generator_microkernel_kloop( io_generated_code,
@@ -191,14 +200,10 @@ void libxsmm_generator_gemm_imci_avx512_kernel_mloop( libxsmm_generated_code*
                                                   i_arch,
                                                   i_n_blocking );
 
-    /* if we are generating for KNL && i_n_blocking is greater 26 && we prefetch via C -> push prefetch gpr */
-    if ( (i_n_blocking > 26)          &&
-         (strcmp(i_arch, "knc") != 0) &&
-         (i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_BL2_VIA_C ||
-          i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C ||
-          i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C_AHEAD ||
-          i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C_JPST ) ) {
-      libxsmm_x86_instruction_pop_reg( io_generated_code, i_gp_reg_mapping->gp_reg_b_prefetch );
+    /* if we are generating for KNL && i_n_blocking is greater 18  -> push prefetch gpr */
+    if ( (i_n_blocking > 18)          &&
+         (strcmp(i_arch, "knc") != 0) ) {
+      libxsmm_x86_instruction_pop_reg( io_generated_code, i_gp_reg_mapping->gp_reg_c );
     }
 
     l_generator_store( io_generated_code, i_gp_reg_mapping, &l_micro_kernel_config_mask,
@@ -225,7 +230,7 @@ void libxsmm_generator_gemm_imci_avx512_kernel_mloop( libxsmm_generated_code*
   }
 }
 
-LIBXSMM_INTERNAL_API_DEFINITION
+LIBXSMM_API_INTERN
 void libxsmm_generator_gemm_imci_avx512_kernel( libxsmm_generated_code*         io_generated_code,
                                                  const libxsmm_gemm_descriptor* i_xgemm_desc,
                                                  const char*                    i_arch ) {
@@ -233,7 +238,8 @@ void libxsmm_generator_gemm_imci_avx512_kernel( libxsmm_generated_code*
   libxsmm_loop_label_tracker l_loop_label_tracker;
   libxsmm_gp_reg_mapping l_gp_reg_mapping;
 
-  unsigned int l_number_of_chunks = 1+((i_xgemm_desc->n-1)/30);
+  unsigned int l_max_n_rb_block = ( (strcmp(i_arch, "knm") == 0) || ( (strcmp(i_arch, "skx") == 0) && (LIBXSMM_GEMM_PRECISION_I16 == LIBXSMM_GETENUM_INP( i_xgemm_desc->datatype ) ) ) ) ? 28 : 30;
+  unsigned int l_number_of_chunks = 1+((i_xgemm_desc->n-1)/l_max_n_rb_block);
   unsigned int l_modulo = i_xgemm_desc->n%l_number_of_chunks;
   unsigned int l_n2 = i_xgemm_desc->n/l_number_of_chunks;
   unsigned int l_n1 = l_n2 + 1;
@@ -243,21 +249,31 @@ void libxsmm_generator_gemm_imci_avx512_kernel( libxsmm_generated_code*
 
   /* define gp register mapping */
   libxsmm_reset_x86_gp_reg_mapping( &l_gp_reg_mapping );
-  /* matching calling convention on Linux */
+#if defined(_WIN32) || defined(__CYGWIN__)
+  l_gp_reg_mapping.gp_reg_a = LIBXSMM_X86_GP_REG_RCX;
+  l_gp_reg_mapping.gp_reg_b = LIBXSMM_X86_GP_REG_RDX;
+  l_gp_reg_mapping.gp_reg_c = LIBXSMM_X86_GP_REG_R8;
+  l_gp_reg_mapping.gp_reg_a_prefetch = LIBXSMM_X86_GP_REG_R9;
+  /* TODO: full support for Windows calling convention */
+  l_gp_reg_mapping.gp_reg_b_prefetch = LIBXSMM_X86_GP_REG_RDI;
+  l_gp_reg_mapping.gp_reg_c_prefetch = LIBXSMM_X86_GP_REG_RSI;
+#else /* match calling convention on Linux */
   l_gp_reg_mapping.gp_reg_a = LIBXSMM_X86_GP_REG_RDI;
   l_gp_reg_mapping.gp_reg_b = LIBXSMM_X86_GP_REG_RSI;
   l_gp_reg_mapping.gp_reg_c = LIBXSMM_X86_GP_REG_RDX;
   l_gp_reg_mapping.gp_reg_a_prefetch = LIBXSMM_X86_GP_REG_RCX;
   l_gp_reg_mapping.gp_reg_b_prefetch = LIBXSMM_X86_GP_REG_R8;
+  l_gp_reg_mapping.gp_reg_c_prefetch = LIBXSMM_X86_GP_REG_R9;
+#endif
   l_gp_reg_mapping.gp_reg_mloop = LIBXSMM_X86_GP_REG_R12;
   l_gp_reg_mapping.gp_reg_nloop = LIBXSMM_X86_GP_REG_R13;
   l_gp_reg_mapping.gp_reg_kloop = LIBXSMM_X86_GP_REG_R14;
   l_gp_reg_mapping.gp_reg_help_0 = LIBXSMM_X86_GP_REG_R15; /* masking */
   l_gp_reg_mapping.gp_reg_help_1 = LIBXSMM_X86_GP_REG_RAX; /* B stride helper */
   l_gp_reg_mapping.gp_reg_help_2 = LIBXSMM_X86_GP_REG_RBX; /* B stride helper */
-  l_gp_reg_mapping.gp_reg_help_3 = LIBXSMM_X86_GP_REG_R9;  /* B stride helper */
+  l_gp_reg_mapping.gp_reg_help_3 = LIBXSMM_X86_GP_REG_R11;  /* B stride helper */
   l_gp_reg_mapping.gp_reg_help_4 = LIBXSMM_X86_GP_REG_R10; /* B stride helper */
-  l_gp_reg_mapping.gp_reg_help_5 = LIBXSMM_X86_GP_REG_R11; /* B stride helper */
+  l_gp_reg_mapping.gp_reg_help_5 = LIBXSMM_X86_GP_REG_R9; /* B stride helper */
 
   /* define loop_label_tracker */
   libxsmm_reset_loop_label_tracker( &l_loop_label_tracker );
@@ -265,7 +281,7 @@ void libxsmm_generator_gemm_imci_avx512_kernel( libxsmm_generated_code*
   /* define the micro kernel code gen properties */
   libxsmm_generator_gemm_init_micro_kernel_config_fullvector( &l_micro_kernel_config, i_xgemm_desc, i_arch, 0 );
 
-  if (l_n1 > 30) l_n1 = 30; /* this just the case if i_xgemm_desc->n/l_number_of_chunks has no remainder */
+  if (l_n1 > l_max_n_rb_block) l_n1 = l_max_n_rb_block; /* this just the case if i_xgemm_desc->n/l_number_of_chunks has no remainder */
   for (l_chunk = 0; l_chunk < l_number_of_chunks; l_chunk++) {
     if (l_chunk < l_modulo) {
       l_N1 += l_n1;
