@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2017, Intel Corporation                                **
+** Copyright (c) 2015-2018, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -31,13 +31,19 @@
 
 #include "generator_gemm_imci_microkernel.h"
 #include "generator_x86_instructions.h"
-#include <libxsmm_intrinsics_x86.h>
+#include "libxsmm_main.h"
 
-#include <stdio.h>
+#if defined(LIBXSMM_OFFLOAD_TARGET)
+# pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
+#endif
 #include <stdlib.h>
 #include <string.h>
+#include <stdio.h>
+#if defined(LIBXSMM_OFFLOAD_TARGET)
+# pragma offload_attribute(pop)
+#endif
 
-LIBXSMM_INTERNAL_API_DEFINITION
+LIBXSMM_API_INTERN
 void libxsmm_generator_gemm_imci_microkernel( libxsmm_generated_code*             io_generated_code,
                                                const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
                                                const libxsmm_micro_kernel_config* i_micro_kernel_config,
@@ -51,11 +57,11 @@ void libxsmm_generator_gemm_imci_microkernel( libxsmm_generated_code*
 
 #if !defined(NDEBUG)
   if ( i_n_blocking > 30 ) {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_N_BLOCK );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_N_BLOCK );
     return;
   }
   if ( (i_offset >= 0) && (i_k_blocking != 1) ) {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_K_BLOCK );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_K_BLOCK );
     return;
   }
 #endif
@@ -157,7 +163,7 @@ void libxsmm_generator_gemm_imci_microkernel( libxsmm_generated_code*
   }
 }
 
-LIBXSMM_INTERNAL_API_DEFINITION
+LIBXSMM_API_INTERN
 unsigned int libxsmm_generator_gemm_imci_kernel_kloop( libxsmm_generated_code*             io_generated_code,
                                                         libxsmm_loop_label_tracker*        io_loop_label_tracker,
                                                         const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
@@ -243,7 +249,7 @@ unsigned int libxsmm_generator_gemm_imci_kernel_kloop( libxsmm_generated_code*
   return l_k_unrolled;
 }
 
-LIBXSMM_INTERNAL_API_DEFINITION
+LIBXSMM_API_INTERN
 void libxsmm_x86_instruction_vec_move_imci( libxsmm_generated_code* io_generated_code,
                                         const unsigned int      i_instruction_set,
                                         const unsigned int      i_vmove_instr,
@@ -256,7 +262,7 @@ void libxsmm_x86_instruction_vec_move_imci( libxsmm_generated_code* io_generated
                                         const unsigned int      i_use_masking,
                                         const unsigned int      i_is_store ) {
   if ( (i_vmove_instr == LIBXSMM_X86_INSTR_VMOVAPD) ||
-       (i_vmove_instr == LIBXSMM_X86_INSTR_VMOVAPS)    ) {
+       (i_vmove_instr == LIBXSMM_X86_INSTR_VMOVAPS) ) {
     libxsmm_x86_instruction_vec_move( io_generated_code, i_instruction_set, i_vmove_instr,
                                   i_gp_reg_base, i_gp_reg_idx, i_scale, i_displacement, i_vector_name, i_vec_reg_number_0, i_use_masking, i_is_store );
   } else if ( i_vmove_instr == LIBXSMM_X86_INSTR_VMOVUPD ) {
@@ -288,12 +294,12 @@ void libxsmm_x86_instruction_vec_move_imci( libxsmm_generated_code* io_generated
     libxsmm_x86_instruction_vec_move( io_generated_code, i_instruction_set, l_instr_2,
                                   i_gp_reg_base, i_gp_reg_idx, i_scale, i_displacement+64, i_vector_name, i_vec_reg_number_0, i_use_masking, i_is_store );
   } else {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_VEC_MOVE_IMCI );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_VEC_MOVE_IMCI );
     return;
   }
 }
 
-LIBXSMM_INTERNAL_API_DEFINITION
+LIBXSMM_API_INTERN
 void libxsmm_generator_gemm_load_C_imci( libxsmm_generated_code*             io_generated_code,
                                           const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
                                           const libxsmm_micro_kernel_config* i_micro_kernel_config,
@@ -314,15 +320,15 @@ void libxsmm_generator_gemm_load_C_imci( libxsmm_generated_code*             io_
      This is not done in release mode and therefore bad
      things might happen.... HUAAH */
   if (i_micro_kernel_config->instruction_set != LIBXSMM_X86_IMCI ) {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_NO_IMCI );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_NO_IMCI );
     return;
   }
   if ( (i_n_blocking > 30) || (i_n_blocking < 1) || (i_m_blocking != i_micro_kernel_config->vector_length) ) {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_REG_BLOCK );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_REG_BLOCK );
     return;
   }
   if ( i_m_blocking % i_micro_kernel_config->vector_length != 0 ) {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_M_BLOCK );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_M_BLOCK );
     return;
   }
 #endif /*NDEBUG*/
@@ -340,7 +346,7 @@ void libxsmm_generator_gemm_load_C_imci( libxsmm_generated_code*             io_
                                          i_micro_kernel_config->vector_name,
                                          l_vec_reg_acc_start + l_m + (l_m_blocking * l_n), i_micro_kernel_config->use_masking_a_c, 0 );
       if ( i_micro_kernel_config->c_vmove_instruction == LIBXSMM_X86_INSTR_VMOVAPD ||
-           i_micro_kernel_config->c_vmove_instruction == LIBXSMM_X86_INSTR_VMOVAPS    ) {
+           i_micro_kernel_config->c_vmove_instruction == LIBXSMM_X86_INSTR_VMOVAPS ) {
         libxsmm_x86_instruction_prefetch( io_generated_code,
                                       LIBXSMM_X86_INSTR_VPREFETCH1,
                                       i_gp_reg_mapping->gp_reg_c,
@@ -367,7 +373,7 @@ void libxsmm_generator_gemm_load_C_imci( libxsmm_generated_code*             io_
   }
 }
 
-LIBXSMM_INTERNAL_API_DEFINITION
+LIBXSMM_API_INTERN
 void libxsmm_generator_gemm_store_C_imci( libxsmm_generated_code*             io_generated_code,
                                            const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
                                            const libxsmm_micro_kernel_config* i_micro_kernel_config,
@@ -389,15 +395,15 @@ void libxsmm_generator_gemm_store_C_imci( libxsmm_generated_code*             io
      This is not done in release mode and therefore bad
      things might happen.... HUAAH */
   if (i_micro_kernel_config->instruction_set != LIBXSMM_X86_IMCI ) {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_NO_IMCI );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_NO_IMCI );
     return;
   }
   if ( (i_n_blocking > 30) || (i_n_blocking < 1) || (i_m_blocking != i_micro_kernel_config->vector_length) ) {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_REG_BLOCK );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_REG_BLOCK );
     return;
   }
   if ( i_m_blocking % i_micro_kernel_config->vector_length != 0 ) {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_M_BLOCK );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_M_BLOCK );
     return;
   }
 #endif
