@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2017, Intel Corporation                                **
+** Copyright (c) 2015-2018, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -35,7 +35,7 @@
 #include "generator_common.h"
 #include "generator_gemm_common.h"
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_gemm_imci_microkernel( libxsmm_generated_code*             io_generated_code,
                                                const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
                                                const libxsmm_micro_kernel_config* i_micro_kernel_config,
@@ -44,7 +44,7 @@ void libxsmm_generator_gemm_imci_microkernel( libxsmm_generated_code*
                                                const unsigned int                 i_k_blocking,
                                                const int                          i_offset );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 unsigned int libxsmm_generator_gemm_imci_kernel_kloop( libxsmm_generated_code*             io_generated_code,
                                                         libxsmm_loop_label_tracker*        io_loop_label_tracker,
                                                         const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
@@ -53,7 +53,7 @@ unsigned int libxsmm_generator_gemm_imci_kernel_kloop( libxsmm_generated_code*
                                                         const char*                        i_arch,
                                                         unsigned int                       i_n_blocking );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_gemm_load_C_imci( libxsmm_generated_code*             io_generated_code,
                                           const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
                                           const libxsmm_micro_kernel_config* i_micro_kernel_config,
@@ -61,7 +61,7 @@ void libxsmm_generator_gemm_load_C_imci( libxsmm_generated_code*             io_
                                           const unsigned int                 i_m_blocking,
                                           const unsigned int                 i_n_blocking );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_generator_gemm_store_C_imci( libxsmm_generated_code*             io_generated_code,
                                            const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
                                            const libxsmm_micro_kernel_config* i_micro_kernel_config,
@@ -69,7 +69,7 @@ void libxsmm_generator_gemm_store_C_imci( libxsmm_generated_code*             io
                                            const unsigned int                 i_m_blocking,
                                            const unsigned int                 i_n_blocking );
 
-LIBXSMM_INTERNAL_API
+LIBXSMM_API_INTERN
 void libxsmm_x86_instruction_vec_move_imci( libxsmm_generated_code* io_generated_code,
                                             const unsigned int      i_instruction_set,
                                             const unsigned int      i_vmove_instr,
