@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2017, Intel Corporation                                **
+** Copyright (c) 2015-2018, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -28,7 +28,6 @@
 ******************************************************************************/
 /* Alexander Heinecke (Intel Corp.)
 ******************************************************************************/
-#include <libxsmm_generator.h>
 #include "generator_common.h"
 #include "generator_spgemm_csc_reader.h"
 #include "generator_spgemm_csr_reader.h"
@@ -38,14 +37,21 @@
 #include "generator_spgemm_csr_asparse_reg.h"
 #include "generator_spgemm_csr_bsparse_soa.h"
 #include "generator_spgemm_csr_asparse_soa.h"
+#include "generator_spgemm_csc_bsparse_soa.h"
+#include "libxsmm_main.h"
 
+#if defined(LIBXSMM_OFFLOAD_TARGET)
+# pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
+#endif
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
 #include <stdio.h>
+#if defined(LIBXSMM_OFFLOAD_TARGET)
+# pragma offload_attribute(pop)
+#endif
 
-
-LIBXSMM_INTERNAL_API_DEFINITION
+LIBXSMM_API
 void libxsmm_generator_spgemm_csc_kernel( libxsmm_generated_code*        io_generated_code,
                                           const libxsmm_gemm_descriptor* i_xgemm_desc,
                                           const char*                    i_arch,
@@ -56,12 +62,12 @@ void libxsmm_generator_spgemm_csc_kernel( libxsmm_generated_code*        io_gene
   if ( (i_xgemm_desc->lda == 0) && (i_xgemm_desc->ldb > 0) && (i_xgemm_desc->ldc > 0) ) {
     /* check LDB */
     if ( i_xgemm_desc->ldb < i_xgemm_desc->k ) {
-      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_LDB );
+      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_LDB );
       return;
     }
     /* check LDC */
     if ( i_xgemm_desc->ldc < i_xgemm_desc->m ) {
-      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_LDC );
+      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_LDC );
       return;
     }
     libxsmm_generator_spgemm_csc_asparse( io_generated_code, i_xgemm_desc, i_arch, i_row_idx, i_column_idx, i_values );
@@ -69,23 +75,23 @@ void libxsmm_generator_spgemm_csc_kernel( libxsmm_generated_code*        io_gene
   } else if ( (i_xgemm_desc->lda > 0) && (i_xgemm_desc->ldb == 0) && (i_xgemm_desc->ldc > 0) ) {
     /* check LDA */
     if ( i_xgemm_desc->lda < i_xgemm_desc->m ) {
-      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_LDA );
+      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_LDA );
       return;
     }
     /* check LDC */
     if ( i_xgemm_desc->ldc < i_xgemm_desc->m ) {
-      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_LDC );
+      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_LDC );
       return;
     }
     libxsmm_generator_spgemm_csc_bsparse( io_generated_code, i_xgemm_desc, i_arch, i_row_idx, i_column_idx, i_values );
   } else {
     /* something bad happened... */
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_SPGEMM_GEN );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_SPGEMM_GEN );
     return;
   }
 }
 
-LIBXSMM_INTERNAL_API_DEFINITION
+LIBXSMM_API
 void libxsmm_generator_spgemm_csr_kernel( libxsmm_generated_code*        io_generated_code,
                                           const libxsmm_gemm_descriptor* i_xgemm_desc,
                                           const char*                    i_arch,
@@ -96,12 +102,12 @@ void libxsmm_generator_spgemm_csr_kernel( libxsmm_generated_code*        io_gene
   if ( (i_xgemm_desc->lda == 0) && (i_xgemm_desc->ldb > 0) && (i_xgemm_desc->ldc > 0) ) {
     /* check LDB */
     if ( i_xgemm_desc->ldb < i_xgemm_desc->n ) {
-      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_LDB );
+      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_LDB );
       return;
     }
     /* check LDC */
     if ( i_xgemm_desc->ldc < i_xgemm_desc->n ) {
-      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_LDC );
+      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_LDC );
       return;
     }
     libxsmm_generator_spgemm_csr_asparse( io_generated_code, i_xgemm_desc, i_arch, i_row_idx, i_column_idx, i_values );
@@ -109,25 +115,25 @@ void libxsmm_generator_spgemm_csr_kernel( libxsmm_generated_code*        io_gene
   } else if ( (i_xgemm_desc->lda > 0) && (i_xgemm_desc->ldb == 0) && (i_xgemm_desc->ldc > 0) ) {
     /* check LDA */
     if ( i_xgemm_desc->lda < i_xgemm_desc->k ) {
-      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_LDA );
+      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_LDA );
       return;
     }
     /* check LDC */
     if ( i_xgemm_desc->ldc < i_xgemm_desc->n ) {
-      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_LDC );
+      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_LDC );
       return;
     }
     /* something bad happened... */
-    fprintf( stderr, "LIBXSMM ERROR, B sparse for CSR datastructure is not yet available, FATAL ERROR, EXTI!\n");
+    fprintf(stderr, "LIBXSMM fatal error: B sparse for CSR data structure is not yet available!\n");
     exit(-1);
   } else {
     /* something bad happened... */
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_SPGEMM_GEN );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_SPGEMM_GEN );
     return;
   }
 }
 
-LIBXSMM_INTERNAL_API_DEFINITION
+LIBXSMM_API
 void libxsmm_generator_spgemm_csr_reg_kernel( libxsmm_generated_code*        io_generated_code,
                                               const libxsmm_gemm_descriptor* i_xgemm_desc,
                                               const char*                    i_arch,
@@ -138,12 +144,12 @@ void libxsmm_generator_spgemm_csr_reg_kernel( libxsmm_generated_code*        io_
   if ( (i_xgemm_desc->lda == 0) && (i_xgemm_desc->ldb > 0) && (i_xgemm_desc->ldc > 0) ) {
     /* check LDB */
     if ( i_xgemm_desc->ldb < i_xgemm_desc->n ) {
-      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_LDB );
+      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_LDB );
       return;
     }
     /* check LDC */
     if ( i_xgemm_desc->ldc < i_xgemm_desc->n ) {
-      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_LDC );
+      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_LDC );
       return;
     }
     libxsmm_generator_spgemm_csr_asparse_reg( io_generated_code, i_xgemm_desc, i_arch, i_row_idx, i_column_idx, i_values );
@@ -151,41 +157,41 @@ void libxsmm_generator_spgemm_csr_reg_kernel( libxsmm_generated_code*        io_
   } else if ( (i_xgemm_desc->lda > 0) && (i_xgemm_desc->ldb == 0) && (i_xgemm_desc->ldc > 0) ) {
     /* check LDA */
     if ( i_xgemm_desc->lda < i_xgemm_desc->k ) {
-      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_LDA );
+      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_LDA );
       return;
     }
     /* check LDC */
     if ( i_xgemm_desc->ldc < i_xgemm_desc->n ) {
-      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_LDC );
+      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_LDC );
       return;
     }
     /* something bad happened... */
-    fprintf( stderr, "LIBXSMM ERROR, B sparse for CSR datastructure is not yet available, FATAL ERROR, EXTI!\n");
+    fprintf(stderr, "LIBXSMM fatal error:B sparse for CSR data structure is not yet available!\n");
     exit(-1);
   } else {
     /* something bad happened... */
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_SPGEMM_GEN );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_SPGEMM_GEN );
     return;
   }
 }
 
-LIBXSMM_INTERNAL_API_DEFINITION
+LIBXSMM_API
 void libxsmm_generator_spgemm_csr_soa_kernel( libxsmm_generated_code*        io_generated_code,
                                               const libxsmm_gemm_descriptor* i_xgemm_desc,
                                               const char*                    i_arch,
                                               const unsigned int*            i_row_idx,
                                               const unsigned int*            i_column_idx,
-                                              const double*                  i_values ) {
+                                              const void*                    i_values ) {
   /* A matrix is sparse */
   if ( (i_xgemm_desc->lda == 0) && (i_xgemm_desc->ldb > 0) && (i_xgemm_desc->ldc > 0) ) {
     /* check LDB */
     if ( i_xgemm_desc->ldb < i_xgemm_desc->n ) {
-      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_LDB );
+      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_LDB );
       return;
     }
     /* check LDC */
     if ( i_xgemm_desc->ldc < i_xgemm_desc->n ) {
-      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_LDC );
+      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_LDC );
       return;
     }
     libxsmm_generator_spgemm_csr_asparse_soa( io_generated_code, i_xgemm_desc, i_arch, i_row_idx, i_column_idx, i_values );
@@ -193,29 +199,56 @@ void libxsmm_generator_spgemm_csr_soa_kernel( libxsmm_generated_code*        io_
   } else if ( (i_xgemm_desc->lda > 0) && (i_xgemm_desc->ldb == 0) && (i_xgemm_desc->ldc > 0) ) {
     /* check LDA */
     if ( i_xgemm_desc->lda < i_xgemm_desc->k ) {
-      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_LDA );
+      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_LDA );
       return;
     }
     /* check LDC */
     if ( i_xgemm_desc->ldc < i_xgemm_desc->n ) {
-      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_LDC );
+      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_LDC );
       return;
     }
     libxsmm_generator_spgemm_csr_bsparse_soa( io_generated_code, i_xgemm_desc, i_arch, i_row_idx, i_column_idx, i_values );
   } else {
     /* something bad happened... */
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_SPGEMM_GEN );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_SPGEMM_GEN );
     return;
   }
 }
 
-LIBXSMM_INTERNAL_API_DEFINITION
+LIBXSMM_API
+void libxsmm_generator_spgemm_csc_soa_kernel( libxsmm_generated_code*        io_generated_code,
+                                              const libxsmm_gemm_descriptor* i_xgemm_desc,
+                                              const char*                    i_arch,
+                                              const unsigned int*            i_row_idx,
+                                              const unsigned int*            i_column_idx,
+                                              const void*                    i_values ) {
+  /* B matrix is sparse */
+  if ( (i_xgemm_desc->lda > 0) && (i_xgemm_desc->ldb == 0) && (i_xgemm_desc->ldc > 0) ) {
+    /* check LDA */
+    if ( i_xgemm_desc->lda < i_xgemm_desc->k ) {
+      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_LDA );
+      return;
+    }
+    /* check LDC */
+    if ( i_xgemm_desc->ldc < i_xgemm_desc->n ) {
+      LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_LDC );
+      return;
+    }
+    libxsmm_generator_spgemm_csc_bsparse_soa( io_generated_code, i_xgemm_desc, i_arch, i_row_idx, i_column_idx, i_values );
+  } else {
+    /* something bad happened... */
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_SPGEMM_GEN );
+    return;
+  }
+}
+
+LIBXSMM_API
 void libxsmm_generator_spgemm( const char*                    i_file_out,
                                const char*                    i_routine_name,
                                const libxsmm_gemm_descriptor* i_xgemm_desc,
                                const char*                    i_arch,
                                const char*                    i_file_in,
-                               const int                      i_is_csr        ) {
+                               const int                      i_is_csr ) {
   /* CSC/CSR structure */
   unsigned int* l_row_idx = NULL;
   unsigned int* l_column_idx = NULL;
@@ -240,8 +273,9 @@ void libxsmm_generator_spgemm( const char*                    i_file_out,
   }
 
   /* check if generate to CSC */
-  if ( i_is_csr == 0 ) {
-    /* read CSC file and consturct CSC datastructure */
+  /* @TODO, this i_is_csr is very hacky.... change it in future */
+  if ( (i_is_csr == 0) || (i_is_csr > 9) ) {
+    /* read CSC file and construct CSC data structure */
     libxsmm_sparse_csc_reader( &l_generated_code, i_file_in, &l_row_idx, &l_column_idx, &l_values, &l_row_count, &l_column_count, &l_element_count );
 
 #if !defined(NDEBUG)
@@ -250,11 +284,14 @@ void libxsmm_generator_spgemm( const char*                    i_file_out,
       unsigned int l_n;
       unsigned int l_m;
 
+      /* mute static analysis about garbage content */
+      memset(l_tmp, 0, l_row_count * l_column_count * sizeof(double));
+
       printf("CSC matrix data structure we just read:\n");
       printf("rows: %u, columns: %u, elements: %u\n", l_row_count, l_column_count, l_element_count);
 
       if (l_tmp == NULL) {
-        fprintf( stderr, "LIBXSMM ERROR, Could allocate dense value array to test CSC datastructure!\n");
+        fprintf(stderr, "LIBXSMM fatal error:Could allocate dense value array to test CSC data structure!\n");
         exit(-1);
       }
 
@@ -284,6 +321,7 @@ void libxsmm_generator_spgemm( const char*                    i_file_out,
         }
       }
 
+      assert(0 != l_tmp);
       for ( l_n = 0; l_n < l_row_count; l_n++) {
         for ( l_m = 0; l_m < l_column_count; l_m++) {
           printf("%f ", l_tmp[(l_n * l_column_count) + l_m]);
@@ -295,9 +333,15 @@ void libxsmm_generator_spgemm( const char*                    i_file_out,
     }
 #endif
     /* generate the actual kernel code for current description depending on the architecture */
-    libxsmm_generator_spgemm_csc_kernel( &l_generated_code, i_xgemm_desc, i_arch, l_row_idx, l_column_idx, l_values );
+    if (i_is_csr == 0) {
+      libxsmm_generator_spgemm_csc_kernel( &l_generated_code, i_xgemm_desc, i_arch, l_row_idx, l_column_idx, l_values );
+    } else if (i_is_csr == 10) {
+      libxsmm_generator_spgemm_csc_soa_kernel( &l_generated_code, i_xgemm_desc, i_arch, l_row_idx, l_column_idx, l_values );
+    } else {
+      assert(0/*should not happen*/);
+    }
   } else {
-    /* read CSR file and consturct CSR datastructure */
+    /* read CSR file and construct CSR data structure */
     libxsmm_sparse_csr_reader( &l_generated_code, i_file_in, &l_row_idx, &l_column_idx, &l_values, &l_row_count, &l_column_count, &l_element_count );
 
 #if !defined(NDEBUG)
@@ -306,11 +350,14 @@ void libxsmm_generator_spgemm( const char*                    i_file_out,
       unsigned int l_n;
       unsigned int l_m;
 
+      /* mute static analysis about garbage content */
+      memset(l_tmp, 0, l_row_count * l_column_count * sizeof(double));
+
       printf("CSR matrix data structure we just read:\n");
       printf("rows: %u, columns: %u, elements: %u\n", l_row_count, l_column_count, l_element_count);
 
       if (l_tmp == NULL) {
-        fprintf( stderr, "LIBXSMM ERROR, Could allocate dense value array to test CSR datastructure!\n");
+        fprintf(stderr, "LIBXSMM fatal error:Could allocate dense value array to test CSR data structure!\n");
         exit(-1);
       }
 
@@ -340,6 +387,7 @@ void libxsmm_generator_spgemm( const char*                    i_file_out,
         }
       }
 
+      assert(0 != l_tmp);
       for ( l_n = 0; l_n < l_row_count; l_n++) {
         for ( l_m = 0; l_m < l_column_count; l_m++) {
           printf("%f ", l_tmp[(l_n * l_column_count) + l_m]);
@@ -360,7 +408,7 @@ void libxsmm_generator_spgemm( const char*                    i_file_out,
       /* generate the actual kernel code for current description depending on the architecture */
       libxsmm_generator_spgemm_csr_reg_kernel( &l_generated_code, i_xgemm_desc, i_arch, l_row_idx, l_column_idx, l_values );
     } else {
-      /* shouldn't happen */
+      assert(0/*should not happen*/);
     }
   }
 
@@ -380,8 +428,7 @@ void libxsmm_generator_spgemm( const char*                    i_file_out,
 
   /* check for errors during code generation */
   if ( l_generated_code.last_error != 0 ) {
-    fprintf(stderr, "LIBXSMM ERROR there was an error generating code. Last known error is:\n%s\n",
-      libxsmm_strerror(l_generated_code.last_error));
+    LIBXSMM_HANDLE_ERROR_VERBOSE( &l_generated_code, l_generated_code.last_error );
     exit(-1);
   }
 
@@ -393,7 +440,7 @@ void libxsmm_generator_spgemm( const char*                    i_file_out,
       fputs( (const char*)l_generated_code.generated_code, l_file_handle );
       fclose( l_file_handle );
     } else {
-      fprintf(stderr, "LIBXSMM ERROR, libxsmm_generator_spgemm: could not write to into destination source file\n");
+      fprintf(stderr, "LIBXSMM ERROR: libxsmm_generator_spgemm could not write to into destination source file\n");
       exit(-1);
     }
   }
