@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2017, Intel Corporation                                **
+** Copyright (c) 2015-2018, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -28,15 +28,21 @@
 ******************************************************************************/
 /* Alexander Heinecke (Intel Corp.)
 ******************************************************************************/
-
 #include "generator_spgemm_csr_asparse.h"
 #include "generator_common.h"
-#include <libxsmm_macros.h>
-#include <stdio.h>
+#include "libxsmm_main.h"
+
+#if defined(LIBXSMM_OFFLOAD_TARGET)
+# pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
+#endif
 #include <stdlib.h>
 #include <string.h>
+#include <stdio.h>
+#if defined(LIBXSMM_OFFLOAD_TARGET)
+# pragma offload_attribute(pop)
+#endif
 
-LIBXSMM_INTERNAL_API_DEFINITION
+LIBXSMM_API_INTERN
 void libxsmm_generator_spgemm_csr_asparse( libxsmm_generated_code*         io_generated_code,
                                            const libxsmm_gemm_descriptor*  i_xgemm_desc,
                                            const char*                     i_arch,
@@ -69,7 +75,7 @@ void libxsmm_generator_spgemm_csr_asparse( libxsmm_generated_code*         io_ge
       l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "    #pragma vector aligned\n");
       libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
     }
-    if ( (LIBXSMM_GEMM_FLAG_F32PREC & i_xgemm_desc->flags) == 0 ) {
+    if ( LIBXSMM_GEMM_PRECISION_F64 == LIBXSMM_GETENUM_INP( i_xgemm_desc->datatype ) ) {
       l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "    for ( l_n = 0; l_n < %u; l_n++) { C[(l_m*%u)+l_n] = 0.0; }\n", (unsigned int)i_xgemm_desc->ldc, (unsigned int)i_xgemm_desc->ldc);
     } else {
       l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "    for ( l_n = 0; l_n < %u; l_n++) { C[(l_m*%u)+l_n] = 0.0f; }\n", (unsigned int)i_xgemm_desc->ldc, (unsigned int)i_xgemm_desc->ldc);
@@ -85,7 +91,7 @@ void libxsmm_generator_spgemm_csr_asparse( libxsmm_generated_code*         io_ge
   if ( ( strcmp( i_arch, "noarch" ) == 0 ) ||
        ( strcmp( i_arch, "wsm" ) == 0 )    ||
        ( strcmp( i_arch, "snb" ) == 0 )    ||
-       ( strcmp( i_arch, "hsw" ) == 0 )       ) {
+       ( strcmp( i_arch, "hsw" ) == 0 ) ) {
     if ( i_xgemm_desc->n > 7 ) {
       l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "  #pragma simd vectorlength(8)\n");
       libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
@@ -100,19 +106,19 @@ void libxsmm_generator_spgemm_csr_asparse( libxsmm_generated_code*         io_ge
 
   } else if ( ( strcmp( i_arch, "knc" ) == 0 ) ||
               ( strcmp( i_arch, "knl" ) == 0 ) ||
-              ( strcmp( i_arch, "skx" ) == 0 )    ) {
+              ( strcmp( i_arch, "skx" ) == 0 ) ) {
     if ( (i_xgemm_desc->n > 1) ) {
       l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "  #pragma simd vectorlength(16)\n");
       libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
     }
   } else {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_ARCH );
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_ARCH );
     return;
   }
 
   if ( (i_xgemm_desc->n > 1)          &&
        ((LIBXSMM_GEMM_FLAG_ALIGN_A & i_xgemm_desc->flags) != 0) &&
-       ((LIBXSMM_GEMM_FLAG_ALIGN_C & i_xgemm_desc->flags) != 0)    ) {
+       ((LIBXSMM_GEMM_FLAG_ALIGN_C & i_xgemm_desc->flags) != 0) ) {
     l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "  #pragma vector aligned\n");
     libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
   }
