@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2017, Intel Corporation                                **
+** Copyright (c) 2015-2018, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -28,16 +28,23 @@
 ******************************************************************************/
 /* Alexander Heinecke (Intel Corp.)
 ******************************************************************************/
-
 #include "generator_spgemm_csr_asparse_reg.h"
 #include "generator_x86_instructions.h"
 #include "generator_gemm_common.h"
-#include <libxsmm_macros.h>
-#include <stdio.h>
+#include "libxsmm_main.h"
+
+#if defined(LIBXSMM_OFFLOAD_TARGET)
+# pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
+#endif
 #include <stdlib.h>
 #include <string.h>
+#include <assert.h>
+#include <stdio.h>
+#if defined(LIBXSMM_OFFLOAD_TARGET)
+# pragma offload_attribute(pop)
+#endif
 
-LIBXSMM_INTERNAL_API_DEFINITION
+LIBXSMM_API_INTERN
 void libxsmm_mmfunction_signature_asparse_reg( libxsmm_generated_code*         io_generated_code,
                                   const char*                     i_routine_name,
                                   const libxsmm_gemm_descriptor* i_xgemm_desc ) {
@@ -51,14 +58,14 @@ void libxsmm_mmfunction_signature_asparse_reg( libxsmm_generated_code*         i
     l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, ".global %s\n.type %s, @function\n%s:\n", i_routine_name, i_routine_name, i_routine_name);
   } else {
     /* selecting the correct signature */
-    if (0 != (LIBXSMM_GEMM_FLAG_F32PREC & i_xgemm_desc->flags)) {
-      if (LIBXSMM_PREFETCH_NONE == i_xgemm_desc->prefetch) {
+    if (LIBXSMM_GEMM_PRECISION_F32 == LIBXSMM_GETENUM_INP( i_xgemm_desc->datatype ) ) {
+      if (LIBXSMM_GEMM_PREFETCH_NONE == i_xgemm_desc->prefetch) {
         l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "void %s(const float* A, const float* B, float* C) {\n", i_routine_name);
       } else {
         l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "void %s(const float* A, const float* B, float* C, const float* A_prefetch, const float* B_prefetch, const float* C_prefetch) {\n", i_routine_name);
       }
     } else {
-      if (LIBXSMM_PREFETCH_NONE == i_xgemm_desc->prefetch) {
+      if (LIBXSMM_GEMM_PREFETCH_NONE == i_xgemm_desc->prefetch) {
         l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "void %s(const double* A, const double* B, double* C) {\n", i_routine_name);
       } else {
         l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "void %s(const double* A, const double* B, double* C, const double* A_prefetch, const double* B_prefetch, const double* C_prefetch) {\n", i_routine_name);
@@ -69,7 +76,7 @@ void libxsmm_mmfunction_signature_asparse_reg( libxsmm_generated_code*         i
   libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
 }
 
-LIBXSMM_INTERNAL_API_DEFINITION
+LIBXSMM_API_INTERN
 void libxsmm_generator_spgemm_csr_asparse_reg( libxsmm_generated_code*         io_generated_code,
                                                const libxsmm_gemm_descriptor*  i_xgemm_desc,
                                                const char*                     i_arch,
@@ -83,21 +90,34 @@ void libxsmm_generator_spgemm_csr_asparse_reg( libxsmm_generated_code*         i
   unsigned int l_unique;
   unsigned int l_hit;
   unsigned int l_n_blocking = 1;
-  double* l_unique_values = (double*)malloc(sizeof(double)*i_row_idx[i_xgemm_desc->m]);
-  unsigned int* l_unique_pos = (unsigned int*)malloc(sizeof(unsigned int)*i_row_idx[i_xgemm_desc->m]);
-  double l_code_const[8];
+  double *const l_unique_values = (double*)malloc(sizeof(double)*i_row_idx[i_xgemm_desc->m]);
+  unsigned int *const l_unique_pos = (unsigned int*)malloc(sizeof(unsigned int)*i_row_idx[i_xgemm_desc->m]);
+  double l_code_const_dp[8];
+  float l_code_const_fp[16];
 
   libxsmm_micro_kernel_config l_micro_kernel_config;
   libxsmm_loop_label_tracker l_loop_label_tracker;
   libxsmm_gp_reg_mapping l_gp_reg_mapping;
 
-  /* check that we build for AVX512 */
-  if ( (strcmp(i_arch, "knl") != 0) &&
-       (strcmp(i_arch, "skx") != 0)    ) {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_ARCH );
+  /* check if mallocs were successful */
+  if ( 0 == l_unique_values || 0 == l_unique_pos ) {
+    free(l_unique_values); free(l_unique_pos);
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_CSR_ALLOC_DATA );
     return;
   }
 
+  /* check that we build for AVX512 */
+  if ( (strcmp(i_arch, "knl") != 0) &&
+       (strcmp(i_arch, "knm") != 0) &&
+       (strcmp(i_arch, "skx") != 0) ) {
+    free(l_unique_values); free(l_unique_pos);
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_ARCH );
+    return;
+  }
+
+  /* prerequisite */
+  assert(0 != i_values);
+
   /* Let's figure out how many unique values we have */
   l_unique = 1;
   l_unique_values[0] = i_values[0];
@@ -121,19 +141,29 @@ void libxsmm_generator_spgemm_csr_asparse_reg( libxsmm_generated_code*         i
 
   /* check that we have enough registers (N=20) for now */
   if ( l_unique > 31 ) {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_UNIQUE_VAL );
+    free(l_unique_values); free(l_unique_pos);
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_UNIQUE_VAL );
     return;
   }
 
   /* define gp register mapping */
   libxsmm_reset_x86_gp_reg_mapping( &l_gp_reg_mapping );
-  /* matching calling convention on Linux */
+#if defined(_WIN32) || defined(__CYGWIN__)
+  l_gp_reg_mapping.gp_reg_a = LIBXSMM_X86_GP_REG_RCX;
+  l_gp_reg_mapping.gp_reg_b = LIBXSMM_X86_GP_REG_RDX;
+  l_gp_reg_mapping.gp_reg_c = LIBXSMM_X86_GP_REG_R8;
+  /* TODO: full support for Windows calling convention */
+  l_gp_reg_mapping.gp_reg_a_prefetch = LIBXSMM_X86_GP_REG_R9;
+  l_gp_reg_mapping.gp_reg_b_prefetch = LIBXSMM_X86_GP_REG_RDI;
+  l_gp_reg_mapping.gp_reg_c_prefetch = LIBXSMM_X86_GP_REG_RSI;
+#else /* match calling convention on Linux */
   l_gp_reg_mapping.gp_reg_a = LIBXSMM_X86_GP_REG_RDI;
   l_gp_reg_mapping.gp_reg_b = LIBXSMM_X86_GP_REG_RSI;
   l_gp_reg_mapping.gp_reg_c = LIBXSMM_X86_GP_REG_RDX;
   l_gp_reg_mapping.gp_reg_a_prefetch = LIBXSMM_X86_GP_REG_RCX;
   l_gp_reg_mapping.gp_reg_b_prefetch = LIBXSMM_X86_GP_REG_R8;
   l_gp_reg_mapping.gp_reg_c_prefetch = LIBXSMM_X86_GP_REG_R9;
+#endif
   l_gp_reg_mapping.gp_reg_mloop = LIBXSMM_X86_GP_REG_R12;
   l_gp_reg_mapping.gp_reg_nloop = LIBXSMM_X86_GP_REG_R13;
   l_gp_reg_mapping.gp_reg_kloop = LIBXSMM_X86_GP_REG_R14;
@@ -151,8 +181,9 @@ void libxsmm_generator_spgemm_csr_asparse_reg( libxsmm_generated_code*         i
   libxsmm_generator_gemm_init_micro_kernel_config_fullvector( &l_micro_kernel_config, i_xgemm_desc, i_arch, 0 );
 
   /* inner chunk size */
-  if ( i_xgemm_desc->n != (int)l_micro_kernel_config.vector_length ) {
-    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_N_BLOCK );
+  if ( i_xgemm_desc->n != l_micro_kernel_config.vector_length ) {
+    free(l_unique_values); free(l_unique_pos);
+    LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_N_BLOCK );
     return;
   }
 
@@ -163,19 +194,25 @@ void libxsmm_generator_spgemm_csr_asparse_reg( libxsmm_generated_code*         i
   for ( l_z = 0; l_z < l_unique; l_z++) {
     char l_id[65];
     LIBXSMM_SNPRINTF(l_id, 64, "%u", l_z);
-    l_code_const[0] = l_unique_values[l_z];
-    l_code_const[1] = l_unique_values[l_z];
-    l_code_const[2] = l_unique_values[l_z];
-    l_code_const[3] = l_unique_values[l_z];
-    l_code_const[4] = l_unique_values[l_z];
-    l_code_const[5] = l_unique_values[l_z];
-    l_code_const[6] = l_unique_values[l_z];
-    l_code_const[7] = l_unique_values[l_z];
-    libxsmm_x86_instruction_full_vec_load_of_constants ( io_generated_code,
-                                                         (unsigned char*)l_code_const,
-                                                         l_id,
-                                                         l_micro_kernel_config.vector_name,
-                                                         l_z );
+    if ( LIBXSMM_GEMM_PRECISION_F64 == LIBXSMM_GETENUM_INP( i_xgemm_desc->datatype )  ) {
+      for ( l_m = 0; l_m < 8; l_m++) {
+        l_code_const_dp[l_m] = l_unique_values[l_z];
+      }
+      libxsmm_x86_instruction_full_vec_load_of_constants ( io_generated_code,
+                                                           (unsigned char*)l_code_const_dp,
+                                                           l_id,
+                                                           l_micro_kernel_config.vector_name,
+                                                           l_z );
+    } else {
+      for ( l_m = 0; l_m < 16; l_m++) {
+        l_code_const_fp[l_m] = (float)l_unique_values[l_z];
+      }
+      libxsmm_x86_instruction_full_vec_load_of_constants ( io_generated_code,
+                                                           (unsigned char*)l_code_const_fp,
+                                                           l_id,
+                                                           l_micro_kernel_config.vector_name,
+                                                           l_z );
+    }
   }
 
   /* n loop */
@@ -188,15 +225,26 @@ void libxsmm_generator_spgemm_csr_asparse_reg( libxsmm_generated_code*         i
     l_row_elements = i_row_idx[l_m+1] - i_row_idx[l_m];
     if (l_row_elements > 0) {
       for ( l_n = 0; l_n < l_n_blocking; l_n++ ) {
-        libxsmm_x86_instruction_vec_move( io_generated_code,
-                                          l_micro_kernel_config.instruction_set,
-                                          l_micro_kernel_config.c_vmove_instruction,
-                                          l_gp_reg_mapping.gp_reg_c,
-                                          LIBXSMM_X86_GP_REG_UNDEF, 0,
-                                          l_m*i_xgemm_desc->ldc*l_micro_kernel_config.datatype_size +
-                                            l_n*l_micro_kernel_config.datatype_size*l_micro_kernel_config.vector_length,
-                                          l_micro_kernel_config.vector_name,
-                                          l_unique+l_n, 0, 0 );
+        /* load C or reset to 0 depending on beta */
+        if ( i_xgemm_desc->beta != 0 ) {
+          libxsmm_x86_instruction_vec_move( io_generated_code,
+                                            l_micro_kernel_config.instruction_set,
+                                            l_micro_kernel_config.c_vmove_instruction,
+                                            l_gp_reg_mapping.gp_reg_c,
+                                            LIBXSMM_X86_GP_REG_UNDEF, 0,
+                                            l_m*i_xgemm_desc->ldc*l_micro_kernel_config.datatype_size +
+                                              l_n*l_micro_kernel_config.datatype_size*l_micro_kernel_config.vector_length,
+                                            l_micro_kernel_config.vector_name,
+                                            l_unique+l_n, 0, 0 );
+        } else {
+          libxsmm_x86_instruction_vec_compute_reg( io_generated_code,
+                                                   l_micro_kernel_config.instruction_set,
+                                                   l_micro_kernel_config.vxor_instruction,
+                                                   l_micro_kernel_config.vector_name,
+                                                   l_unique+l_n,
+                                                   l_unique+l_n,
+                                                   l_unique+l_n );
+        }
 
         libxsmm_x86_instruction_prefetch( io_generated_code,
                                           LIBXSMM_X86_INSTR_PREFETCHT2,
