@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2016-2017, Intel Corporation                                **
+** Copyright (c) 2016-2018, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -40,14 +40,13 @@
 
 /** Execute CPUID, and receive results (EAX, EBX, ECX, EDX) for requested FUNCTION. */
 #if defined(__GNUC__) || defined(__PGI)
-# if (0 != (__x86_64__)) || (4 < (__SIZEOF_PTRDIFF_T__)) \
-  || (4294967295U < (__SIZE_MAX__)) || defined(_CRAYC)
+# if (64 > (LIBXSMM_BITS))
+LIBXSMM_EXTERN LIBXSMM_RETARGETABLE int __get_cpuid(unsigned int, unsigned int*, unsigned int*, unsigned int*, unsigned int*);
+#   define LIBXSMM_CPUID_X86(FUNCTION, EAX, EBX, ECX, EDX) __get_cpuid(FUNCTION, &(EAX), &(EBX), &(ECX), &(EDX))
+# else
 #   define LIBXSMM_CPUID_X86(FUNCTION, EAX, EBX, ECX, EDX) __asm__ __volatile__ ( \
       ".byte 0x0f, 0xa2" /*cpuid*/ : "=a"(EAX), "=b"(EBX), "=c"(ECX), "=d"(EDX) : "a"(FUNCTION), "c"(0) \
     )
-# else
-LIBXSMM_EXTERN LIBXSMM_RETARGETABLE int __get_cpuid(unsigned int, unsigned int*, unsigned int*, unsigned int*, unsigned int*);
-#   define LIBXSMM_CPUID_X86(FUNCTION, EAX, EBX, ECX, EDX) __get_cpuid(FUNCTION, &(EAX), &(EBX), &(ECX), &(EDX))
 # endif
 #elif !defined(_CRAYC)
 # define LIBXSMM_CPUID_X86(FUNCTION, EAX, EBX, ECX, EDX) { \
@@ -78,7 +77,7 @@ LIBXSMM_EXTERN LIBXSMM_RETARGETABLE int __get_cpuid(unsigned int, unsigned int*,
 #endif
 
 
-LIBXSMM_API_DEFINITION int libxsmm_cpuid_x86(void)
+LIBXSMM_API int libxsmm_cpuid_x86(void)
 {
   int target_arch = LIBXSMM_STATIC_TARGET_ARCH;
   unsigned int eax = 0, ebx = 0, ecx = 0, edx = 0;
@@ -103,11 +102,21 @@ LIBXSMM_API_DEFINITION int libxsmm_cpuid_x86(void)
           if (0x10010000 == (0x10010000 & ebx)) { /* Common */
             /* AVX512DQ(0x00020000), AVX512BW(0x40000000), AVX512VL(0x80000000) */
             if (0xC0020000 == (0xC0020000 & ebx)) { /* SKX (Core) */
-              target_arch = LIBXSMM_X86_AVX512_CORE;
+              if (0x00000800 == (0x00000800 & ecx)) { /* ICL (CORE) */
+                target_arch = LIBXSMM_X86_AVX512_ICL;
+              }
+              else { /* SKX (CORE) */
+                target_arch = LIBXSMM_X86_AVX512_CORE;
+              }
             }
             /* AVX512PF(0x04000000), AVX512ER(0x08000000) */
-            else if (0x0C000000 == (0x0C000000 & ebx)) { /* KNL (MIC) */
-              target_arch = LIBXSMM_X86_AVX512_MIC;
+            else if (0x0C000000 == (0x0C000000 & ebx)) {
+              if (0x0000000C == (0x0000000C & edx)) { /* KNM (MIC) */
+                target_arch = LIBXSMM_X86_AVX512_KNM;
+              }
+              else { /* KNL (MIC) */
+                target_arch = LIBXSMM_X86_AVX512_MIC;
+              }
             }
             else { /* Common */
               target_arch = LIBXSMM_X86_AVX512;
@@ -133,7 +142,7 @@ LIBXSMM_API_DEFINITION int libxsmm_cpuid_x86(void)
 }
 
 
-LIBXSMM_API_DEFINITION int libxsmm_cpuid(void)
+LIBXSMM_API int libxsmm_cpuid(void)
 {
   return libxsmm_cpuid_x86();
 }
