@@ -26,12 +26,21 @@
 ** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
 ** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
 ******************************************************************************/
-/* Alexander Heinecke (Intel Corp.), Hans Pabst (Intel Corp.)
+/* Alexander Heinecke, Hans Pabst (Intel Corp.)
 ******************************************************************************/
 #include "libxsmm_dnn_convolution_forward.h"
-#include <libxsmm.h>
+#include <libxsmm_intrinsics_x86.h>
+#include "libxsmm_main.h"
 
-LIBXSMM_API_DEFINITION libxsmm_dnn_err_t libxsmm_dnn_convolve_st_fwd_custom_custom(libxsmm_dnn_layer* handle, int start_thread, int tid)
+#if defined(LIBXSMM_OFFLOAD_TARGET)
+# pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
+#endif
+#include <string.h>
+#if defined(LIBXSMM_OFFLOAD_TARGET)
+# pragma offload_attribute(pop)
+#endif
+
+LIBXSMM_API_INTERN libxsmm_dnn_err_t libxsmm_dnn_convolve_st_fwd_custom_custom(libxsmm_dnn_layer* handle, int start_thread, int tid)
 {
   libxsmm_dnn_err_t status = LIBXSMM_DNN_SUCCESS;
 
@@ -53,22 +62,46 @@ LIBXSMM_API_DEFINITION libxsmm_dnn_err_t libxsmm_dnn_convolve_st_fwd_custom_cust
       typedef float element_input_type;
       typedef float element_output_type;
       typedef float element_filter_type;
+      if (handle->padding_flag == 1) {
+#define INPUT_PADDING
 # include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_fallback.tpl.c"
+#undef INPUT_PADDING
+      } else {
+# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_fallback.tpl.c"
+      }
     } else if (handle->datatype == LIBXSMM_DNN_DATATYPE_I16 && handle->datatype_itm == LIBXSMM_DNN_DATATYPE_I32 ) {
       typedef short element_input_type;
       typedef int element_output_type;
       typedef short element_filter_type;
+      if (handle->padding_flag == 1) {
+#define INPUT_PADDING
 # include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_fallback.tpl.c"
+#undef INPUT_PADDING
+      } else {
+# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_fallback.tpl.c"
+      }
     } else if (handle->datatype ==  LIBXSMM_DNN_DATATYPE_I8 && handle->datatype_itm == LIBXSMM_DNN_DATATYPE_I16 && (handle->desc.options & LIBXSMM_DNN_CONV_OPTION_ACTIVATION_UNSIGNED) > 0 ) {
       typedef unsigned char element_input_type;
       typedef short element_output_type;
       typedef char element_filter_type;
+      if (handle->padding_flag == 1) {
+#define INPUT_PADDING
 # include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_fallback.tpl.c"
+#undef INPUT_PADDING
+      } else {
+# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_fallback.tpl.c"
+      }
     } else if (handle->datatype ==  LIBXSMM_DNN_DATATYPE_I8 && handle->datatype_itm == LIBXSMM_DNN_DATATYPE_I32 && (handle->desc.options & LIBXSMM_DNN_CONV_OPTION_ACTIVATION_UNSIGNED) > 0 ) {
       typedef unsigned char element_input_type;
       typedef int element_output_type;
       typedef char element_filter_type;
+      if (handle->padding_flag == 1) {
+#define INPUT_PADDING
 # include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_fallback.tpl.c"
+#undef INPUT_PADDING
+      } else {
+# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_fallback.tpl.c"
+      }
     } else {
       status = LIBXSMM_DNN_ERR_UNSUPPORTED_DATATYPE;
       return status;
@@ -82,11 +115,25 @@ LIBXSMM_API_DEFINITION libxsmm_dnn_err_t libxsmm_dnn_convolve_st_fwd_custom_cust
         typedef float element_filter_type;
         typedef libxsmm_sconvfunction libxsmm_convfunction;
         if (handle->desc.u == 1 && handle->desc.v == 1) {
+          if (handle->padding_flag == 1) {
+#define LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
+#define INPUT_PADDING
+# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom.tpl.c"
+#undef LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
+#undef INPUT_PADDING
+          } else {
 #define LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
 # include "template/libxsmm_dnn_convolve_st_fwd_custom_custom.tpl.c"
 #undef LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
+          }
         } else {
+          if (handle->padding_flag == 1) {
+#define INPUT_PADDING
 # include "template/libxsmm_dnn_convolve_st_fwd_custom_custom.tpl.c"
+#undef INPUT_PADDING
+          } else {
+# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom.tpl.c"
+          }
         }
       }
       else {
@@ -95,11 +142,25 @@ LIBXSMM_API_DEFINITION libxsmm_dnn_err_t libxsmm_dnn_convolve_st_fwd_custom_cust
         typedef float element_filter_type;
         typedef libxsmm_sconvfunction libxsmm_convfunction;
         if (handle->desc.u == 1 && handle->desc.v == 1) {
+          if (handle->padding_flag == 1) {
+#define LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
+#define INPUT_PADDING
+# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_img_par.tpl.c"
+#undef LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
+#undef INPUT_PADDING
+          } else {
 #define LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
 # include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_img_par.tpl.c"
 #undef LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
+          }
         } else {
+          if (handle->padding_flag == 1) {
+#define INPUT_PADDING
 # include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_img_par.tpl.c"
+#undef INPUT_PADDING
+          } else {
+# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_img_par.tpl.c"
+          }
         }
       }
     } else if (handle->datatype ==  LIBXSMM_DNN_DATATYPE_I16 && handle->datatype_itm == LIBXSMM_DNN_DATATYPE_I32 ) {
@@ -109,11 +170,25 @@ LIBXSMM_API_DEFINITION libxsmm_dnn_err_t libxsmm_dnn_convolve_st_fwd_custom_cust
         typedef short element_filter_type;
         typedef libxsmm_wconvfunction libxsmm_convfunction;
         if (handle->desc.u == 1 && handle->desc.v == 1) {
+          if (handle->padding_flag == 1) {
 #define LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
-# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom.tpl.c"
+#define INPUT_PADDING
+# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_1.tpl.c"
+#undef INPUT_PADDING
+#undef LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ON
+          } else {
+#define LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
+# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_1.tpl.c"
 #undef LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
+          }
         } else {
-# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom.tpl.c"
+          if (handle->padding_flag == 1) {
+#define INPUT_PADDING
+# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_1.tpl.c"
+#undef INPUT_PADDING
+          } else {
+# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_1.tpl.c"
+          }
         }
       }
       else {
@@ -122,11 +197,25 @@ LIBXSMM_API_DEFINITION libxsmm_dnn_err_t libxsmm_dnn_convolve_st_fwd_custom_cust
         typedef short element_filter_type;
         typedef libxsmm_wconvfunction libxsmm_convfunction;
         if (handle->desc.u == 1 && handle->desc.v == 1) {
+          if (handle->padding_flag == 1) {
+#define LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
+#define INPUT_PADDING
+# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_img_par.tpl.c"
+#undef INPUT_PADDING
+#undef LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
+          } else {
 #define LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
 # include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_img_par.tpl.c"
 #undef LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
+          }
         } else {
-# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_img_par.tpl.c"
+          if (handle->padding_flag == 1) {
+#define INPUT_PADDING
+#include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_img_par.tpl.c"
+#undef INPUT_PADDING
+          } else {
+#include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_img_par.tpl.c"
+          }
         }
       }
     } else if (handle->datatype == LIBXSMM_DNN_DATATYPE_I8 && handle->datatype_itm == LIBXSMM_DNN_DATATYPE_I16 && (handle->desc.options & LIBXSMM_DNN_CONV_OPTION_ACTIVATION_UNSIGNED) > 0 ) {
@@ -136,11 +225,25 @@ LIBXSMM_API_DEFINITION libxsmm_dnn_err_t libxsmm_dnn_convolve_st_fwd_custom_cust
         typedef char element_filter_type;
         typedef libxsmm_busconvfunction libxsmm_convfunction;
         if (handle->desc.u == 1 && handle->desc.v == 1) {
+          if (handle->padding_flag == 1) {
 #define LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
-# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom.tpl.c"
+#define INPUT_PADDING
+# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_1.tpl.c"
+#undef INPUT_PADDING
 #undef LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
+          } else {
+#define LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
+# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_1.tpl.c"
+#undef LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
+          }
         } else {
-# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom.tpl.c"
+          if (handle->padding_flag == 1) {
+#define INPUT_PADDING
+# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_1.tpl.c"
+#undef INPUT_PADDING
+          } else {
+# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_1.tpl.c"
+          }
         }
       }
       else {
@@ -149,11 +252,25 @@ LIBXSMM_API_DEFINITION libxsmm_dnn_err_t libxsmm_dnn_convolve_st_fwd_custom_cust
         typedef char element_filter_type;
         typedef libxsmm_busconvfunction libxsmm_convfunction;
         if (handle->desc.u == 1 && handle->desc.v == 1) {
+          if (handle->padding_flag == 1) {
+#define LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
+#define INPUT_PADDING
+# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_img_par.tpl.c"
+#undef INPUT_PADDING
+#undef LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
+          } else {
 #define LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
 # include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_img_par.tpl.c"
 #undef LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
+          }
         } else {
+          if (handle->padding_flag == 1) {
+#define INPUT_PADDING
 # include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_img_par.tpl.c"
+#undef INPUT_PADDING
+          } else {
+# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_img_par.tpl.c"
+          }
         }
       }
     } else if (handle->datatype == LIBXSMM_DNN_DATATYPE_I8 && handle->datatype_itm == LIBXSMM_DNN_DATATYPE_I32 && (handle->desc.options & LIBXSMM_DNN_CONV_OPTION_ACTIVATION_UNSIGNED) > 0 ) {
@@ -163,11 +280,25 @@ LIBXSMM_API_DEFINITION libxsmm_dnn_err_t libxsmm_dnn_convolve_st_fwd_custom_cust
         typedef char element_filter_type;
         typedef libxsmm_budconvfunction libxsmm_convfunction;
         if (handle->desc.u == 1 && handle->desc.v == 1) {
+          if (handle->padding_flag == 1) {
 #define LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
-# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom.tpl.c"
+#define INPUT_PADDING
+# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_1.tpl.c"
+#undef INPUT_PADDING
 #undef LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
+          } else {
+#define LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
+# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_1.tpl.c"
+#undef LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
+          }
         } else {
-# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom.tpl.c"
+          if (handle->padding_flag == 1) {
+#define INPUT_PADDING
+# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_1.tpl.c"
+#undef INPUT_PADDING
+          } else {
+# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_1.tpl.c"
+          }
         }
       }
       else {
@@ -176,11 +307,25 @@ LIBXSMM_API_DEFINITION libxsmm_dnn_err_t libxsmm_dnn_convolve_st_fwd_custom_cust
         typedef char element_filter_type;
         typedef libxsmm_budconvfunction libxsmm_convfunction;
         if (handle->desc.u == 1 && handle->desc.v == 1) {
+          if (handle->padding_flag == 1) {
 #define LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
-# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_img_par.tpl.c"
+#define INPUT_PADDING
+#include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_img_par.tpl.c"
+#undef INPUT_PADDING
 #undef LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
+          } else {
+#define LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
+#include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_img_par.tpl.c"
+#undef LIBXSMM_DNN_CONV_FWD_INTERNAL_STRIDE_ONE
+          }
         } else {
-# include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_img_par.tpl.c"
+          if (handle->padding_flag == 1) {
+#define INPUT_PADDING
+#include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_img_par.tpl.c"
+#undef INPUT_PADDING
+          } else {
+#include "template/libxsmm_dnn_convolve_st_fwd_custom_custom_img_par.tpl.c"
+          }
         }
       }
     } else {
@@ -193,7 +338,7 @@ LIBXSMM_API_DEFINITION libxsmm_dnn_err_t libxsmm_dnn_convolve_st_fwd_custom_cust
 }
 
 
-LIBXSMM_API_DEFINITION libxsmm_dnn_err_t libxsmm_dnn_convolve_st_fwd_nhwc_custom(libxsmm_dnn_layer* handle, int start_thread, int tid)
+LIBXSMM_API_INTERN libxsmm_dnn_err_t libxsmm_dnn_convolve_st_fwd_nhwc_custom(libxsmm_dnn_layer* handle, int start_thread, int tid)
 {
   libxsmm_dnn_err_t status = LIBXSMM_DNN_SUCCESS;
 
@@ -209,7 +354,13 @@ LIBXSMM_API_DEFINITION libxsmm_dnn_err_t libxsmm_dnn_convolve_st_fwd_nhwc_custom
       typedef float element_input_type;
       typedef float element_output_type;
       typedef float element_filter_type;
+      if (handle->padding_flag == 1) {
+#define INPUT_PADDING
 # include "template/libxsmm_dnn_convolve_st_fwd_nhwc_custom_fallback.tpl.c"
+#undef INPUT_PADDING
+      } else {
+# include "template/libxsmm_dnn_convolve_st_fwd_nhwc_custom_fallback.tpl.c"
+      }
     } else {
       status = LIBXSMM_DNN_ERR_UNSUPPORTED_DATATYPE;
       return status;
@@ -221,14 +372,26 @@ LIBXSMM_API_DEFINITION libxsmm_dnn_err_t libxsmm_dnn_convolve_st_fwd_nhwc_custom
         typedef float element_output_type;
         typedef float element_filter_type;
         typedef libxsmm_sconvfunction libxsmm_convfunction;
+        if (handle->padding_flag == 1) {
+#define INPUT_PADDING
 # include "template/libxsmm_dnn_convolve_st_fwd_nhwc_custom.tpl.c"
+#undef INPUT_PADDING
+        } else {
+# include "template/libxsmm_dnn_convolve_st_fwd_nhwc_custom.tpl.c"
+        }
       }
       else {
         typedef float element_input_type;
         typedef float element_output_type;
         typedef float element_filter_type;
         typedef libxsmm_sconvfunction libxsmm_convfunction;
+        if (handle->padding_flag == 1) {
+#define INPUT_PADDING
 # include "template/libxsmm_dnn_convolve_st_fwd_nhwc_custom_img_par.tpl.c"
+#undef INPUT_PADDING
+        } else {
+# include "template/libxsmm_dnn_convolve_st_fwd_nhwc_custom_img_par.tpl.c"
+        }
       }
     } else {
       status = LIBXSMM_DNN_ERR_UNSUPPORTED_DATATYPE;
@@ -240,7 +403,7 @@ LIBXSMM_API_DEFINITION libxsmm_dnn_err_t libxsmm_dnn_convolve_st_fwd_nhwc_custom
 }
 
 
-LIBXSMM_API_DEFINITION libxsmm_dnn_err_t libxsmm_dnn_convolve_st_fwd_nhwc_rsck(libxsmm_dnn_layer* handle, int start_thread, int tid)
+LIBXSMM_API_INTERN libxsmm_dnn_err_t libxsmm_dnn_convolve_st_fwd_nhwc_rsck(libxsmm_dnn_layer* handle, int start_thread, int tid)
 {
   libxsmm_dnn_err_t status = LIBXSMM_DNN_SUCCESS;
 
@@ -256,7 +419,13 @@ LIBXSMM_API_DEFINITION libxsmm_dnn_err_t libxsmm_dnn_convolve_st_fwd_nhwc_rsck(l
       typedef float element_input_type;
       typedef float element_output_type;
       typedef float element_filter_type;
+      if (handle->padding_flag == 1) {
+#define INPUT_PADDING
 # include "template/libxsmm_dnn_convolve_st_fwd_nhwc_rsck_fallback.tpl.c"
+#undef INPUT_PADDING
+      } else {
+# include "template/libxsmm_dnn_convolve_st_fwd_nhwc_rsck_fallback.tpl.c"
+      }
     } else {
       status = LIBXSMM_DNN_ERR_UNSUPPORTED_DATATYPE;
       return status;
@@ -268,14 +437,26 @@ LIBXSMM_API_DEFINITION libxsmm_dnn_err_t libxsmm_dnn_convolve_st_fwd_nhwc_rsck(l
         typedef float element_output_type;
         typedef float element_filter_type;
         typedef libxsmm_sconvfunction libxsmm_convfunction;
+        if (handle->padding_flag == 1) {
+#define INPUT_PADDING
 # include "template/libxsmm_dnn_convolve_st_fwd_nhwc_rsck.tpl.c"
+#undef INPUT_PADDING
+        } else {
+# include "template/libxsmm_dnn_convolve_st_fwd_nhwc_rsck.tpl.c"
+        }
       }
       else {
         typedef float element_input_type;
         typedef float element_output_type;
         typedef float element_filter_type;
         typedef libxsmm_sconvfunction libxsmm_convfunction;
+        if (handle->padding_flag == 1) {
+#define INPUT_PADDING
 # include "template/libxsmm_dnn_convolve_st_fwd_nhwc_rsck_img_par.tpl.c"
+#undef INPUT_PADDING
+        } else {
+# include "template/libxsmm_dnn_convolve_st_fwd_nhwc_rsck_img_par.tpl.c"
+        }
       }
     } else {
       status = LIBXSMM_DNN_ERR_UNSUPPORTED_DATATYPE;
