@@ -32,8 +32,10 @@
 #define LIBXSMM_DNN_HANDLE_H
 
 #include <libxsmm_dnn.h>
-#include "libxsmm_main.h"
 
-LIBXSMM_API libxsmm_dnn_err_t libxsmm_dnn_internal_create_conv_handle_direct( libxsmm_dnn_layer* handle );
+
+LIBXSMM_API_INTERN libxsmm_dnn_err_t libxsmm_dnn_internal_create_conv_handle_direct( libxsmm_dnn_layer* handle );
+
+LIBXSMM_API_INTERN libxsmm_dnn_err_t libxsmm_dnn_internal_create_conv_handle_winograd_check( libxsmm_dnn_layer* handle );
 
 #endif /* LIBXSMM_DNN_HANDLE_H */
