@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2016-2017, Intel Corporation                                **
+** Copyright (c) 2016-2018, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -33,7 +33,6 @@
 
 #include "libxsmm_gemm_diff.h"
 #include "libxsmm_main.h"
-#include <libxsmm_intrinsics_x86.h>
 
 #if defined(LIBXSMM_OFFLOAD_TARGET)
 # pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
@@ -45,33 +44,25 @@
 #endif
 
 /* Enable/disable specific code paths */
-#if !defined(LIBXSMM_GEMM_DIFF_KNC) && !defined(LIBXSMM_INTRINSICS_NONE) && defined(__MIC__)
+#if defined(LIBXSMM_INTRINSICS_KNC) && !defined(LIBXSMM_GEMM_DIFF_KNC)
 /*#define LIBXSMM_GEMM_DIFF_KNC*/
 #endif
-#if !defined(LIBXSMM_GEMM_DIFF_SSE) && !defined(LIBXSMM_INTRINSICS_NONE) && ( \
-     (!defined(LIBXSMM_INTRINSICS_LEGACY) && (LIBXSMM_X86_SSE3 <= LIBXSMM_MAX_STATIC_TARGET_ARCH)) \
-  || (defined(__clang__) && LIBXSMM_X86_SSE3 <= LIBXSMM_STATIC_TARGET_ARCH))
-# define LIBXSMM_GEMM_DIFF_SSE
+#if defined(LIBXSMM_INTRINSICS_SSE3) && !defined(LIBXSMM_GEMM_DIFF_SSE3)
+# define LIBXSMM_GEMM_DIFF_SSE3
 #endif
-#if !defined(LIBXSMM_GEMM_DIFF_AVX) && !defined(LIBXSMM_INTRINSICS_NONE) && defined(LIBXSMM_GEMM_DIFF_SSE) && ( \
-     (!defined(LIBXSMM_INTRINSICS_LEGACY) && (LIBXSMM_X86_AVX2 <= LIBXSMM_MAX_STATIC_TARGET_ARCH)) \
-  || (defined(__clang__) && LIBXSMM_X86_AVX2 <= LIBXSMM_STATIC_TARGET_ARCH))
+#if defined(LIBXSMM_INTRINSICS_AVX) && !defined(LIBXSMM_GEMM_DIFF_AVX)
 # define LIBXSMM_GEMM_DIFF_AVX
 #endif
-#if !defined(LIBXSMM_GEMM_DIFF_AVX2) && !defined(LIBXSMM_INTRINSICS_NONE) && defined(LIBXSMM_GEMM_DIFF_AVX) && ( \
-     (!defined(LIBXSMM_INTRINSICS_LEGACY) && (LIBXSMM_X86_AVX2 <= LIBXSMM_MAX_STATIC_TARGET_ARCH)) \
-  || (defined(__clang__) && LIBXSMM_X86_AVX2 <= LIBXSMM_STATIC_TARGET_ARCH))
+#if defined(LIBXSMM_INTRINSICS_AVX2) && !defined(LIBXSMM_GEMM_DIFF_AVX2)
 # define LIBXSMM_GEMM_DIFF_AVX2
 #endif
-#if !defined(LIBXSMM_GEMM_DIFF_AVX512) && !defined(LIBXSMM_INTRINSICS_NONE) && defined(LIBXSMM_GEMM_DIFF_AVX2) && ( \
-     (!defined(LIBXSMM_INTRINSICS_LEGACY) && (LIBXSMM_X86_AVX512 <= LIBXSMM_MAX_STATIC_TARGET_ARCH)) \
-  || (defined(__clang__) && LIBXSMM_X86_AVX512 <= LIBXSMM_STATIC_TARGET_ARCH))
+#if defined(LIBXSMM_INTRINSICS_AVX512) && !defined(LIBXSMM_GEMM_DIFF_AVX512)
 # define LIBXSMM_GEMM_DIFF_AVX512
 #endif
 
 
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE libxsmm_gemm_diff_function internal_gemm_diff_fn;
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE libxsmm_gemm_diffn_function internal_gemm_diffn_fn;
+LIBXSMM_APIVAR(libxsmm_gemm_diff_function internal_gemm_diff_fn);
+LIBXSMM_APIVAR(libxsmm_gemm_diffn_function internal_gemm_diffn_fn);
 
 
 LIBXSMM_GEMM_DIFF_API_DEFINITION void libxsmm_gemm_diff_init(int target_arch)
@@ -158,7 +149,7 @@ LIBXSMM_GEMM_DIFF_API_DEFINITION LIBXSMM_INTRINSICS(LIBXSMM_X86_SSE3)
 unsigned int libxsmm_gemm_diff_sse(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* desc)
 {
   assert(0 != reference && 0 != desc);
-#if defined(LIBXSMM_GEMM_DIFF_SSE)
+#if defined(LIBXSMM_GEMM_DIFF_SSE3)
   assert(0 == LIBXSMM_MOD2(LIBXSMM_GEMM_DESCRIPTOR_SIZE, sizeof(unsigned int)));
 # if (16 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
   {
@@ -175,7 +166,7 @@ unsigned int libxsmm_gemm_diff_sse(const libxsmm_gemm_descriptor* reference, con
     if (0 != libxsmm_verbosity /* library code is expected to be mute */
      && 1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED))
     {
-      fprintf(stderr, "LIBXSMM: unable to enter SSE code path!\n");
+      fprintf(stderr, "LIBXSMM WARNING: unable to enter SSE code path!\n");
     }
   }
   return libxsmm_gemm_diff_sw(reference, desc);
@@ -215,7 +206,7 @@ unsigned int libxsmm_gemm_diff_avx(const libxsmm_gemm_descriptor* reference, con
     if (0 != libxsmm_verbosity /* library code is expected to be mute */
      && 1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED))
     {
-      fprintf(stderr, "LIBXSMM: unable to enter AVX code path!\n");
+      fprintf(stderr, "LIBXSMM WARNING: unable to enter AVX code path!\n");
     }
   }
   return libxsmm_gemm_diff_sse(reference, desc);
@@ -257,7 +248,7 @@ unsigned int libxsmm_gemm_diff_avx2(const libxsmm_gemm_descriptor* reference, co
     if (0 != libxsmm_verbosity /* library code is expected to be mute */
      && 1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED))
     {
-      fprintf(stderr, "LIBXSMM: unable to enter AVX2 code path!\n");
+      fprintf(stderr, "LIBXSMM WARNING: unable to enter AVX2 code path!\n");
     }
   }
   return libxsmm_gemm_diff_avx(reference, desc);
@@ -286,8 +277,8 @@ LIBXSMM_GEMM_DIFF_API_DEFINITION unsigned int libxsmm_gemm_diff_imci(const libxs
 }
 
 
-LIBXSMM_GEMM_DIFF_API_DEFINITION unsigned int libxsmm_gemm_diffn(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
-  unsigned int hint, unsigned int ndescs, int nbytes)
+LIBXSMM_GEMM_DIFF_API_DEFINITION unsigned int libxsmm_gemm_diffn(const libxsmm_gemm_descriptor* reference,
+  const void* descs, unsigned int hint, unsigned int ndescs, int nbytes)
 {
   /* attempt to rely on static code path avoids to rely on capability of inlining pointer-based function call */
 #if defined(LIBXSMM_GEMM_DIFF_SW) && (0 != LIBXSMM_GEMM_DIFF_SW)
@@ -312,13 +303,12 @@ LIBXSMM_GEMM_DIFF_API_DEFINITION unsigned int libxsmm_gemm_diffn(const libxsmm_g
 }
 
 
-LIBXSMM_GEMM_DIFF_API_DEFINITION unsigned int libxsmm_gemm_diffn_sw(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
-  unsigned int hint, unsigned int ndescs, int nbytes)
+LIBXSMM_GEMM_DIFF_API_DEFINITION unsigned int libxsmm_gemm_diffn_sw(const libxsmm_gemm_descriptor* reference,
+  const void* descs, unsigned int hint, unsigned int ndescs, int nbytes)
 {
   const char *const desc = (const char*)descs;
   const unsigned int end = hint + ndescs;
   unsigned int i;
-#if defined(LIBXSMM_GEMM_DIFF_SW) && (2 == (LIBXSMM_GEMM_DIFF_SW)) /* most general implementation */
   for (i = hint; i < end; ++i) {
     const unsigned int j = i % ndescs; /* wrap around index */
     /* negative stride runs backwards */
@@ -326,26 +316,16 @@ LIBXSMM_GEMM_DIFF_API_DEFINITION unsigned int libxsmm_gemm_diffn_sw(const libxsm
       return j;
     }
   }
-#else /* slightly more optimized implementation */
-  assert(/*is pot*/ndescs == (1u << LIBXSMM_LOG2(ndescs)));
-  for (i = hint; i < end; ++i) {
-    const unsigned int j = LIBXSMM_MOD2(i, ndescs); /* wrap around index */
-    if (0 == libxsmm_gemm_diff_sw(reference, (const libxsmm_gemm_descriptor*)(desc + j * nbytes))) {
-      return j;
-    }
-  }
-#endif
   return ndescs;
 }
 
 
 LIBXSMM_GEMM_DIFF_API_DEFINITION LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX)
-unsigned int libxsmm_gemm_diffn_avx(
-  const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
-  unsigned int hint, unsigned int ndescs, int nbytes)
+unsigned int libxsmm_gemm_diffn_avx(const libxsmm_gemm_descriptor* reference,
+  const void* descs, unsigned int hint, unsigned int ndescs, int nbytes)
 {
 #if defined(LIBXSMM_GEMM_DIFF_AVX)
-  assert(/*is pot*/ndescs == (1u << LIBXSMM_LOG2(ndescs)));
+  assert(/*is pot*/ndescs == LIBXSMM_UP2POT(ndescs));
 # if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
   assert(32 == nbytes); /* padded descriptor array */
   {
@@ -386,7 +366,7 @@ unsigned int libxsmm_gemm_diffn_avx(
     if (0 != libxsmm_verbosity /* library code is expected to be mute */
      && 1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED))
     {
-      fprintf(stderr, "LIBXSMM: unable to enter AVX code path!\n");
+      fprintf(stderr, "LIBXSMM WARNING: unable to enter AVX code path!\n");
     }
   }
   return libxsmm_gemm_diffn_sw(reference, descs, hint, ndescs, nbytes);
@@ -395,12 +375,11 @@ unsigned int libxsmm_gemm_diffn_avx(
 
 
 LIBXSMM_GEMM_DIFF_API_DEFINITION LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX2)
-unsigned int libxsmm_gemm_diffn_avx2(
-  const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
-  unsigned int hint, unsigned int ndescs, int nbytes)
+unsigned int libxsmm_gemm_diffn_avx2(const libxsmm_gemm_descriptor* reference,
+  const void* descs, unsigned int hint, unsigned int ndescs, int nbytes)
 {
 #if defined(LIBXSMM_GEMM_DIFF_AVX2)
-  assert(/*is pot*/ndescs == (1u << LIBXSMM_LOG2(ndescs)));
+  assert(/*is pot*/ndescs == LIBXSMM_UP2POT(ndescs));
 # if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
   assert(32 == nbytes); /* padded descriptor array */
   {
@@ -441,7 +420,7 @@ unsigned int libxsmm_gemm_diffn_avx2(
     if (0 != libxsmm_verbosity /* library code is expected to be mute */
      && 1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED))
     {
-      fprintf(stderr, "LIBXSMM: unable to enter AVX2 code path!\n");
+      fprintf(stderr, "LIBXSMM WARNING: unable to enter AVX2 code path!\n");
     }
   }
   return libxsmm_gemm_diffn_avx(reference, descs, hint, ndescs, nbytes);
@@ -450,12 +429,11 @@ unsigned int libxsmm_gemm_diffn_avx2(
 
 
 LIBXSMM_GEMM_DIFF_API_DEFINITION LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX512)
-unsigned int libxsmm_gemm_diffn_avx512(
-  const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
-  unsigned int hint, unsigned int ndescs, int nbytes)
+unsigned int libxsmm_gemm_diffn_avx512(const libxsmm_gemm_descriptor* reference,
+  const void* descs, unsigned int hint, unsigned int ndescs, int nbytes)
 {
-#if defined(LIBXSMM_GEMM_DIFF_AVX512) && !defined(LIBXSMM_INTRINSICS_INCOMPLETE_AVX512)
-  assert(/*is pot*/ndescs == (1 << LIBXSMM_LOG2(ndescs)));
+#if defined(LIBXSMM_GEMM_DIFF_AVX512) && !defined(LIBXSMM_INTRINSICS_AVX512_NOREDUCTIONS)
+  assert(/*is pot*/ndescs == LIBXSMM_UP2POT(ndescs));
 # if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
   assert(32 == nbytes); /* padded descriptor array */
   {
@@ -498,7 +476,7 @@ unsigned int libxsmm_gemm_diffn_avx512(
     if (0 != libxsmm_verbosity /* library code is expected to be mute */
      && 1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED))
     {
-      fprintf(stderr, "LIBXSMM: unable to enter AVX-512 code path!\n");
+      fprintf(stderr, "LIBXSMM WARNING: unable to enter AVX-512 code path!\n");
     }
   }
   return libxsmm_gemm_diffn_avx2(reference, descs, hint, ndescs, nbytes);
@@ -506,11 +484,11 @@ unsigned int libxsmm_gemm_diffn_avx512(
 }
 
 
-LIBXSMM_GEMM_DIFF_API_DEFINITION unsigned int libxsmm_gemm_diffn_imci(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
-  unsigned int hint, unsigned int ndescs, int nbytes)
+LIBXSMM_GEMM_DIFF_API_DEFINITION unsigned int libxsmm_gemm_diffn_imci(const libxsmm_gemm_descriptor* reference,
+  const void* descs, unsigned int hint, unsigned int ndescs, int nbytes)
 {
 #if defined(LIBXSMM_GEMM_DIFF_KNC) && (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
-  assert(/*is pot*/ndescs == (1 << LIBXSMM_LOG2(ndescs)));
+  assert(/*is pot*/ndescs == LIBXSMM_UP2POT(ndescs));
   assert(32 == nbytes); /* padded descriptor array */
   {
     const unsigned int hint_even = (hint & 0xFFFFFFFE), end = hint_even + ndescs;
@@ -536,7 +514,7 @@ LIBXSMM_GEMM_DIFF_API_DEFINITION unsigned int libxsmm_gemm_diffn_imci(const libx
     return ndescs;
   }
 #elif defined(LIBXSMM_GEMM_DIFF_KNC) && (16 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
-  assert(/*is pot*/ndescs == (1 << LIBXSMM_LOG2(ndescs)));
+  assert(/*is pot*/ndescs == LIBXSMM_UP2POT(ndescs));
   assert(16 == nbytes); /* padded descriptor array */
   { /* TODO: implement for 16 Byte descriptor */
     return libxsmm_gemm_diffn_sw(reference, descs, hint, ndescs, nbytes);
