@@ -33,7 +33,6 @@
 #include <string.h>
 #include <stdio.h>
 
-
 LIBXSMM_INLINE void print_help(void) {
   printf("\nUsage:\n");
   printf("    inlineasm/plainasm\n");
@@ -54,7 +53,7 @@ LIBXSMM_INLINE void print_help(void) {
   printf("    stride_h\n");
   printf("    stride_w\n");
   printf("    precision (0=FP32,1=INT16)\n");
-  printf("    ARCH: knl, skx\n");
+  printf("    ARCH: knl, knm, skx\n");
   printf("\n\n");
 }
 
@@ -111,17 +110,18 @@ int main(int argc, char* argv []) {
   /* arch specific stuff */
   l_arch = argv[19];
 
-  /* some intial parameters checks */
+  /* some initial parameters checks */
   /* check for sparse / dense only */
   if ( (strcmp(l_type, "inlineasm") != 0) &&
-       (strcmp(l_type, "plainasm")  != 0)    ) {
+       (strcmp(l_type, "plainasm")  != 0) ) {
     print_help();
     return -1;
   }
 
   /* check value of arch flag */
   if ( (strcmp(l_arch, "knl") != 0)    &&
-       (strcmp(l_arch, "skx") != 0)       ) {
+       (strcmp(l_arch, "knm") != 0)    &&
+       (strcmp(l_arch, "skx") != 0) ) {
     print_help();
     return -1;
   }
