@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2017, Intel Corporation                                **
+** Copyright (c) 2015-2018, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -29,8 +29,7 @@
 /* Alexander Heinecke (Intel Corp.)
 ******************************************************************************/
 
-#include <libxsmm_generator.h>
-#include <libxsmm_macros.h>
+#include <libxsmm.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
@@ -52,7 +51,7 @@ LIBXSMM_INLINE void print_help(void) {
   printf("    beta: 0 or 1\n");
   printf("    0: unaligned A, otherwise aligned (ignored for sparse)\n");
   printf("    0: unaligned C, otherwise aligned (ignored for sparse)\n");
-  printf("    ARCH: noarch, wsm, snb, hsw, knc, knl\n");
+  printf("    ARCH: noarch, wsm, snb, hsw, knc, knl, knm, skx\n");
   printf("    PREFETCH: nopf (none), pfsigonly, other dense options fall-back to pfsigonly\n");
   printf("    PRECISION: SP, DP\n");
   printf("    matrix input (CSC mtx file)\n");
@@ -71,21 +70,26 @@ LIBXSMM_INLINE void print_help(void) {
   printf("    beta: 0 or 1\n");
   printf("    0: unaligned A, otherwise aligned\n");
   printf("    0: unaligned C, otherwise aligned\n");
-  printf("    ARCH: noarch, wsm, snb, hsw, knc, knl\n");
-  printf("    PREFETCH: nopf (none), pfsigonly, BL2viaC, AL2, curAL2, AL2jpst, AL2_BL2viaC, curAL2_BL2viaC, AL2jpst_BL2viaC, AL2_BL2viaC_CL2\n");
-  printf("    PRECISION: SP, DP\n");
+  printf("    ARCH: noarch, wsm, snb, hsw, knc, knl, knm, skx\n");
+  printf("    PREFETCH: nopf (none), pfsigonly, BL2viaC, AL2, curAL2,\n"
+         "              AL2jpst, AL2_BL2viaC, curAL2_BL2viaC,\n"
+         "              AL2jpst_BL2viaC, AL1, BL1, CL1,\n"
+         "              AL1_BL1, BL1_CL1, AL1_CL1, AL1_BL1_CL1\n");
+  printf("    PRECISION: I16, SP, DP\n");
   printf("\n\n\n\n");
 }
 
 int main(int argc, char* argv []) {
-  libxsmm_gemm_descriptor l_xgemm_desc;
+  const libxsmm_gemm_descriptor* l_xgemm_desc = 0;
+  int l_flags = LIBXSMM_GEMM_FLAGS('N', 'N');
+  libxsmm_gemm_prefetch_type l_prefetch;
+  libxsmm_descriptor_blob l_xgemm_blob;
   char* l_type;
   char* l_file_out;
   char* l_matrix_file_in;
   char* l_routine_name;
   char* l_arch;
   char* l_precision;
-  int l_prefetch;
   int l_m = 0;
   int l_n = 0;
   int l_k = 0;
@@ -102,7 +106,7 @@ int main(int argc, char* argv []) {
   /* check argument count for a valid range */
   if (argc != 17 && argc != 18) {
     print_help();
-    return -1;
+    return EXIT_FAILURE;
   }
 
   /* names of files and routines */
@@ -130,20 +134,23 @@ int main(int argc, char* argv []) {
   l_aligned_a = atoi(argv[12]);
   l_aligned_c = atoi(argv[13]);
 
+  l_flags |= (0 != l_aligned_a ? LIBXSMM_GEMM_FLAG_ALIGN_A : 0);
+  l_flags |= (0 != l_aligned_c ? LIBXSMM_GEMM_FLAG_ALIGN_C : 0);
+
   /* arch specific stuff */
   l_arch = argv[14];
   l_precision = argv[16];
 
-  /* some intial parameters checks */
+  /* some initial parameters checks */
   /* check for sparse / dense only */
   if ( (strcmp(l_type, "sparse")         != 0) &&
        (strcmp(l_type, "sparse_csr")     != 0) &&
        (strcmp(l_type, "sparse_csr_reg") != 0) &&
        (strcmp(l_type, "sparse_csr_soa") != 0) &&
        (strcmp(l_type, "dense")          != 0) &&
-       (strcmp(l_type, "dense_asm")      != 0)    ) {
+       (strcmp(l_type, "dense_asm")      != 0) ) {
     print_help();
-    return -1;
+    return EXIT_FAILURE;
   }
 
   /* check for the right number of arguments depending on type */
@@ -152,45 +159,63 @@ int main(int argc, char* argv []) {
        ( (strcmp(l_type, "sparse_csr_reg") == 0) && (argc != 18) ) ||
        ( (strcmp(l_type, "sparse_csr_soa") == 0) && (argc != 18) ) ||
        ( (strcmp(l_type, "dense")  == 0) && (argc != 17) )         ||
-       ( (strcmp(l_type, "dense_asm")  == 0) && (argc != 17) )        ) {
+       ( (strcmp(l_type, "dense_asm")  == 0) && (argc != 17) ) ) {
     print_help();
-    return -1;
+    return EXIT_FAILURE;
   }
 
   /* set value of prefetch flag */
   if (strcmp("nopf", argv[15]) == 0) {
-    l_prefetch = LIBXSMM_PREFETCH_NONE;
+    l_prefetch = LIBXSMM_GEMM_PREFETCH_NONE;
   }
   else if (strcmp("pfsigonly", argv[15]) == 0) {
-    l_prefetch = LIBXSMM_PREFETCH_SIGONLY;
+    l_prefetch = LIBXSMM_GEMM_PREFETCH_SIGONLY;
   }
   else if (strcmp("BL2viaC", argv[15]) == 0) {
-    l_prefetch = LIBXSMM_PREFETCH_BL2_VIA_C;
+    l_prefetch = LIBXSMM_GEMM_PREFETCH_BL2_VIA_C;
   }
   else if (strcmp("curAL2", argv[15]) == 0) {
-    l_prefetch = LIBXSMM_PREFETCH_AL2_AHEAD;
+    l_prefetch = LIBXSMM_GEMM_PREFETCH_AL2_AHEAD;
   }
   else if (strcmp("curAL2_BL2viaC", argv[15]) == 0) {
-    l_prefetch = LIBXSMM_PREFETCH_AL2BL2_VIA_C_AHEAD;
+    l_prefetch = LIBXSMM_GEMM_PREFETCH_AL2BL2_VIA_C_AHEAD;
   }
   else if (strcmp("AL2", argv[15]) == 0) {
-    l_prefetch = LIBXSMM_PREFETCH_AL2;
+    l_prefetch = LIBXSMM_GEMM_PREFETCH_AL2;
   }
   else if (strcmp("AL2_BL2viaC", argv[15]) == 0) {
-    l_prefetch = LIBXSMM_PREFETCH_AL2BL2_VIA_C;
+    l_prefetch = LIBXSMM_GEMM_PREFETCH_AL2BL2_VIA_C;
   }
   else if (strcmp("AL2jpst", argv[15]) == 0) {
-    l_prefetch = LIBXSMM_PREFETCH_AL2_JPST;
+    l_prefetch = LIBXSMM_GEMM_PREFETCH_AL2_JPST;
   }
   else if (strcmp("AL2jpst_BL2viaC", argv[15]) == 0) {
-    l_prefetch = LIBXSMM_PREFETCH_AL2BL2_VIA_C_JPST;
+    l_prefetch = LIBXSMM_GEMM_PREFETCH_AL2BL2_VIA_C_JPST;
   }
-  else if (strcmp("AL2_BL2viaC_CL2", argv[15]) == 0) {
-    l_prefetch = LIBXSMM_PREFETCH_AL2CL2BL2_VIA_C;
+  else if (strcmp("AL1", argv[15]) == 0) {
+    l_prefetch = LIBXSMM_GEMM_PREFETCH_AL1;
+  }
+  else if (strcmp("BL1", argv[15]) == 0) {
+    l_prefetch = LIBXSMM_GEMM_PREFETCH_BL1;
+  }
+  else if (strcmp("CL1", argv[15]) == 0) {
+    l_prefetch = LIBXSMM_GEMM_PREFETCH_CL1;
+  }
+  else if (strcmp("AL1_BL1", argv[15]) == 0) {
+    l_prefetch = LIBXSMM_GEMM_PREFETCH_AL1_BL1;
+  }
+  else if (strcmp("BL1_CL1", argv[15]) == 0) {
+    l_prefetch = LIBXSMM_GEMM_PREFETCH_BL1_CL1;
+  }
+  else if (strcmp("AL1_CL1", argv[15]) == 0) {
+    l_prefetch = LIBXSMM_GEMM_PREFETCH_AL1_CL1;
+  }
+  else if (strcmp("AL1_BL1_CL1", argv[15]) == 0) {
+    l_prefetch = LIBXSMM_GEMM_PREFETCH_AL1_BL1_CL1;
   }
   else {
     print_help();
-    return -1;
+    return EXIT_FAILURE;
   }
 
   /* check value of arch flag */
@@ -199,60 +224,80 @@ int main(int argc, char* argv []) {
        (strcmp(l_arch, "hsw") != 0)    &&
        (strcmp(l_arch, "knc") != 0)    &&
        (strcmp(l_arch, "knl") != 0)    &&
+       (strcmp(l_arch, "knm") != 0)    &&
        (strcmp(l_arch, "skx") != 0)    &&
-       (strcmp(l_arch, "noarch") != 0)    ) {
+       (strcmp(l_arch, "noarch") != 0) ) {
     print_help();
-    return -1;
+    return EXIT_FAILURE;
   }
 
-  /* check and evaluate precison flag */
+  /* check and evaluate precision flag */
   if ( strcmp(l_precision, "SP") == 0 ) {
     l_single_precision = 1;
   } else if ( strcmp(l_precision, "DP") == 0 ) {
     l_single_precision = 0;
+  } else if ( strcmp(l_precision, "I16") == 0 ) {
+    l_single_precision = 2;
   } else {
     print_help();
-    return -1;
+    return EXIT_FAILURE;
   }
 
   /* check alpha */
   if ((l_alpha < -1 || -1 < l_alpha) && (l_alpha < 1 || 1 < l_alpha)) {
     print_help();
-    return -1;
+    return EXIT_FAILURE;
   }
 
   /* check beta */
   if ((l_beta < 0 || 0 < l_beta) && (l_beta < 1 || 1 < l_beta)) {
     print_help();
-    return -1;
+    return EXIT_FAILURE;
   }
 
-  LIBXSMM_GEMM_DESCRIPTOR(l_xgemm_desc, 1,
-    (0 == l_single_precision ? 0 : LIBXSMM_GEMM_FLAG_F32PREC)
-      | (0 != l_aligned_a ? LIBXSMM_GEMM_FLAG_ALIGN_A : 0)
-      | (0 != l_aligned_c ? LIBXSMM_GEMM_FLAG_ALIGN_C : 0),
-    l_m, l_n, l_k, l_lda, l_ldb, l_ldc,
-    l_alpha, l_beta, l_prefetch);
+  switch (l_single_precision) {
+    case 0: {
+      l_xgemm_desc = libxsmm_gemm_descriptor_dinit(&l_xgemm_blob, LIBXSMM_GEMM_PRECISION_F64,
+        l_m, l_n, l_k, l_lda, l_ldb, l_ldc, l_alpha, l_beta, l_flags, l_prefetch);
+    } break;
+    case 1: {
+      l_xgemm_desc = libxsmm_gemm_descriptor_dinit(&l_xgemm_blob, LIBXSMM_GEMM_PRECISION_F32,
+        l_m, l_n, l_k, l_lda, l_ldb, l_ldc, l_alpha, l_beta, l_flags, l_prefetch);
+    } break;
+    case 2: {
+      l_xgemm_desc = libxsmm_gemm_descriptor_dinit(&l_xgemm_blob, LIBXSMM_GEMM_PRECISION_I16,
+        l_m, l_n, l_k, l_lda, l_ldb, l_ldc, l_alpha, l_beta, l_flags, l_prefetch);
+    } break;
+    default: {
+      print_help();
+      return EXIT_FAILURE;
+    }
+  }
 
   if ( strcmp(l_type, "sparse") == 0 || strcmp(l_type, "sparse_csr") == 0 ||
        strcmp(l_type, "sparse_csr_reg") == 0 || strcmp(l_type, "sparse_csr_soa") == 0 ) {
-    /* read additional paramter for CSC/CSR description */
+    /* read additional parameter for CSC/CSR description */
     l_matrix_file_in = argv[17];
 
     /* some more restrictive checks are needed in case of sparse */
     if ( (l_alpha < 1) || (1 < l_alpha) ) {
       print_help();
-      return -1;
+      return EXIT_FAILURE;
     }
 
     if (l_lda < 1 && l_ldb < 1) {
       print_help();
-      return -1;
+      return EXIT_FAILURE;
     }
 
     if (l_ldc < 1) {
       print_help();
-      return -1;
+      return EXIT_FAILURE;
+    }
+
+    if ( l_single_precision > 1 ) {
+      print_help();
+      return EXIT_FAILURE;
     }
 
     if ( strcmp(l_type, "sparse_csr") == 0 ) {
@@ -265,23 +310,23 @@ int main(int argc, char* argv []) {
       l_is_csr = 3;
     }
 
-    libxsmm_generator_spgemm( l_file_out, l_routine_name, &l_xgemm_desc, l_arch, l_matrix_file_in, l_is_csr );
+    libxsmm_generator_spgemm( l_file_out, l_routine_name, l_xgemm_desc, l_arch, l_matrix_file_in, l_is_csr );
   }
 
   if ( (strcmp(l_type, "dense")     == 0) ||
-       (strcmp(l_type, "dense_asm") == 0)    ) {
+       (strcmp(l_type, "dense_asm") == 0) ) {
     if (l_lda < 1 || l_ldb < 1 || l_ldc < 1) {
       print_help();
-      return -1;
+      return EXIT_FAILURE;
     }
 
     if ( strcmp(l_type, "dense")  == 0 ) {
-      libxsmm_generator_gemm_inlineasm( l_file_out, l_routine_name, &l_xgemm_desc, l_arch );
+      libxsmm_generator_gemm_inlineasm( l_file_out, l_routine_name, l_xgemm_desc, l_arch );
     } else {
-      libxsmm_generator_gemm_directasm( l_file_out, l_routine_name, &l_xgemm_desc, l_arch );
+      libxsmm_generator_gemm_directasm( l_file_out, l_routine_name, l_xgemm_desc, l_arch );
     }
   }
 
-  return 0;
+  return EXIT_SUCCESS;
 }
 
