@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2017, Intel Corporation                                **
+** Copyright (c) 2015-2018, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -33,49 +33,41 @@
 
 #include <libxsmm.h>
 
-#if !defined(LIBXSMM_HASH_SW)
-/*# define LIBXSMM_HASH_SW*/
+#if !defined(LIBXSMM_HASH_SW) && 0
+# define LIBXSMM_HASH_SW
 #endif
 
 #if defined(LIBXSMM_BUILD) && !defined(LIBXSMM_HASH_NOINLINE)
-# define LIBXSMM_HASH_API LIBXSMM_INLINE LIBXSMM_RETARGETABLE
+# define LIBXSMM_HASH_API LIBXSMM_API_INLINE
 # define LIBXSMM_HASH_API_DEFINITION LIBXSMM_HASH_API LIBXSMM_ATTRIBUTE_UNUSED
 #else
 # define LIBXSMM_HASH_API LIBXSMM_API
-# define LIBXSMM_HASH_API_DEFINITION LIBXSMM_API_DEFINITION
+# define LIBXSMM_HASH_API_DEFINITION LIBXSMM_API
 #endif
 
 
 /** Function type representing the CRC32 functionality. */
-typedef LIBXSMM_RETARGETABLE unsigned int (*libxsmm_hash_function)(const void*, unsigned int, unsigned int);
+LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE unsigned int (*libxsmm_hash_function)(
+  const void*, size_t, unsigned int);
 
 /** Initialize hash function module; not thread-safe. */
 LIBXSMM_HASH_API void libxsmm_hash_init(int target_arch);
 LIBXSMM_HASH_API void libxsmm_hash_finalize(void);
 
+LIBXSMM_HASH_API unsigned int libxsmm_crc32_u32(unsigned int seed, unsigned int value);
+LIBXSMM_HASH_API unsigned int libxsmm_crc32_u64(unsigned int seed, unsigned long long value);
+
 /** Dispatched implementation which may (or may not) use a SIMD extension. */
 LIBXSMM_HASH_API unsigned int libxsmm_crc32(
-  const void* data, unsigned int size, unsigned int seed);
+  const void* data, size_t size, unsigned int seed);
 
 /** Calculate the CRC32 for a given quantity (size) of raw data according to the seed. */
 LIBXSMM_HASH_API unsigned int libxsmm_crc32_sw(
-  const void* data, unsigned int size, unsigned int seed);
+  const void* data, size_t size, unsigned int seed);
 
 /** Similar to libxsmm_crc32_sw (uses CRC32 instructions available since SSE4.2). */
 LIBXSMM_HASH_API unsigned int libxsmm_crc32_sse4(
-  const void* data, unsigned int size, unsigned int seed);
-
-/** Calculate a hash value for a given quantity (size) of raw data according to the seed. */
-LIBXSMM_HASH_API unsigned int libxsmm_hash(
-  const void* data, unsigned int size,
-  /** Upper bound of the result. */
-  unsigned int n);
-
-/** Calculate a hash value for a given quantity (size) of raw data according to the seed. */
-LIBXSMM_HASH_API unsigned int libxsmm_hash_npot(
-  const void* data, unsigned int size,
-  /** Upper bound of the result. */
-  unsigned int npot);
+  const void* data, size_t size, unsigned int seed);
 
 
 #if defined(LIBXSMM_BUILD) && !defined(LIBXSMM_HASH_NOINLINE)
