@@ -34,8 +34,9 @@
 # pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
 #endif
 #include "perf_jitdump.h"
-#include <stdio.h>
+#include <inttypes.h>
 #include <stdlib.h>
+#include <stdio.h>
 #if defined(LIBXSMM_PERF_JITDUMP) && !defined(_WIN32)
 # include <sys/mman.h>
 # include <string.h>
@@ -46,6 +47,9 @@
 # include <errno.h>
 # include <time.h>
 #endif
+#if defined(__linux__)
+# include <syscall.h>
+#endif
 #if defined(_WIN32)
 # include <windows.h>
 # define LIBXSMM_MAX_PATH MAX_PATH
@@ -71,14 +75,14 @@
 #endif
 
 
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE FILE * fp;
+LIBXSMM_APIVAR(FILE* internal_perf_fp);
 #if defined(LIBXSMM_PERF_JITDUMP) && !defined(_WIN32)
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void* marker_addr;
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE int code_index /*= 0*/;
+LIBXSMM_APIVAR(void* internal_perf_marker);
+LIBXSMM_APIVAR(int internal_perf_codeidx);
 #endif
 
 
-LIBXSMM_API_DEFINITION void libxsmm_perf_init(void)
+LIBXSMM_API_INTERN void libxsmm_perf_init(void)
 {
   const uint32_t pid = (uint32_t)libxsmm_get_pid();
   char file_name[LIBXSMM_MAX_PATH];
@@ -102,24 +106,24 @@ LIBXSMM_API_DEFINITION void libxsmm_perf_init(void)
   JITDUMP_CODE_CLOSE = 3;
 
   path_base = getenv("JITDUMPDIR");
-  if(path_base == NULL) {
+  if (path_base == NULL) {
     path_base = getenv("HOME");
   }
-  if(path_base == NULL) {
+  if (path_base == NULL) {
     path_base = getenv(".");
   }
 
   LIBXSMM_SNPRINTF(file_path, sizeof(file_path), "%s/.debug/", path_base);
   res = mkdir(file_path, S_IRWXU);
-  if(res < 0 && errno != EEXIST) {
-    LIBXSMM_PERF_ERROR("LIBXSMM: failed to create .debug dir\n");
+  if (res < 0 && errno != EEXIST) {
+    LIBXSMM_PERF_ERROR("LIBXSMM ERROR: failed to create .debug dir\n");
     goto error;
   }
 
   LIBXSMM_SNPRINTF(file_path, sizeof(file_path), "%s/.debug/jit", path_base);
   res = mkdir(file_path, S_IRWXU);
-  if(res < 0 && errno != EEXIST) {
-    LIBXSMM_PERF_ERROR("LIBXSMM: failed to create .debug/jit dir\n");
+  if (res < 0 && errno != EEXIST) {
+    LIBXSMM_PERF_ERROR("LIBXSMM ERROR: failed to create .debug/jit dir\n");
     goto error;
   }
 
@@ -128,8 +132,8 @@ LIBXSMM_API_DEFINITION void libxsmm_perf_init(void)
   LIBXSMM_SNPRINTF(file_path, sizeof(file_path),
            "%s/.debug/jit/libxsmm-jit-%s.XXXXXX", path_base, date);
   path_base = mkdtemp(file_path);
-  if(path_base == NULL) {
-    LIBXSMM_PERF_ERROR("LIBXSMM: failed to create temporary dir\n");
+  if (path_base == NULL) {
+    LIBXSMM_PERF_ERROR("LIBXSMM ERROR: failed to create temporary dir\n");
     goto error;
   }
 
@@ -137,27 +141,27 @@ LIBXSMM_API_DEFINITION void libxsmm_perf_init(void)
 
   fd = open(file_name, O_CREAT|O_TRUNC|O_RDWR, 0600);
   if (fd < 0) {
-    LIBXSMM_PERF_ERROR("LIBXSMM: failed to open file\n");
+    LIBXSMM_PERF_ERROR("LIBXSMM ERROR: failed to open file\n");
     goto error;
   }
 
   page_size = sysconf(_SC_PAGESIZE);
   if (page_size < 0) {
-    LIBXSMM_PERF_ERROR("LIBXSMM: failed to get page size\n");
+    LIBXSMM_PERF_ERROR("LIBXSMM ERROR: failed to get page size\n");
     goto error;
   }
-  marker_addr = mmap(NULL, page_size, PROT_READ|PROT_EXEC, MAP_PRIVATE, fd, 0);
-  if (marker_addr == MAP_FAILED) {
-    LIBXSMM_PERF_ERROR("LIBXSMM: mmap failed.\n");
+  internal_perf_marker = mmap(NULL, page_size, PROT_READ|PROT_EXEC, MAP_PRIVATE, fd, 0);
+  if (internal_perf_marker == MAP_FAILED) {
+    LIBXSMM_PERF_ERROR("LIBXSMM ERROR: mmap failed.\n");
     goto error;
   }
 
   /* initialize code index */
-  code_index = 0;
+  internal_perf_codeidx = 0;
 
-  fp = fdopen(fd, "wb+");
-  if (fp == NULL) {
-    LIBXSMM_PERF_ERROR("LIBXSMM: fdopen failed.\n");
+  internal_perf_fp = fdopen(fd, "wb+");
+  if (internal_perf_fp == NULL) {
+    LIBXSMM_PERF_ERROR("LIBXSMM ERROR: fdopen failed.\n");
     goto error;
   }
 
@@ -167,20 +171,20 @@ LIBXSMM_API_DEFINITION void libxsmm_perf_init(void)
   header.elf_mach   = 62;  /* EM_X86_64 */
   header.total_size = sizeof(header);
   header.pid        = pid;
-  header.timestamp  = libxsmm_timer_xtick();
+  header.timestamp  = libxsmm_timer_tick_rdtsc();
   header.flags      = JITDUMP_FLAGS_ARCH_TIMESTAMP;
 
-  res = fwrite(&header, sizeof(header), 1, fp);
+  res = fwrite(&header, sizeof(header), 1, internal_perf_fp);
   if (res != 1) {
-    LIBXSMM_PERF_ERROR("LIBXSMM: failed to write header.\n");
+    LIBXSMM_PERF_ERROR("LIBXSMM ERROR: failed to write header.\n");
     goto error;
   }
 
 #else
   LIBXSMM_SNPRINTF(file_name, sizeof(file_name), "/tmp/perf-%u.map", pid);
-  fp = fopen(file_name, "w+");
-  if (fp == NULL) {
-    LIBXSMM_PERF_ERROR("LIBXSMM: failed to open map file\n");
+  internal_perf_fp = fopen(file_name, "w+");
+  if (internal_perf_fp == NULL) {
+    LIBXSMM_PERF_ERROR("LIBXSMM ERROR: failed to open map file\n");
     goto error;
   }
 #endif
@@ -188,59 +192,70 @@ LIBXSMM_API_DEFINITION void libxsmm_perf_init(void)
   return;
 
 error:
-  if (fp != NULL) {
-    fclose(fp);
-    fp = NULL;
+  if (internal_perf_fp != NULL) {
+    fclose(internal_perf_fp);
+    internal_perf_fp = NULL;
   }
   assert(0);
 }
 
 
-LIBXSMM_API_DEFINITION void libxsmm_perf_finalize(void)
+LIBXSMM_API_INTERN void libxsmm_perf_finalize(void)
 {
 #if defined(LIBXSMM_PERF_JITDUMP) && !defined(_WIN32)
   int res, page_size;
   struct jitdump_record_header hdr;
 
-  if (fp == NULL) {
-    LIBXSMM_PERF_ERROR("LIBXSMM: jit dump file not opened\n");
+  if (internal_perf_fp == NULL) {
+    LIBXSMM_PERF_ERROR("LIBXSMM ERROR: jit dump file not opened\n");
     goto error;
   }
 
   memset(&hdr, 0, sizeof(hdr));
   hdr.id = JITDUMP_CODE_CLOSE;
   hdr.total_size = sizeof(hdr);
-  hdr.timestamp = libxsmm_timer_xtick();
-  res = fwrite(&hdr, sizeof(hdr), 1, fp);
+  hdr.timestamp = libxsmm_timer_tick_rdtsc();
+  res = fwrite(&hdr, sizeof(hdr), 1, internal_perf_fp);
 
   if (res != 1) {
-    LIBXSMM_PERF_ERROR("LIBXSMM: failed to write JIT_CODE_CLOSE record\n");
+    LIBXSMM_PERF_ERROR("LIBXSMM ERROR: failed to write JIT_CODE_CLOSE record\n");
     goto error;
   }
 
   page_size = sysconf(_SC_PAGESIZE);
   if (page_size < 0) {
-    LIBXSMM_PERF_ERROR("LIBXSMM: failed to get page_size\n");
+    LIBXSMM_PERF_ERROR("LIBXSMM ERROR: failed to get page_size\n");
     goto error;
   }
-  munmap(marker_addr, page_size);
-  fclose(fp);
+  munmap(internal_perf_marker, page_size);
+  fclose(internal_perf_fp);
   return;
 
 error:
   assert(0);
 #else
-  fclose(fp);
+  fclose(internal_perf_fp);
 #endif
 }
 
 
-LIBXSMM_API_DEFINITION void libxsmm_perf_dump_code(const void* memory, size_t size, const char* name)
+/** Utility function to receive the OS-specific thread ID. */
+LIBXSMM_API_INLINE unsigned int internal_perf_get_tid(void)
 {
-  assert(fp != NULL);
+#if defined(__linux__)
+  return (unsigned int)syscall(__NR_gettid);
+#else /* fallback */
+  return libxsmm_get_tid();
+#endif
+}
+
+
+LIBXSMM_API_INTERN void libxsmm_perf_dump_code(const void* memory, size_t size, const char* name)
+{
+  assert(internal_perf_fp != NULL);
   assert(name && *name);
   assert(memory != NULL && size != 0);
-  if (fp != NULL) {
+  if (internal_perf_fp != NULL) {
 #if defined(LIBXSMM_PERF_JITDUMP) && !defined(_WIN32)
     int res;
     struct jitdump_record_header hdr;
@@ -252,39 +267,34 @@ LIBXSMM_API_DEFINITION void libxsmm_perf_dump_code(const void* memory, size_t si
 
     hdr.id = JITDUMP_CODE_LOAD;
     hdr.total_size = sizeof(hdr) + sizeof(rec) + name_len + size;
-    hdr.timestamp = libxsmm_timer_xtick();
+    hdr.timestamp = libxsmm_timer_tick_rdtsc();
 
     rec.code_size = size;
     rec.vma = (uintptr_t) memory;
     rec.code_addr = (uintptr_t) memory;
     rec.pid = (uint32_t) libxsmm_get_pid();
-    rec.tid = (uint32_t) libxsmm_get_tid();
+    rec.tid = (uint32_t) internal_perf_get_tid();
 
-#if !defined(LIBXSMM_NO_SYNC)
-    flockfile(fp);
-#endif
+    LIBXSMM_FLOCK(internal_perf_fp);
 
     /* This will be unique as we hold the file lock. */
-    rec.code_index = code_index++;
+    rec.internal_perf_codeidx = internal_perf_codeidx++;
 
     /* Count number of written items to check for errors. */
     res = 0;
-    res += fwrite_unlocked(&hdr, sizeof(hdr), 1, fp);
-    res += fwrite_unlocked(&rec, sizeof(rec), 1, fp);
-    res += fwrite_unlocked(name, name_len, 1, fp);
-    res += fwrite_unlocked((const void*) memory, size, 1, fp);
+    res += fwrite_unlocked(&hdr, sizeof(hdr), 1, internal_perf_fp);
+    res += fwrite_unlocked(&rec, sizeof(rec), 1, internal_perf_fp);
+    res += fwrite_unlocked(name, name_len, 1, internal_perf_fp);
+    res += fwrite_unlocked((const void*) memory, size, 1, internal_perf_fp);
 
-#if !defined(LIBXSMM_NO_SYNC)
-    funlockfile(fp);
-#endif
-
-    fflush(fp);
+    LIBXSMM_FUNLOCK(internal_perf_fp);
+    fflush(internal_perf_fp);
 
     assert(res == 4); /* Expected 4 items written above */
 
 #else
-    fprintf(fp, "%llx %lx %s\n", (unsigned long long)((uintptr_t)memory), (unsigned long)size, name);
-    fflush(fp);
+    fprintf(internal_perf_fp, "%" PRIxPTR " %lx %s\n", (uintptr_t)memory, (unsigned long)size, name);
+    fflush(internal_perf_fp);
 #endif
   }
 }
