@@ -34,9 +34,9 @@
 #include <libxsmm.h>
 
 
-LIBXSMM_API void libxsmm_perf_init(void);
-LIBXSMM_API void libxsmm_perf_finalize(void);
-LIBXSMM_API void libxsmm_perf_dump_code(
+LIBXSMM_API_INTERN void libxsmm_perf_init(void);
+LIBXSMM_API_INTERN void libxsmm_perf_finalize(void);
+LIBXSMM_API_INTERN void libxsmm_perf_dump_code(
   const void* memory, size_t size,
   const char* name);
 
