@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2016-2017, Intel Corporation                                **
+** Copyright (c) 2016-2018, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -31,20 +31,21 @@
 #include "libxsmm_trace.h"
 #include <libxsmm_sync.h>
 
-#if !defined(LIBXSMM_TRACE_DLINFO)
-/*# define LIBXSMM_TRACE_DLINFO*/
+#if !defined(LIBXSMM_TRACE_DLINFO) && 0
+# define LIBXSMM_TRACE_DLINFO
 #endif
 
 #if defined(LIBXSMM_OFFLOAD_TARGET)
 # pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
 #endif
+#include <inttypes.h>
 #include <assert.h>
 #include <stdlib.h>
 # if (!defined(_BSD_SOURCE) || 0 == _BSD_SOURCE) && (!defined(_SVID_SOURCE) || 0 == _SVID_SOURCE) && \
      (!defined(_XOPEN_SOURCE) || !defined(_XOPEN_SOURCE_EXTENDED) || 500 > _XOPEN_SOURCE) && \
      (!defined(_POSIX_C_SOURCE) || 200112L > _POSIX_C_SOURCE)
 /* C89: avoid warning about mkstemp declared implicitly */
-int mkstemp(char* filename_template);
+LIBXSMM_EXTERN int mkstemps(char*, int);
 # endif
 #include <string.h>
 #include <stdio.h>
@@ -72,7 +73,7 @@ int mkstemp(char* filename_template);
 # endif
 # if defined(__APPLE__) && defined(__MACH__)
 /* taken from "libtransmission" fdlimit.c */
-LIBXSMM_INLINE LIBXSMM_RETARGETABLE int posix_fallocate(int fd, off_t offset, off_t length)
+LIBXSMM_API_INLINE int posix_fallocate(int fd, off_t offset, off_t length)
 {
   fstore_t fst;
   fst.fst_flags = F_ALLOCATECONTIG;
@@ -85,9 +86,12 @@ LIBXSMM_INLINE LIBXSMM_RETARGETABLE int posix_fallocate(int fd, off_t offset, of
 # elif (!defined(_XOPEN_SOURCE) || 600 > _XOPEN_SOURCE) && \
        (!defined(_POSIX_C_SOURCE) || 200112L > _POSIX_C_SOURCE)
 /* C89: avoid warning about posix_fallocate declared implicitly */
-int posix_fallocate(int, off_t, off_t);
+LIBXSMM_EXTERN int posix_fallocate(int, off_t, off_t);
 # endif
 #endif
+#if !defined(_WIN32)
+# include <pthread.h>
+#endif
 #if defined(LIBXSMM_OFFLOAD_TARGET)
 # pragma offload_attribute(pop)
 #endif
@@ -103,25 +107,26 @@ int posix_fallocate(int, off_t, off_t);
 
 #if defined(_WIN32) || defined(__CYGWIN__)
 # define LIBXSMM_TRACE_MINDEPTH 5
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE volatile LONG internal_trace_initialized /*= -1*/;
+LIBXSMM_APIVAR(volatile LONG internal_trace_initialized);
 #else
 # define LIBXSMM_TRACE_MINDEPTH 4
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE int internal_trace_initialized /*= -1*/;
+LIBXSMM_APIVAR(volatile int internal_trace_initialized);
 #if !defined(LIBXSMM_NO_SYNC)
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE pthread_key_t internal_trace_key /*= 0*/;
+LIBXSMM_APIVAR(pthread_key_t internal_trace_key);
 #endif
-LIBXSMM_API void internal_delete(void* value);
-LIBXSMM_API_DEFINITION void internal_delete(void* value)
+LIBXSMM_API_INLINE void internal_delete(void* value)
 {
   int fd;
+#if !(defined(__APPLE__) && defined(__MACH__))
   assert(value);
+#endif
   fd = *((int*)value);
 #if defined(NDEBUG)
   munmap(value, LIBXSMM_TRACE_SYMBOLSIZE);
 #else /* library code is expected to be mute */
   if (0 != munmap(value, LIBXSMM_TRACE_SYMBOLSIZE)) {
     const int error = errno;
-    fprintf(stderr, "LIBXSMM: %s (munmap error #%i at %p)\n",
+    fprintf(stderr, "LIBXSMM ERROR: %s (munmap error #%i at %p)\n",
       strerror(error), error, value);
   }
 #endif
@@ -130,16 +135,16 @@ LIBXSMM_API_DEFINITION void internal_delete(void* value)
   }
 #if !defined(NDEBUG) /* library code is expected to be mute */
   else {
-    fprintf(stderr, "LIBXSMM: invalid file descriptor (%i)\n", fd);
+    fprintf(stderr, "LIBXSMM ERROR: invalid file descriptor (%i)\n", fd);
   }
 #endif
 }
 #endif /*!defined(_WIN32) && !defined(__CYGWIN__)*/
 
 
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE int internal_trace_mindepth /*=  0*/;
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE int internal_trace_threadid /*= -1*/;
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE int internal_trace_maxnsyms /*= -1*/;
+LIBXSMM_APIVAR(int internal_trace_mindepth);
+LIBXSMM_APIVAR(int internal_trace_threadid);
+LIBXSMM_APIVAR(int internal_trace_maxnsyms);
 
 
 LIBXSMM_API
@@ -148,24 +153,22 @@ LIBXSMM_ATTRIBUTE(no_instrument_function)
 #endif
 int libxsmm_trace_init(int filter_threadid, int filter_mindepth, int filter_maxnsyms);
 
-LIBXSMM_API_DEFINITION int libxsmm_trace_init(int filter_threadid, int filter_mindepth, int filter_maxnsyms)
+LIBXSMM_API int libxsmm_trace_init(int filter_threadid, int filter_mindepth, int filter_maxnsyms)
 {
   int result = EXIT_SUCCESS;
   internal_trace_initialized = -1; /* disabled */
-#if defined(__TRACE)
-  if (0 != filter_maxnsyms) { /* enabled */
+#if defined(LIBXSMM_TRACE)
 # if defined(_WIN32) || defined(__CYGWIN__)
-    SymSetOptions(SYMOPT_DEFERRED_LOADS | SYMOPT_UNDNAME);
-    result = (FALSE != SymInitialize(GetCurrentProcess(), NULL, TRUE) ? EXIT_SUCCESS : GetLastError());
+  SymSetOptions(SYMOPT_DEFERRED_LOADS | SYMOPT_UNDNAME);
+  result = (FALSE != SymInitialize(GetCurrentProcess(), NULL, TRUE) ? EXIT_SUCCESS : GetLastError());
 # elif !defined(LIBXSMM_NO_SYNC)
-    result = pthread_key_create(&internal_trace_key, internal_delete);
+  result = pthread_key_create(&internal_trace_key, internal_delete);
 # endif
-    if (EXIT_SUCCESS == result) {
-      internal_trace_threadid = filter_threadid;
-      internal_trace_maxnsyms = filter_maxnsyms;
-      internal_trace_mindepth = filter_mindepth;
-      internal_trace_initialized = 0; /* enabled */
-    }
+  if (EXIT_SUCCESS == result) {
+    internal_trace_threadid = filter_threadid;
+    internal_trace_maxnsyms = filter_maxnsyms;
+    internal_trace_mindepth = filter_mindepth;
+    internal_trace_initialized = 0; /* enabled */
   }
 #else
   LIBXSMM_UNUSED(filter_threadid);
@@ -182,36 +185,55 @@ LIBXSMM_ATTRIBUTE(no_instrument_function)
 #endif
 int libxsmm_trace_finalize(void);
 
-LIBXSMM_API_DEFINITION int libxsmm_trace_finalize(void)
+LIBXSMM_API int libxsmm_trace_finalize(void)
 {
   int result;
-#if !defined(__TRACE)
-  result = EXIT_FAILURE;
-#else
-  const int initialized = LIBXSMM_ATOMIC_LOAD(&internal_trace_initialized, LIBXSMM_ATOMIC_RELAXED);
-  if (0 == initialized) {
-    LIBXSMM_ATOMIC_STORE(&internal_trace_initialized, -1/*disable*/, LIBXSMM_ATOMIC_SEQ_CST);
+#if defined(LIBXSMM_TRACE)
+  result = EXIT_SUCCESS;
+  if (0 <= internal_trace_initialized) {
+    internal_trace_initialized = -1; /* disable */
 # if defined(_WIN32) || defined(__CYGWIN__)
     result = (FALSE != SymCleanup(GetCurrentProcess()) ? EXIT_SUCCESS : GetLastError());
 # elif !defined(LIBXSMM_NO_SYNC)
     result = pthread_key_delete(internal_trace_key);
 # endif
   }
-  else {
-    result = EXIT_SUCCESS;
-  }
+#else
+  result = EXIT_FAILURE;
 #endif
   return result;
 }
 
 
+LIBXSMM_API
+#if defined(__GNUC__)
+LIBXSMM_ATTRIBUTE(no_instrument_function)
+#endif
+unsigned int libxsmm_backtrace(void* buffer[], unsigned int size);
+
+LIBXSMM_API
+#if defined(_WIN32)
+/*TODO: no inline*/
+#elif defined(__GNUC__)
+/*LIBXSMM_ATTRIBUTE(noinline)*/
+#endif
+unsigned int libxsmm_backtrace(void* buffer[], unsigned int size)
+{
+#if defined(_WIN32) || defined(__CYGWIN__)
+  return CaptureStackBackTrace(0, size, buffer, NULL);
+#else
+  return (unsigned int)backtrace(buffer, (int)size);
+#endif
+}
+
+
 LIBXSMM_API
 #if defined(__GNUC__)
 LIBXSMM_ATTRIBUTE(no_instrument_function)
 #endif
 const char* libxsmm_trace_info(unsigned int* depth, unsigned int* threadid, const int* filter_threadid, const int* filter_mindepth, const int* filter_maxnsyms);
 
-LIBXSMM_API_DEFINITION
+LIBXSMM_API
 #if defined(_WIN32)
 /*TODO: no inline*/
 #elif defined(__GNUC__)
@@ -220,10 +242,10 @@ LIBXSMM_API_DEFINITION
 const char* libxsmm_trace_info(unsigned int* depth, unsigned int* threadid, const int* filter_threadid, const int* filter_mindepth, const int* filter_maxnsyms)
 {
   const char *fname = NULL;
-#if defined(__TRACE)
-  const int max_n = depth ? (LIBXSMM_TRACE_MAXDEPTH) : 2;
-  const int min_n = depth ? (LIBXSMM_TRACE_MINDEPTH + *depth) : 2;
-  void *stack[LIBXSMM_TRACE_MAXDEPTH], **symbol = stack + LIBXSMM_MIN(depth ? ((int)(*depth + 1)) : 1, max_n - 1);
+#if defined(LIBXSMM_TRACE)
+  const int max_n = (0 != depth ? (LIBXSMM_TRACE_MAXDEPTH) : 2);
+  const int min_n = (0 != depth ? (LIBXSMM_TRACE_MINDEPTH + *depth) : 2);
+  void *stacktrace[LIBXSMM_TRACE_MAXDEPTH], **symbol = stacktrace + LIBXSMM_MIN(0 != depth ? ((int)(*depth + 1)) : 1, max_n - 1);
   static LIBXSMM_TLS int cerberus = 0;
   int i;
 
@@ -237,19 +259,15 @@ const char* libxsmm_trace_info(unsigned int* depth, unsigned int* threadid, cons
 # endif
     i = LIBXSMM_ATOMIC_LOAD(&internal_trace_initialized, LIBXSMM_ATOMIC_RELAXED);
     if (0 <= i) { /* do nothing if not yet initialized */
-      const int mindepth = filter_mindepth ? *filter_mindepth : internal_trace_mindepth;
-      const int maxnsyms = filter_maxnsyms ? *filter_maxnsyms : internal_trace_maxnsyms;
-# if defined(_WIN32) || defined(__CYGWIN__)
-      i = CaptureStackBackTrace(0, max_n, stack, NULL);
-# else
-      i = backtrace(stack, max_n);
-# endif
+      const int mindepth = (0 != filter_mindepth ? *filter_mindepth : internal_trace_mindepth);
+      const int maxnsyms = (0 != filter_maxnsyms ? *filter_maxnsyms : internal_trace_maxnsyms);
+      i = libxsmm_backtrace(stacktrace, max_n);
       /* filter depth against filter_mindepth and filter_maxnsyms */
       if ((0 >= mindepth ||      (min_n + mindepth) <= i) &&
           (0 >  maxnsyms || i <= (min_n + mindepth + maxnsyms - 1)))
       {
         if (min_n <= i) { /* check against min. depth */
-          const int filter = (filter_threadid ? *filter_threadid : internal_trace_threadid);
+          const int filter = (0 != filter_threadid ? *filter_threadid : internal_trace_threadid);
           int abs_tid = 0;
 # if defined(_WIN32) || defined(__CYGWIN__)
           static LIBXSMM_TLS char buffer[sizeof(SYMBOL_INFO)+LIBXSMM_TRACE_SYMBOLSIZE];
@@ -264,7 +282,7 @@ const char* libxsmm_trace_info(unsigned int* depth, unsigned int* threadid, cons
           }
           else {
             abs_tid = LIBXSMM_ATOMIC_ADD_FETCH(&internal_trace_initialized, 1, LIBXSMM_ATOMIC_RELAXED);
-            /* use sign bit to flag enabled fallback for symbol resolution */
+            /* use sign bit to flag enabled fall-back for symbol resolution */
             tid = -abs_tid;
           }
 
@@ -273,12 +291,16 @@ const char* libxsmm_trace_info(unsigned int* depth, unsigned int* threadid, cons
             if (FALSE != SymFromAddr(GetCurrentProcess(), (DWORD64)*symbol, NULL, value)
               && 0 < value->NameLen)
             {
-              /* disable fallback allowing unresolved symbol names */
+              /* disable fall-back allowing unresolved symbol names */
               tid = abs_tid; /* make unsigned */
               fname = value->Name;
             }
-            else if (0 > tid) { /* fallback allowing unresolved symbol names */
-              sprintf(buffer, "0x%llx", (unsigned long long)*symbol);
+            else if (0 > tid) { /* fall-back allowing unresolved symbol names */
+#   if defined(__MINGW32__)
+              sprintf(buffer, "%p", *symbol);
+#   else
+              sprintf(buffer, "0x%" PRIxPTR, (uintptr_t)*symbol);
+#   endif
               fname = buffer;
             }
             if (depth) *depth = i - min_n;
@@ -287,7 +309,7 @@ const char* libxsmm_trace_info(unsigned int* depth, unsigned int* threadid, cons
 # else
 #   if defined(LIBXSMM_NO_SYNC)
           static char raw_c;
-          char *const raw_value = &raw_c;
+          char */*const*/ raw_value = &raw_c; /* const: avoid warning (below / constant control-flow) */
 #   else
           char *const raw_value = (char*)pthread_getspecific(internal_trace_key);
 #   endif
@@ -305,14 +327,14 @@ const char* libxsmm_trace_info(unsigned int* depth, unsigned int* threadid, cons
               }
 #   if !defined(NDEBUG) /* library code is expected to be mute */
               else {
-                fprintf(stderr, "LIBXSMM: failed to get buffer\n");
+                fprintf(stderr, "LIBXSMM ERROR: failed to get buffer\n");
               }
 #   endif
             }
           }
           else {
-            char filename[] = "/tmp/fileXXXXXX";
-            fd = mkstemp(filename);
+            char filename[] = "/tmp/.libxsmm_XXXXXX.map";
+            fd = mkstemps(filename, 4/*.map*/);
 
             if (0 <= fd && 0 == posix_fallocate(fd, 0, LIBXSMM_TRACE_SYMBOLSIZE)) {
               char *const buffer = (char*)mmap(NULL, LIBXSMM_TRACE_SYMBOLSIZE,
@@ -333,7 +355,7 @@ const char* libxsmm_trace_info(unsigned int* depth, unsigned int* threadid, cons
                 {
                   abs_tid = LIBXSMM_ATOMIC_ADD_FETCH(&internal_trace_initialized, 1, LIBXSMM_ATOMIC_RELAXED);
                   assert(0 < abs_tid);
-                  /* use sign bit to flag enabled fallback for symbol resolution */
+                  /* use sign bit to flag enabled fall-back for symbol resolution */
                   ivalue[1] = -abs_tid;
 
                   if (0 > filter || filter == abs_tid - 1) {
@@ -342,7 +364,7 @@ const char* libxsmm_trace_info(unsigned int* depth, unsigned int* threadid, cons
                 }
                 else {
 #   if !defined(NDEBUG) /* library code is expected to be mute */
-                  fprintf(stderr, "LIBXSMM: failed to setup buffer\n");
+                  fprintf(stderr, "LIBXSMM ERROR: failed to setup buffer\n");
 #   endif
                   internal_delete(buffer);
                 }
@@ -350,14 +372,14 @@ const char* libxsmm_trace_info(unsigned int* depth, unsigned int* threadid, cons
 #   if !defined(NDEBUG)
               else {
                 const int error = errno;
-                fprintf(stderr, "LIBXSMM: %s (mmap allocation error #%i)\n",
+                fprintf(stderr, "LIBXSMM ERROR: %s (mmap allocation error #%i)\n",
                   strerror(error), error);
               }
 #   endif
             }
 #   if !defined(NDEBUG) /* library code is expected to be mute */
             else {
-              fprintf(stderr, "LIBXSMM: failed to setup file descriptor (%i)\n", fd);
+              fprintf(stderr, "LIBXSMM ERROR: failed to setup file descriptor (%i)\n", fd);
             }
 #   endif
           }
@@ -370,14 +392,14 @@ const char* libxsmm_trace_info(unsigned int* depth, unsigned int* threadid, cons
               char* c;
               for (c = value; '+' != *c && *c; ++c);
               if ('+' == *c) {
-                /* disable fallback allowing unresolved symbol names */
+                /* disable fall-back allowing unresolved symbol names */
                 ivalue[1] = abs_tid; /* make unsigned */
                 fname = value;
                 *c = 0;
               }
             }
 
-            /* fallback to symbol address */
+            /* fall-back to symbol address */
             if (0 > ivalue[1] && 0 == fname) {
               sprintf(value, "0x%llx", (unsigned long long)*symbol);
               fname = value;
@@ -398,7 +420,7 @@ const char* libxsmm_trace_info(unsigned int* depth, unsigned int* threadid, cons
   LIBXSMM_UNUSED(filter_threadid);
   LIBXSMM_UNUSED(filter_mindepth);
   LIBXSMM_UNUSED(filter_maxnsyms);
-#endif /*defined(__TRACE)*/
+#endif
 
   return fname;
 }
@@ -410,15 +432,15 @@ LIBXSMM_ATTRIBUTE(no_instrument_function)
 #endif
 void libxsmm_trace(FILE* stream, unsigned int depth, const int* filter_threadid, const int* filter_mindepth, const int* filter_maxnsyms);
 
-LIBXSMM_API_DEFINITION void libxsmm_trace(FILE* stream, unsigned int depth, const int* filter_threadid, const int* filter_mindepth, const int* filter_maxnsyms)
+LIBXSMM_API void libxsmm_trace(FILE* stream, unsigned int depth, const int* filter_threadid, const int* filter_mindepth, const int* filter_maxnsyms)
 {
-#if defined(__TRACE)
+#if defined(LIBXSMM_TRACE)
   unsigned int depth1 = depth + 1, threadid;
   const char *const name = libxsmm_trace_info(&depth1, &threadid,
     filter_threadid, filter_mindepth, filter_maxnsyms);
 
   if (name && *name) { /* implies actual other results to be valid */
-    const int depth0 = LIBXSMM_MAX(filter_mindepth ? *filter_mindepth : internal_trace_mindepth, 0);
+    const int depth0 = LIBXSMM_MAX(0 != filter_mindepth ? *filter_mindepth : internal_trace_mindepth, 0);
     assert(0 != stream/*otherwise fprintf handle the error*/);
     if ((0 == filter_threadid && 0 > internal_trace_threadid) || (0 != filter_threadid && 0 > *filter_threadid)) {
       fprintf(stream, "%*s%s@%u\n", (int)(depth1 - depth0), "", name, threadid);
@@ -436,53 +458,38 @@ LIBXSMM_API_DEFINITION void libxsmm_trace(FILE* stream, unsigned int depth, cons
 }
 
 
-#if defined(__GNUC__)
+#if defined(__TRACE) && defined(__GNUC__) && defined(LIBXSMM_BUILD)
 
-#if !defined(LIBXSMM_BUILD) && defined(__APPLE__) && defined(__MACH__)
-LIBXSMM_PRAGMA_OPTIMIZE_OFF
-#endif
-
-LIBXSMM_EXTERN_C
-#if defined(__cplusplus) || defined(LIBXSMM_BUILD)
-LIBXSMM_API
-#else
-static
-#endif
-LIBXSMM_ATTRIBUTE(no_instrument_function) void __cyg_profile_func_enter(void* this_fn, void* call_site);
-
-LIBXSMM_EXTERN_C
-#if defined(__cplusplus) || defined(LIBXSMM_BUILD)
-LIBXSMM_API_DEFINITION
-#else
-static
-#endif
-void __cyg_profile_func_enter(void* this_fn, void* call_site)
+LIBXSMM_API LIBXSMM_ATTRIBUTE(no_instrument_function) void __cyg_profile_func_enter(void* this_fn, void* call_site);
+LIBXSMM_API void __cyg_profile_func_enter(void* this_fn, void* call_site)
 {
-#if defined(__TRACE)
+#if defined(LIBXSMM_TRACE)
 # if !defined(LIBXSMM_TRACE_DLINFO)
   LIBXSMM_UNUSED(this_fn); LIBXSMM_UNUSED(call_site); /* suppress warning */
-  libxsmm_trace(stderr, 1/*no need for parent (0) but parent of parent (1)*/,
+  libxsmm_trace(stderr, 2/*no need for parent (0) but parent of parent (1)*/,
     /* inherit global settings from libxsmm_trace_init */
     NULL, NULL, NULL);
 # else
-# if 1
-  Dl_info info;
-# else
-  struct {
-      const char *dli_fname;
-      void       *dli_fbase;  /* Address at which shared object
-                                 is loaded */
-      const char *dli_sname;  /* Name of nearest symbol with address
-                                 lower than addr */
-      void       *dli_saddr;
-  } info;
-# endif
-  if (0 != dladdr(this_fn, (Dl_info*)&info)) {
-    if (0 != info.dli_sname) {
-      fprintf(stderr, "%s\n", info.dli_sname);
-    }
-    else if (0 != info.dli_saddr) {
-      fprintf(stderr, "0x%llx\n", (unsigned long long)info.dli_saddr);
+  if (0 <= internal_trace_initialized && 0 != internal_trace_maxnsyms) {
+#   if 1
+    Dl_info info;
+#   else
+    struct {
+      const char* dli_fname;
+      /* address at which shared object is loaded */
+      void* dli_fbase;
+      /* name of nearest symbol with address lower than address */
+      const char* dli_sname;
+      void* dli_saddr;
+    } info;
+#   endif
+    if (0 != dladdr(this_fn, (Dl_info*)&info)) {
+      if (0 != info.dli_sname) {
+        fprintf(stderr, "%s\n", info.dli_sname);
+      }
+      else if (0 != info.dli_saddr) {
+        fprintf(stderr, "0x%llx\n", (unsigned long long)info.dli_saddr);
+      }
     }
   }
 # endif
@@ -492,28 +499,11 @@ void __cyg_profile_func_enter(void* this_fn, void* call_site)
 }
 
 
-LIBXSMM_EXTERN_C
-#if defined(__cplusplus) || defined(LIBXSMM_BUILD)
-LIBXSMM_API
-#else
-static
-#endif
-LIBXSMM_ATTRIBUTE(no_instrument_function) void __cyg_profile_func_exit(void* this_fn, void* call_site);
-
-LIBXSMM_EXTERN_C
-#if defined(__cplusplus) || defined(LIBXSMM_BUILD)
-LIBXSMM_API_DEFINITION
-#else
-static
-#endif
-void __cyg_profile_func_exit(void* this_fn, void* call_site)
+LIBXSMM_API LIBXSMM_ATTRIBUTE(no_instrument_function) void __cyg_profile_func_exit(void* this_fn, void* call_site);
+LIBXSMM_API void __cyg_profile_func_exit(void* this_fn, void* call_site)
 {
   LIBXSMM_UNUSED(this_fn); LIBXSMM_UNUSED(call_site); /* suppress warning */
 }
 
-#if !defined(LIBXSMM_BUILD) && defined(__APPLE__) && defined(__MACH__)
-LIBXSMM_PRAGMA_OPTIMIZE_ON
-#endif
-
-#endif /*defined(__GNUC__)*/
+#endif /*defined(__TRACE) && defined(__GNUC__) && defined(LIBXSMM_BUILD)*/
 
