@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2016-2017, Intel Corporation                                **
+** Copyright (c) 2016-2018, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -40,6 +40,10 @@
 # pragma offload_attribute(pop)
 #endif
 
+#if (defined(__TRACE) || defined(LIBXSMM_BUILD) || !defined(_WIN32))
+# define LIBXSMM_TRACE
+#endif
+
 
 /** Initializes the trace facility; NOT thread-safe. */
 LIBXSMM_API int libxsmm_trace_init(
@@ -53,6 +57,9 @@ LIBXSMM_API int libxsmm_trace_init(
 /** Finalizes the trace facility; NOT thread-safe. */
 LIBXSMM_API int libxsmm_trace_finalize(void);
 
+/** Receives the backtrace of up to 'size' addresses. Returns the actual number of addresses (n <= size). */
+LIBXSMM_API unsigned int libxsmm_backtrace(void* buffer[], unsigned int size);
+
 /** Returns the name of the function where libxsmm_trace is called from; thread-safe. */
 LIBXSMM_API const char* libxsmm_trace_info(
   /* Specify relative pos. in stack (NULL/0: default); output: abs. location in trace. */
@@ -76,3 +83,4 @@ LIBXSMM_API void libxsmm_trace(FILE* stream, unsigned int depth,
   const int* filter_maxnsyms);
 
 #endif /*LIBXSMM_TRACE_H*/
+
