@@ -1,3 +1,4 @@
+
 /******************************************************************************
 ** Copyright (c) 2016-2017, Intel Corporation                                **
 ** All rights reserved.                                                      **
@@ -26,7 +27,7 @@
 ** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
 ** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
 ******************************************************************************/
-/* Alexander Heinecke (Intel Corp.), Hans Pabst (Intel Corp.)
+/* Alexander Heinecke, Hans Pabst (Intel Corp.)
 ******************************************************************************/
 
 int imgofm1, img, ofm1, ifm1, oj, ij, oi, ii, kj, ki, ifm2, ofm2;
@@ -59,11 +60,35 @@ if (handle->datatype != handle->datatype_itm) {
   LIBXSMM_VLA_DECL(5, element_input_type, output_lp, out_lp, handle->blocksofm, handle->ofhp, handle->ofwp, handle->ofmblock);
   LIBXSMM_VLA_DECL(5, const element_input_type, input, (element_input_type*)handle->reg_input->data, handle->blocksifm, handle->ifhp, handle->ifwp, handle->ifmblock);
   LIBXSMM_VLA_DECL(6, const element_filter_type, weight, (element_filter_type*)handle->reg_filter->data, handle->blocksifm, handle->desc.R, handle->desc.S, handle->ifmblock, handle->ofmblock);
+#if defined(INPUT_PADDING)
+  /* Variables and initializations related to padding */
+  const int padded_h = handle->ifhp + 2 * handle->desc.pad_h;
+  const int padded_w = handle->ifwp + 2 * handle->desc.pad_w;
+  LIBXSMM_VLA_DECL(3, element_input_type, input_buffer, ((element_input_type*)handle->scratch5) + ltid * padded_h * padded_w * handle->ifmblock, padded_w, handle->ifmblock);
+  /* Reset input padding buffer to zero (in case it is not set to zero due to fwd/bwd computations) */
+  memset(&LIBXSMM_VLA_ACCESS(3, input_buffer, 0, 0, 0, padded_w, handle->ifmblock), 0, padded_w * padded_h * handle->ifmblock * sizeof(element_input_type));
+#endif
+
 
   /* perform convolution */
   for (imgofm1 = thr_begin; imgofm1 < thr_end; ++imgofm1) {
     img = imgofm1 / handle->blocksofm;
     ofm1 = imgofm1 % handle->blocksofm;
+    /* handle fused bias addition */
+    if ( ((handle->fuse_ops & LIBXSMM_DNN_CONV_FUSE_BIAS) > 0) ) {
+      LIBXSMM_VLA_DECL(2, element_output_type, bias, (element_output_type*)handle->reg_bias->data, handle->ofmblock);
+      element_output_type* temp_ptr   = &(LIBXSMM_VLA_ACCESS(  5, output, img, ofm1, 0, 0, 0, handle->blocksofm, handle->ofhp, handle->ofwp, handle->ofmblock));
+      element_output_type* temp_ptr_2 = &(LIBXSMM_VLA_ACCESS(  2, bias, ofm1, 0, handle->ofmblock));
+
+      /* @TODO check these loops for physical output padding */
+      for (oj = 0; oj < handle->ofhp*handle->ofwp; ++oj) {
+        LIBXSMM_PRAGMA_SIMD
+        for (ofm2 = 0; ofm2 < handle->ofmblock; ++ofm2) {
+          temp_ptr[ofm2] = temp_ptr_2[ofm2];
+        }
+        temp_ptr += handle->ofmblock;
+      }
+    }
     /* up-convert */
     if (handle->datatype != handle->datatype_itm) {
       for (oj = 0; oj < handle->ofh; ++oj) {
@@ -76,6 +101,25 @@ if (handle->datatype != handle->datatype_itm) {
       }
     }
     for (ifm1 = 0; ifm1 < handle->blocksifm; ++ifm1) {
+#if defined(INPUT_PADDING)
+      for (oj = 0; oj < handle->ifhp; ++oj) {
+        for (oi = 0; oi < handle->ifwp; ++oi) {
+          for (ifm2 = 0; ifm2 < handle->ifmblock; ++ifm2) {
+            LIBXSMM_VLA_ACCESS(3, input_buffer, oj + handle->desc.pad_h, oi + handle->desc.pad_w, ifm2, padded_w, handle->ifmblock) =
+              LIBXSMM_VLA_ACCESS(5,  input, img, ifm1, oj, oi, ifm2, handle->blocksifm, handle->ifhp, handle->ifwp, handle->ifmblock);
+          }
+        }
+      }
+#endif
+      /* reset result buffer to zero when intent is to overwrite when first block
+         of input channels should be convoluted */
+      if ( (ifm1 == 0) && ((handle->options & LIBXSMM_DNN_CONV_OPTION_OVERWRITE) > 0) && ((handle->fuse_ops & LIBXSMM_DNN_CONV_FUSE_BIAS) == 0) ) {
+        element_output_type* temp_ptr = &(LIBXSMM_VLA_ACCESS(  5, output, img, ofm1, 0, 0, 0, handle->blocksofm, handle->ofhp, handle->ofwp, handle->ofmblock));
+        LIBXSMM_PRAGMA_SIMD
+        for (oj = 0; oj < handle->ofhp*handle->ofwp*handle->ofmblock; oj++) {
+          temp_ptr[oj] = (element_output_type)0;
+        }
+      }
       for (oj = 0; oj < handle->ofh; ++oj) {
         ij = oj * handle->desc.u;
         for (oi = 0; oi < handle->ofw; ++oi) {
@@ -84,8 +128,13 @@ if (handle->datatype != handle->datatype_itm) {
             for (ki = 0; ki< handle->desc.S; ++ki) {
               for (ifm2 = 0; ifm2 < handle->ifmblock; ++ifm2) {
                 for (ofm2 = 0; ofm2 < handle->ofmblock; ++ofm2) {
+
                   LIBXSMM_VLA_ACCESS(  5, output, img, ofm1, oj, oi, ofm2, handle->blocksofm, handle->ofhp, handle->ofwp, handle->ofmblock) += (element_output_type)(
+#if defined(INPUT_PADDING)
+                    LIBXSMM_VLA_ACCESS(3, input_buffer, ij + kj, ii + ki, ifm2, padded_w, handle->ifmblock)
+#else
                     LIBXSMM_VLA_ACCESS(5,  input, img, ifm1, ij + kj, ii + ki, ifm2, handle->blocksifm, handle->ifhp, handle->ifwp, handle->ifmblock)
+#endif
                   * LIBXSMM_VLA_ACCESS(6, weight, ofm1, ifm1, kj, ki, ifm2, ofm2, handle->blocksifm, handle->desc.R, handle->desc.S, handle->ifmblock, handle->ofmblock));
                 }
               }
@@ -94,6 +143,19 @@ if (handle->datatype != handle->datatype_itm) {
         }
       }
     }
+    /* ReLU handling */
+    if ( ((handle->fuse_ops & LIBXSMM_DNN_CONV_FUSE_RELU) > 0) ) {
+      element_output_type* temp_ptr   = &(LIBXSMM_VLA_ACCESS(  5, output, img, ofm1, 0, 0, 0, handle->blocksofm, handle->ofhp, handle->ofwp, handle->ofmblock));
+
+      /* @TODO check these loops for physical output padding */
+      for (oj = 0; oj < handle->ofhp*handle->ofwp; ++oj) {
+        LIBXSMM_PRAGMA_SIMD
+        for (ofm2 = 0; ofm2 < handle->ofmblock; ++ofm2) {
+          temp_ptr[ofm2] = (element_output_type)(temp_ptr[ofm2] < 0 ? 0 : temp_ptr[ofm2]);
+        }
+        temp_ptr += handle->ofmblock;
+      }
+    }
     /* down-convert */
     if (handle->datatype != handle->datatype_itm) {
       for (oj = 0; oj < handle->ofh; ++oj) {
