@@ -26,7 +26,7 @@
 ** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
 ** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
 ******************************************************************************/
-/* Rajkishore Barik (Intel Corp.), Alexander Heinecke (Intel Corp.)
+/* Rajkishore Barik, Alexander Heinecke (Intel Corp.)
 ******************************************************************************/
 
 int ofm1ifm1, img, ofm1, ifm1, oj, ij, oi, ii, kj, ki, ifm2, ofm2;
@@ -45,20 +45,53 @@ LIBXSMM_VLA_DECL(5, const element_output_type, output, out, handle->ofhp, handle
 LIBXSMM_VLA_DECL(5, const element_input_type, input, (element_input_type*)handle->reg_input->data, handle->ifhp, handle->ifwp, handle->blocksifm, handle->ifmblock);
 LIBXSMM_VLA_DECL(6, element_filter_type, weight, (element_filter_type*)handle->grad_filter->data, handle->blocksifm, handle->desc.R, handle->desc.S, handle->ifmblock, handle->ofmblock);
 
+#if defined(INPUT_PADDING)
+/* Variables and initializations related to padding */
+const int padded_h = handle->ifhp + 2 * handle->desc.pad_h;
+const int padded_w = handle->ifwp + 2 * handle->desc.pad_w;
+LIBXSMM_VLA_DECL(3, element_input_type, input_buffer, ((element_input_type*)handle->scratch5) + ltid * padded_h * padded_w * handle->ifmblock, padded_w, handle->ifmblock);
+/* Reset input padding buffer to zero (in case it is not set to zero due to fwd/bwd computations) */
+memset(&LIBXSMM_VLA_ACCESS(3, input_buffer, 0, 0, 0, padded_w, handle->ifmblock), 0, padded_w * padded_h * handle->ifmblock * sizeof(element_input_type));
+#endif
+
 for (ofm1ifm1 = thr_begin; ofm1ifm1 < thr_end; ++ofm1ifm1) {
   ofm1 = ofm1ifm1 / handle->blocksifm;
   ifm1 = ofm1ifm1 % handle->blocksifm;
-  for(img = 0; img < handle->desc.N; ++img) {
-    for(oj = 0; oj < handle->ofh; ++oj) {
+  /* reset result buffer to zero when intent is to overwrite */
+  if ( ((handle->options & LIBXSMM_DNN_CONV_OPTION_OVERWRITE) > 0) ) {
+    element_filter_type* temp_buf = &LIBXSMM_VLA_ACCESS(6, weight, ofm1, ifm1, 0, 0, 0, 0, handle->blocksifm, handle->desc.R, handle->desc.S, handle->ifmblock, handle->ofmblock);
+
+    LIBXSMM_PRAGMA_SIMD
+    for (ii = 0; ii < handle->desc.R*handle->desc.S*handle->ifmblock*handle->ofmblock; ++ii) {
+      temp_buf[ii] = (element_filter_type)0;
+    }
+  }
+  for (img = 0; img < handle->desc.N; ++img) {
+#if defined(INPUT_PADDING)
+    for (oj = 0; oj < handle->ifhp; ++oj) {
+      for (oi = 0; oi < handle->ifwp; ++oi) {
+        for (ifm2 = 0; ifm2 < handle->ifmblock; ++ifm2) {
+          LIBXSMM_VLA_ACCESS(3, input_buffer, oj + handle->desc.pad_h , oi + handle->desc.pad_w, ifm2, padded_w, handle->ifmblock) =
+            LIBXSMM_VLA_ACCESS(5, input, img, oj, oi, ifm1, ifm2, handle->ifhp, handle->ifwp, handle->blocksifm, handle->ifmblock);
+        }
+      }
+    }
+#endif
+
+    for (oj = 0; oj < handle->ofh; ++oj) {
       ij = oj * handle->desc.u;
-      for(oi = 0; oi < handle->ofw; ++oi) {
+      for (oi = 0; oi < handle->ofw; ++oi) {
         ii = oi * handle->desc.v;
-        for(kj = 0; kj < handle->desc.R; ++kj) {
-          for(ki = 0; ki < handle->desc.S; ++ki) {
+        for (kj = 0; kj < handle->desc.R; ++kj) {
+          for (ki = 0; ki < handle->desc.S; ++ki) {
             for (ifm2 = 0; ifm2 < handle->ifmblock; ++ifm2) {
               for (ofm2 = 0; ofm2 < handle->ofmblock; ++ofm2) {
                 LIBXSMM_VLA_ACCESS(6, weight, ofm1, ifm1, kj, ki, ifm2, ofm2, handle->blocksifm, handle->desc.R, handle->desc.S, handle->ifmblock, handle->ofmblock) += (element_filter_type)(
+#if defined(INPUT_PADDING)
+                    LIBXSMM_VLA_ACCESS(3,  input_buffer, ij + kj, ii + ki, ifm2, padded_w, handle->ifmblock)
+#else
                     LIBXSMM_VLA_ACCESS(5,  input, img, ij + kj, ii + ki, ifm1, ifm2, handle->ifhp, handle->ifwp, handle->blocksifm, handle->ifmblock)
+#endif
                   * LIBXSMM_VLA_ACCESS(5, output, img, oj, oi, ofm1, ofm2, handle->ofhp, handle->ofwp, handle->blocksofm, handle->ofmblock));
               }
             }
@@ -68,3 +101,4 @@ for (ofm1ifm1 = thr_begin; ofm1ifm1 < thr_end; ++ofm1ifm1) {
     }
   }
 }
+
